include cyber.inc

.code






service_rj45_Kestrel proc
;    call service_Kestrel_offline    
;    ret
	
	.if rj45base==0                      ; serial card not present
		call service_Kestrel_offline    
	.else
		.if robot[bx].online!=DEV_OFFLINE
			invoke set8032page, INTERNAL
			invoke poke_8032, scratch,27
			invoke peek_8032, scratch
			.if al==27
				invoke poke_8032, scratch,72
				invoke peek_8032, scratch
				.if al==72
					.if robot[bx].online !=DEV_ONLINE
						invoke set8032page, EPROM
						
						mov si,030h
						mov di,0
						.repeat
							invoke peek_8032,si
							mov robot[bx].prom.promid[di],al
							inc si
							inc di
						.until di==sizeof Prom_struc

						invoke set8032page, INTERNAL
						invoke deek_8032,07eh
						mov robot[bx].prom.promcsum,ax

						mov robot[bx].teachmode,INIT
					.endif
					
					.if robot[bx].prom.promid == 'K'
						call putonline
					.endif
				.else
					call putoffline ; increments robot[bx].offlinecnt
				.endif
			.else
				call putoffline     ; increments robot[bx].offlinecnt
			.endif
		.endif


		.if robot[bx].online==DEV_ONLINE
			.if robot[bx].offlinecnt==0             ; really available ?
				call service_Kestrel_online
			.endif
		.ELSE
			call service_Kestrel_offline    
		.endif
	.endif
	ret
service_rj45_Kestrel endp


service_Kestrel_online proc private
	call service_8032_workcell
	
	.if robot[bx].teachmode == zeroing
		invoke peek_8032, 027H      ; read in position flags
									; bit 7 = zeroing
									; bit 6 = jammed
									; bit 5 = axis 5 inpos
		bt ax,7
		.if !carry?
			mov robot[bx].teachmode,init
		.endif

	
	.else
		invoke peek_8032, 027H      ; read in position flags
									; bit 7 = zeroing
									; bit 6 = jammed
									; bit 5 = axis 5 inpos
		
	
	
		bt ax,7
		.if carry?
			mov robot[bx].teachmode,zeroing

		.else
			.if robot[bx].teachmode == jammed
				bt ax,6
				.if !carry?
					mov robot[bx].teachmode,init
				.endif
			.else
				bt ax,6
				.if carry?
					mov robot[bx].teachmode,jammed
					invoke peek_8032, 026H  ; read which axis is jammed
					mov ah,0
					mov robot[bx].errors,ax
					ret
				.endif
			.endif
		.endif
		
		
		and al,0fh
		.if al==0fh
			mov robot[bx].inpos,true
		.else
			mov robot[bx].inpos,false
		.endif


		invoke peek_8032, 024H      ; read pendant active flag
		and al,0100000b
		.if al!=0
			mov robot[bx].inpos,false
			.if robot[bx].teachmode < jammed
				mov robot[bx].teachmode,pendant
			.ENDIF
		.endif



		invoke deek_8032, 070H          ; X
		.if ax>1024
			sub ax,1024
		.else 
			mov ax,0
		.endif
		shr ax,1
		mov robot[bx].feedback[0],ax
	
		invoke deek_8032, 072H          ; Y
		.if ax>1024
			sub ax,1024
		.else 
			mov ax,0
		.endif
		shr ax,1
		mov robot[bx].feedback[2],ax

		invoke deek_8032, 074H          ; Z
		.if ax>1024
			sub ax,1024
		.else 
			mov ax,0
		.endif
		;shr ax,1
		mov robot[bx].feedback[4],ax

		invoke deek_8032, 076H          ; R
		.if ax>1024
			sub ax,1024
		.else 
			mov ax,0
		.endif
		;.if ax>1600 
		;    mov ax,1600
		;.endif
		mov robot[bx].feedback[6],ax

		
		invoke peek_8032, 07aH              ; current gripper state
		and ax,16
		.if ax==16
			mov robot[bx].feedback[8],1    ; Shut
		.else
			mov robot[bx].feedback[8],0    ; Open
		.endif
		
		invoke peek_8032, 07bH              ; current speed
		mov robot[bx].feedback[10],ax


		.if (robot[bx].teachmode==INIT ) || (robot[bx].teachmode==Pendant) || (robot[bx].teachmode==JAMMED)
			mov si,0
			.repeat
				mov ax,robot[bx].feedback[si]
				mov robot[bx].despos[si],ax
				mov robot[bx].lastdespos[si],ax
				add si,2
			.until si==12
			.if robot[bx].teachmode==INIT
				mov robot[bx].teachmode,keyboard
			.endif
		
		.elseif  robot[bx].teachmode==NOSELEAD
			mov si,0
			.repeat
				mov ax,robot[bx].feedback[si]
				mov robot[bx].despos[si],ax
				mov robot[bx].lastdespos[si],ax
				add si,2
			.until si==8   ; Gripper still set by despos

		.endif




		.if robot[bx].teachmode!=Pendant
			mov si,0
			mov di,060h                                 ; start address in Kestrel
			.repeat
				mov cx,robot[bx].despos[si]             ; desired position
				.if si < 4
					shl cx,1                            ; because BASIC uses signed integers
													; we send twice as many steps for major axes
				.endif
			
				.if robot[bx].resend==true
					mov robot[bx].lastdespos[si],-1
				.endif

				.if cx!=robot[bx].lastdespos[si]        ; already sent?
					mov robot[bx].lastdespos[si],cx     ; no, take copy
					add cx,1024                         ; shift, saves negative encoders
					invoke doke_8032,di,cx              ; write it out
				.endif
				add di,2
				add si,2
				.until si==8
				mov robot[bx].resend,false

			mov cx,robot[bx].despos[8]             ; desired gripper
			.if cx!=robot[bx].lastdespos[8]        ; already sent?
				mov robot[bx].lastdespos[8],cx     ; no, take copy
				.if cx==1
					invoke poke_8032,06ah,16        ; write it out
				.else
					invoke poke_8032,06ah,0         ; write it out
				.endif
			.endif

			mov cx,robot[bx].despos[10]             ; desired speed
			.if cx!=robot[bx].lastdespos[10]        ; already sent?
				mov robot[bx].lastdespos[10],cx     ; no, take copy
				invoke poke_8032,06bh,cx            ; write it out
			.endif
		.endif

		.if robot[bx].inpos==true
			mov si,0
			.repeat
				.if robot[bx].stop[si]!=false
					mov robot[bx].stop[si],false 
					mov ax,robot[bx].feedback[si]
					mov robot[bx].despos[si],ax
					mov robot[bx].lastdespos[si],ax
				.endif
				add si,2
			.until si==18
		.endif
	.endif

	ret
service_Kestrel_online endp



service_Kestrel_offline proc private

	.if robot[bx].teachmode == init
		mov robot[bx].teachmode,keyboard
		mov robot[bx].despos[0],0   ;X
		mov robot[bx].despos[2],0   ;Y
		mov robot[bx].despos[4],0   ;Z
		mov robot[bx].despos[6],0   ;R
		mov robot[bx].despos[8],0   ;G
		mov robot[bx].despos[10],3  ;Speed
		
		mov si,0
		.repeat
			mov ax,robot[bx].despos[si]
			mov robot[bx].lastdespos[si],ax
			mov robot[bx].feedback[si],ax
			add si,2
		.until si==18

		mov di,0
		.repeat
			mov robot[bx].prom.promid[di],'?'
			inc di
		.until di==sizeof Prom_struc

	.endif

	
	mov robot.inpos[bx],TRUE        
	
	simulate 0,100          ; simulate movement of axis 0, increment of 100
	simulate 1,100
	simulate 2,100
	simulate 3,10
	simulate 4,10
	simulate 5,1
	mov robot[bx].stop[6*2],false
	mov robot[bx].stop[7*2],false
	simulate 8,3

	
	
	mov robot.pulsecount[bx],0
	mov robot.rs232flag[bx],false
	mov robot.Serialflag[bx],false
	mov robot.mesgflag[bx],false

	ret
service_Kestrel_offline endp

				

end

