SUBTTL Constants     
TITLE **  Control Program for 8032 Kestrel - Version 1.0 12/06/95 **

PLEN 100

;load
    
LIST ; to allow 'L' switch at command line


NULL        equ 0     ;timer 0/1 function call - set r4/5


EPROM       equ 0ffh   ; IBM peek EPROM page
INTERNAL    equ 0feh   ; IBM peek internal RAM page
window      equ 99


AXIS0_SPD equ 00000000b     ; Axis 0/X is channel 0 on slave 2
AXIS2_SPD equ 00100000b     ; Axis 2/Z is channel 1 on slave 2

AXIS1_SPD equ 00000000b     ; Axis 1/Y is channel 0 on slave 3
AXIS3_SPD equ 00100000b     ; Axis 3/R is channel 1 on slave 3




maxdelay    EQU 0      ; Maximum seed value for timer2 

STALLPOLL   EQU 35  ;was 25 11/6/98 - 50 excessive


; HARDWARE ADDRESSES


lsblatch        equ     0a000h  ; latch for lsb address
intlatch        equ     0a001h  ; interrupt from IBM, Ext0
datalatch       equ     0a002h  ; 

workcell_i      equ     0c600h  ; inputs from workcell          - r
workcell_o      equ     0ca00h  ; outputs to workcell           - w

datastrobe      equ     0d200h  ; write strobe at RJ45 interface
dataoen         equ     0d600h  ; data out enable

slave0          equ     0dc00h  ; encoder slave xz 
slave1          equ     0dc40h  ; encoder slave yr
slave2          equ     0dc80h  ; motor slave xz 
slave3          equ     0dcc0h  ; motor slave yr

microsws        equ     0c200h  ; uswitches



; INTERNAL RAM MAP



rbank0          equ     00h      ; register bank 0 - Baseloop
rbank1          equ     08h      ; register bank 1 - IBM interrupt
rbank2          equ     010h     ; register bank 2 - 
rbank3          equ     018h     ; register bank 3  

lastjam    equ 20h
last_stalled0 bit lastjam.0 
last_stalled1 bit lastjam.1 
last_stalled2 bit lastjam.2 
last_stalled3 bit lastjam.3 
last_stalled4 bit lastjam.4 



wco_stat        equ     023h     ; states of workcell outputs - FIXED LOCATION
wco_stat0       bit     wco_stat.0
wco_stat1       bit     wco_stat.1
wco_stat2       bit     wco_stat.2
wco_stat3       bit     wco_stat.3
wco_stat4       bit     wco_stat.4
wco_stat5       bit     wco_stat.5
wco_stat6       bit     wco_stat.6
wco_stat7       bit     wco_stat.7

flags       equ 024h
pen0        bit flags.0
pen1        bit flags.1
pen2        bit flags.2
pen3        bit flags.3

pen_active  bit flags.5     ;FIXED location, IBM reads
enable_pen  bit flags.6


stallflags equ 025h
stalled0   bit stallflags.0
stalled1   bit stallflags.1
stalled2   bit stallflags.2
stalled3   bit stallflags.3
stalled4   bit stallflags.4

stallflags_ibm equ 026h ; copy for IBM to read
    

inpos       equ 027h          ;fixed location
axis0_inpos bit inpos.0
axis1_inpos bit inpos.1
axis2_inpos bit inpos.2
axis3_inpos bit inpos.3

jammed      bit inpos.6
zeroing     bit inpos.7


flags2 equ inpos+1

grip_press bit flags2.0
ibm_mode   bit flags2.1
enable_jam bit flags2.2
inner0     bit flags2.3
inner1     bit flags2.4
inner2     bit flags2.5
inner3     bit flags2.6
inner4     bit flags2.7



Servo_Flags     equ     flags2+1
enable_servo0   bit Servo_Flags.0
enable_servo1   bit Servo_Flags.1
enable_servo2   bit Servo_Flags.2
enable_servo3   bit Servo_Flags.3
dir0            bit Servo_Flags.4
dir1            bit Servo_Flags.5
dir2            bit Servo_Flags.6
dir3            bit Servo_Flags.7

Stop_Flags      equ Servo_Flags+1
Stop0           bit Stop_Flags.0
Stop1           bit Stop_Flags.1
Stop2           bit Stop_Flags.2
Stop3           bit Stop_Flags.3


timer2_func     equ     Stop_Flags+1
wco_low         equ     timer2_func+1
wco_high        equ     wco_low +1
wco_next        equ     wco_high  +1
pulse_width     equ     wco_next +1

control0        equ     pulse_width+1
control1        equ     control0+1
control2        equ     control1+1
control3        equ     control2+1

timer0func      equ     control3+1
change          equ     timer0func+1

led_cnt         equ     change+1
led_cnt_seed    equ     led_cnt+1

max0    equ led_cnt_seed+1
max1    equ max0+1
max2    equ max1+1
max3    equ max2+1

error0  equ max3+1
error1  equ error0+2
error2  equ error1+2
error3  equ error2+2

stall0  equ error3+2
stall1  equ stall0+1
stall2  equ stall1+1
stall3  equ stall2+1

lastencpos0 equ stall3+1
lastencpos1 equ lastencpos0+2
lastencpos2 equ lastencpos1+2
lastencpos3 equ lastencpos2+2



mcntrl          equ     050H            ; FIXED LOCATION
wcntrl          equ     mcntrl+1        ; workcell control
scntrl          equ     wcntrl+1
mcntrl_copy     equ     scntrl+1   
pen_copy0       equ     mcntrl_copy+1
pen_copy1       equ     pen_copy0+1
pen_speed       equ     pen_copy1+1

rs232_mess      equ     05bh            ; Byte to send as "RS232" via workcell
pulses_no       equ     rs232_mess+1    ; number of workcell pulses to be sent
pulses_rate     equ     pulses_no+2     ; rate to send workcell pulses


despos0         equ 060h
despos1         equ 062h
despos2         equ 064h
despos3         equ 066h
despos4         equ 068h
despos_grip     equ 06ah
despos_speed    equ 06bh
maxerror        equ 06ch
seqcnt          equ 06dh

encpos0         equ 070h
encpos1         equ 072h
encpos2         equ 074h
encpos3         equ 076h
encpos4         equ 078h
encpos_grip     equ 07ah
encpos_speed    equ 07bh

seqptr          equ 07ch

csum            equ 07eh      



stack           equ     090H


runcnt          equ 0f0h
stallcnt0       equ runcnt+2
stallcnt1       equ stallcnt0+1
stallcnt2       equ stallcnt1+1
stallcnt3       equ stallcnt2+1
stallcnt4       equ stallcnt3+1

scratch         equ 0ffh        ; used only by IBM    



; PROGRAM MEMORY MAP

PAGE 
SUBTTL Interrupt Vectors

;base           equ     08000h  ; alternatives for RAM and ROM versions
base           equ     0h

ORG     base
PAD     0ffh                    ; to fill unallocated locations


; SET-UP INTERRUPT VECTORS AT START OF PROGRAM SPACE


ORG     base+0h                 ; reset vector address
        jmp     begin           ; start of initialisation

ORG     base+03h                ; external interrupt 0 vector address
        jmp     ibmint          ; used for IBM communications

ORG     base+0bh                ; timer0 interrupt vector
        jmp timer0

ORG     base+13h                ; external interrupt 1 vector address
        reti

ORG     base+1bh                ; timer1 interrupt vector
        RETI

ORG     base+23h                ; serial port vector address
        reti

org     base+2bh                ; timer2 interrupt vector
        jmp     timer2
        ;reti

ORG     base+30h                ; to be tidy


; PROM IDENTIFICATION


        db      "KEST","1.00"     ; Kestrel program, version 1



        db      "04/08/95 ",169," Cybernetic Applications 1995",0    ; Null terminated







PAGE
     
SUBTTL INITIALISATION / RESET

;nb. uses register set 0, as does Baseloop

begin:
        mov  sp,#stack          ; initialise stack pointer 
        
        mov  dptr,#intlatch     ; clear possible IBM interrupt states
        movx a,@dptr

        mov dptr,#Slave0        ; Clear possible interrupt state on encoder xz slave
        movx a,@dptr
        
        mov dptr,#Slave1        ; Clear possible interrupt state on encoder yr slave
        movx a,@dptr
        
        mov dptr,#Slave2
        movx a,@dptr
        
        mov dptr,#Slave3
        movx a,@dptr




        
        clr a                   ; Clear all internal RAM
        mov r0,#255
clearram:
        mov @r0,a
        djnz r0,clearram
        
        
        mov  wco_stat,#0ffh     ; Set all workcell outputs high
        mov  a,wco_stat         ; send to workcell outputs
        mov  dptr,#workcell_o
        movx @dptr,a            

        
        
        mov dptr,#0         ; Calculate EPROM checksum
        mov r0,#0
        mov r1,#0
gc1:
        clr a
        movc a,@a+dptr

        add a,r0
        mov r0,a

        clr a
        addc a,r1
        mov r1,a

        inc dptr
        mov a,dph
        cjne a,#20h,gc1     ; done all EPROM?

        mov csum,r0      ; save for IBM
        mov csum+1,r1

        mov led_cnt,#8          ; set led flash freq    
        mov led_cnt_seed,#8



; Set-up Interrupts and their Priorities


        mov rcap2l,#low  maxdelay
        mov rcap2h,#high maxdelay
        mov t2con,#00000100b    ; timer2, 16bit autoreload timer

        ; To enable serial debugger:        
        ; replace following

;       mov t2con,#00110100b    ; serial generat
;       mov rcap2l,#low  65484  ; gives 9615 baud
;       mov rcap2h,#high 65484
;       mov scon,#01000000b
;       mov sbuf,a
        
        mov  tmod,#00010001b    ; timer0, timer1 both 16 bit timers
        mov  tcon,#00010000b    ; ext0 and ext1 level triggered, timer 0 running
        
        mov  ie  ,#10100011b    ; enable ext0 timer2,timer0  
        mov  ip  ,#00000001b    ; ext0 high priority


    clr  jammed
;    mov control0,#AXIS0_SPD
;    mov control1,#AXIS1_SPD
;    mov control2,#AXIS2_SPD
;    mov control3,#AXIS3_SPD
     
     


    setb enable_jam

    mov despos_speed,#3         ; Default to fastest
    acall setmaxerror


;    clr enable_jam


    page
     
     
    SUBTTL Baseloop

    
    setb enable_pen    

    
    clr enable_jam

    
    lcall home



    

BASELOOP:
    mov a,mcntrl
    jz bl1

    mov mcntrl,#0
    mov mcntrl_copy,a
    mov dptr,#master_decode
    jmp @a+dptr             ; will return to bl0
bl0:
    mov scntrl,mcntrl_copy


bl1:
    jb  enable_pen,bl1a
    jmp bl2
bl1a:

    LCALL readpen1          

    cjne a,#0ffh,bl1_1
    clr  pen_active     ; Tell IBM to set pendant mode
    ajmp bl1_2

bl1_1:    
    setb  pen_active    
    
bl1_2:

    LCALL readpen0      ; 0,1 speed
                        ; 2   store
                        ; 3   grip
    jnb  acc.3,bl9_a
    clr  grip_press
    jmp  bl9_b
bl9_a:
    jb   grip_press,bl9_b
    setb grip_press
    push acc
    mov  a,despos_grip
    cpl  acc.4
    mov  despos_grip,a
    pop  acc

bl9_b:
    anl  a,#11b
    cjne a,#11b,bl1_2a         ;slow
    mov  pen_speed,#0
    jmp  bl1_2c

bl1_2a:
    cjne a,#01b,bl1_2b         ;medium
    mov  pen_speed,#1
    jmp  bl1_2c

bl1_2b:    
    cjne a,#10b,bl1_2c         ;fast
    mov  pen_speed,#3


bl1_2c:
    LCALL readpen1          
                      
    jb   acc.0,bl1_3    ; Axis X
    lcall pen_0m
    jmp bl1_5

bl1_3:
    jb   acc.1,bl1_4
    lcall pen_0p
    jmp bl1_5

bl1_4:
    jnb  pen0,bl1_5
    setb stop0              ; don't want X moving anymore
    


bl1_5:                      ; Axis Y
    jb   acc.2,bl1_6
    lcall pen_1m
    jmp bl1_8
bl1_6:
    jb   acc.3,bl1_7
    lcall pen_1p
    jmp bl1_8
bl1_7:
    jnb  pen1,bl1_8
    setb stop1              ; don't want Y moving anymore


bl1_8:                      ; Axis Z
    jb   acc.4,bl1_9
    lcall pen_2m
    jmp bl1_11
bl1_9:
    jb   acc.5,bl1_10
    lcall pen_2p
    jmp bl1_11
bl1_10:    
    jnb  pen2,bl1_11
    setb stop2              ; don't want Z moving anymore


bl1_11:                     ; Axis R
    jb   acc.6,bl1_12
    lcall pen_3m
    jmp bl1_14
bl1_12:
    jb   acc.7,bl1_13
    lcall pen_3p
    jmp bl1_14
bl1_13:
    jnb  pen3,bl1_14
    setb stop3              ; don't want R moving anymore
bl1_14:
    


bl2:
    mov   a,stallflags      ; any axis jammed?
    jz    bl20

    mov  stallflags_ibm,a   ; tell ibm which axes are jammed
    mov  stallflags,#0      ; re-enable servo
                            ; want to be able to use pendant to back off
    
    setb jammed             ; tell IBM

emer_stop:
    ;orl flags,#01011111b     ; == setb pen0 etc   
    setb stop0              ; causes despos=encpos
    setb stop1
    setb stop2
    setb stop3
    setb enable_pen


bl20:    
    JMP BASELOOP



master_decode:                   ; jump table for IBM intructions
    ajmp bl0                     ; 0 - unused
    ajmp nojams                  ; 2 - turn arm jam off
    ajmp jams                    ; 4 - turn arm jams back on
    ajmp nopen                   ; 6 - turn pendant off
    ajmp emer_stop               ; 8 
    ajmp jog_0m                  ; 10 - move axis 0 towards 0   
    ajmp jog_0p                  ; 12
    ajmp jog_1m                  ; 14 - move axis 1 towards 0   
    ajmp jog_1p                  ; 16
    ajmp jog_2m                  ; 18 - move axis 2 towards 0   
    ajmp jog_2p                  ; 20
    ajmp jog_3m                  ; 22 - move axis 3 towards 0   
    ajmp jog_3p                  ; 24
    ajmp jog_5m                  ; 26 - gripper
    ajmp jog_5p                  ; 28
    ajmp jog_sm                  ; 30 - Speed -ve
    ajmp jog_sp                  ; 32
    ajmp jog_end                 ; 34
    ajmp jog_end                 ; 36
    ajmp jog_end                 ; 38
    ajmp jog_end                 ; 40
    ajmp jog_end                 ; 42
    ajmp jog_end                 ; 44
    ajmp jog_end                 ; 46 - end of jog, re-enable pendant
    ajmp jog_end                 ; 48
    ajmp jog_clearjam            ; 50
    ajmp jog_home                ; 52
    ajmp jog_reset               ; 54

nopen:
    clr enable_pen
    ajmp bl0

nojams:
    clr  enable_jam
    ajmp bl0

jams:
    setb enable_jam
    ajmp bl0

jog_clearjam:
    clr jammed                  ; clear arm jammed flag
    mov stallflags,#0

;    mov control0,#AXIS0_SPD
;    mov control1,#AXIS1_SPD
;    mov control2,#AXIS2_SPD
;    mov control3,#AXIS3_SPD


    ajmp bl0

jog_sm:
    mov a,despos_speed
    jz  sm_1
    dec a
sm_1:    
    mov despos_speed,a
    ajmp bl0

jog_sp:
    mov a,despos_speed
    cjne a,#3,sp_1
    ajmp bl0
     
     
sp_1:    
    inc a
    mov despos_speed,a
    ajmp bl0

jog_0m:
    clr   enable_pen
    lcall pen_0m
    ajmp  bl0 

jog_0p:
    clr   enable_pen
    lcall pen_0p
    ajmp  bl0 

jog_1m:
    clr   enable_pen
    lcall pen_1m
    ajmp  bl0 
jog_1p:
    clr   enable_pen
    lcall pen_1p
    ajmp  bl0 

jog_2m:
    clr   enable_pen
    lcall pen_2m
    ajmp  bl0 
jog_2p:
    clr   enable_pen
    lcall pen_2p
    ajmp  bl0 

jog_3m:
    clr   enable_pen
    lcall pen_3m
    ajmp  bl0 
jog_3p:
    clr   enable_pen
    lcall pen_3p
    ajmp  bl0 

jog_5m:
    mov  despos_grip,#0
    ajmp bl0 

jog_5p:
    mov  despos_grip,#16
    ajmp bl0 

jog_end:
    setb enable_pen
    ajmp bl0


jog_home:
jog_reset:
    jmp bl0


pen_0m:
    mov  despos0  , #low 1024
    mov  despos0+1, #high 1024
    setb pen0
    ret

pen_0p:
    mov  despos0  , #low  51024      ;500*100 + 1024
    mov  despos0+1, #high 51024 
    setb pen0
    ret

pen_1m:
    mov  despos1  , #low 1024
    mov  despos1+1, #high 1024
    setb pen1
    ret

pen_1p:
    mov  despos1  , #low  36024      ;350*100 + 1024 
    mov  despos1+1, #high 36024
    setb pen1
    ret

pen_2m:
    mov despos2  , #low 1024
    mov despos2+1, #high 1024
    setb pen2
    ret
pen_2p:
    mov despos2  , #low  25024      ;150*160 + 1024   
    mov despos2+1, #high 25024
    setb pen2
    ret

pen_3m:
    mov despos3  , #low 1024
    mov despos3+1, #high 1024
    setb pen3
    ret
pen_3p:
    mov despos3  , #low  1824       ;2*400 + 1024
    mov despos3+1, #high 1824
    setb pen3
    ret





pen_runin:
    
    clr a 
    mov r0,#runcnt
    
pen_clear:    
    mov @r0,a
    inc r0
    cjne r0,#stallcnt4 + 1,pen_clear


    mov seqptr  ,#low runseq        ; set pointer to start of sequence
    mov seqptr+1,#high runseq
    mov seqcnt,#1
    mov seqcnt+1,#1

    ajmp pen_run2                   ; send first line
    

pen_run0:    
    mov  a,mcntrl
    jz   pen_run1
    ajmp baseloop       ; return to baseloop
    
pen_run1:
    mov stallflags,#0   ; reset jmammed axes

    mov a,inpos
    anl a,#01fh
    cjne a,#01fh,pen_run4
                         ;arm inposition, set up new data

pen_run2:
    djnz seqcnt  ,pen_run4
    djnz seqcnt+1,pen_run4

    mov seqcnt+1,#16
    
    mov dpl,seqptr
    mov dph,seqptr+1
    mov r0,#despos_speed

    clr  a
    movc a,@a+dptr
    inc  dptr
    jnz  pen_run3

    mov r0,#runcnt
    inc @r0
    mov a,@r0
    jnz pen_run2a
    
    inc r0
    inc @r0
    mov a,@r0
    jnz pen_run2a
    
    jmp baseloop

pen_run2a:    
    mov seqptr  ,#low runseq    ;at end of sequence point back to start
    mov seqptr+1,#high runseq
    ajmp pen_run2


pen_run3:
    clr a
    movc a,@a+dptr
    mov @r0,a
    inc dptr            ; inc pointer to source
    dec r0              ; dec pointer to dest
    mov a,r0
    cjne a,#despos0-1,pen_run3

    mov seqptr,dpl
    mov seqptr+1,dph

pen_run4:
    
    ajmp pen_run0


seterror0:     

    clr  c
    mov  a,despos0
    subb a,encpos0
    mov  r6,a           ; LSB error
    
    mov  a,despos0+1
    subb a,encpos0+1
    mov  r7,a           ; MSB error
    
    mov dir0,c          ; save direction
    jnc setspd0

    clr  c
    mov  a,encpos0
    subb a,despos0
    mov  r6,a           ; absolute LSB error
    
    mov  a,encpos0+1
    subb a,despos0+1
    mov  r7,a           ; absolute MSB error

setspd0:
    mov error0,r6
    mov error0+1,r7

    ret




servo0:
servox:
    mov dptr,#0ce00h
    movx a,@dptr
    jb acc.7,servo0_pow ; Power on servo 
    mov a,#30           ; Send emerstop if no power
    jmp send0

servo0_pow:  
    acall setmaxerror
    mov   maxerror,max0
    lcall seterror0
    
    
    anl a,#11111000b    ; 2048 table
    jz setspd0_1
    
    mov a,maxerror      ; 14 for fast, 12, 10, or 6 for slow
    jmp setdir0

setspd0_1:
    
    acall readerror     ; pointed to by dptr,r6
    cjne a,#window,setdir0 
    mov a,#4
    jnb inner0,setdir0
    clr a
setdir0:
    jz   send0_in       ; stopped? - don't add direction
    clr  inner0             
    mov  c,dir0
    mov  acc.0,c        ; make signed speed
send0:   
    acall send_control0
    ret

send0_in:
    setb inner0         ; real stop open window
    jmp send0



seterror1:
    clr  c
    mov  a,despos1
    subb a,encpos1
    mov  r6,a           ; LSB error
    
    mov  a,despos1+1
    subb a,encpos1+1
    mov  r7,a           ; MSB error
    mov dir1,c           ; save direction
    jnc setspd1

    clr  c
    mov  a,encpos1
    subb a,despos1
    mov  r6,a           ; absolute LSB error
    
    mov  a,encpos1+1
    subb a,despos1+1
    mov  r7,a           ; absolute MSB error

setspd1:
    mov error1,r6
    mov error1+1,r7

    ret




servo1:
    mov dptr,#0ce00h
    movx a,@dptr
    jb acc.7,servo1_pow ; Power on servo 
    mov a,#30           ; Send emerstop if no power
    jmp send1

servo1_pow:  

    acall setmaxerror
    mov   maxerror,max1
    lcall seterror1
    
    anl a,#11111000b    ; 2048 table
    jz setspd1_1
    
    mov a,maxerror      ; = 14 for fast
    jmp setdir1

setspd1_1:
    
    acall readerror     ; pointed to by dptr,r6
    cjne a,#window,setdir1
    mov a,#4
    jnb inner1,setdir1
    clr a

setdir1:
    jz   send1_in       ; stopped? - don't add direction
    clr  inner1
    mov  c,dir1
    cpl  c              ; reverse direction    
    mov  acc.0,c        ; make signed speed
    
send1:   
    acall send_control1
    ret

send1_in:  
    setb inner1
    jmp send1



seterror2:
    clr  c
    mov  a,despos2
    subb a,encpos2
    mov  r6,a           ; LSB error
    
    mov  a,despos2+1
    subb a,encpos2+1
    mov  r7,a           ; MSB error
    mov  dir2,c         ; save direction
    jnc  setspd2

    mov  max0,#0        ; don't servo axes X and Y
    mov  max1,#0

    clr  c
    mov  a,encpos2
    subb a,despos2
    mov  r6,a           ; absolute LSB error
    
    mov  a,encpos2+1
    subb a,despos2+1
    mov  r7,a           ; absolute MSB error

setspd2:
    mov error2,r6
    mov error2+1,r7

    ret



servo2:
    mov dptr,#0ce00h
    movx a,@dptr
    jb acc.7,servo2_pow ; Power on servo 
    mov a,#30           ; Send emerstop if no power
    jmp send2

servo2_pow:  

    acall setmaxerror
    mov   maxerror,max2
    lcall seterror2     ; r6/r7(a) = error

    anl a,#11111000b    ; 2048 error entries for axis 2/Z
    jz setspd2_1
    
    mov a,maxerror      ; = 14 for fast
    jmp setdir2

setspd2_1:
    
    acall readerror     ; pointed to by dptr,r6
    cjne a,#window,setdir2
    mov a,#4
    jnb inner2,setdir2
    clr a

setdir2:
    jz   send2_in       ; stopped? - don't add direction
    clr  inner2
    mov  c,dir2
    mov  acc.0,c        ; make signed speed
    
send2:   
    lcall send_control2
    ret
send2_in:
    setb inner2
    jmp  send2



seterror3:
    clr  c
    mov  a,despos3
    subb a,encpos3
    mov  r6,a           ; LSB error
    
    mov  a,despos3+1
    subb a,encpos3+1
    mov  r7,a           ; MSB error
    mov dir3,c          ; save direction
    jnc setspd3

    clr  c
    mov  a,encpos3
    subb a,despos3
    mov  r6,a           ; absolute LSB error
    
    mov  a,encpos3+1
    subb a,despos3+1
    mov  r7,a           ; absolute MSB error

setspd3:
    mov error3,r6
    mov error3+1,r7

    ret

servo3:
    mov dptr,#0ce00h
    movx a,@dptr
    jb acc.7,servo3_pow ; Power on servo 
    mov a,#30           ; Send emerstop if no power
    jmp send3

servo3_pow:  

    acall setmaxerror
    mov   maxerror,max3
    lcall seterror3
    
    anl a,#11111000b    ; 2048 error entries for axis 3/R
    jz setspd3_1        ;
    
    mov a,maxerror      ; = 14 for fast
    jmp setdir3

setspd3_1:
    
    acall readerror     ; pointed to by dptr,r6
    
    cjne a,#window,setdir3
    mov a,#4
    jnb inner3,setdir3
    clr a
setdir3:
    jz   send3_in       ; stopped? - don't add direction

;    mov a,#2            ; only send slow

    clr  inner3
    mov  c,dir3
    mov  acc.0,c        ; make signed speed
    
send3:   
    lcall send_control3
    ret

send3_in:
    setb inner3
    jmp send3




setmaxerror:
    mov a,pen_speed
    
    jb pen_active,smax1
    
    mov a,despos_speed
    mov encpos_speed,a

smax1:
    jnz spd1
    mov a,#2        ; Slowest speed
    mov max3,#2
    jmp spd4

spd1:
    cjne a,#1,spd2
    mov a,#6
    mov max3,#4     ; Wrist has only two speeds
    jmp spd4
spd2:
    cjne a,#2,spd3
    mov a,#12
    mov max3,#6
    jmp spd4
spd3:
    mov a,#26
    mov max3,#6

spd4:
    mov max0,a      
    mov max1,a
    mov max2,a

    jnb jammed,spd4a        ; If Jammed allow pendant any move
    ret

spd4a:
    mov a,error2+1  ; Z inposition?   
    jnz zfirst_1
    
    mov  a,error2    
    cjne a,#10,$+3  ;a<10 c=1
    jnc   zfirst_1
    ret

zfirst_1:
    jnb dir2,zfirst
    mov a,error0+1
    jnz zfirst_2
    mov a,error1+1
    jnz zfirst_2
    
    mov  a,error0
    cjne a,#10,$+3  ;a<10 c=1
    jnc   zfirst_2
    
    mov  a,error1
    cjne a,#10,$+3  ;a<10 c=1
    jnc   zfirst_2
    ret

zfirst_2:
    mov max2,#0     ; X and Y not yet inpos, don't servo Z
    ret


zfirst:    
    mov max0,#0     ; don't servo axes X and Y
    mov max1,#0
    ret



readerror:

    MOV A,R7
    ADD A,#HIGH errortab
    mov dph,a
    mov dpl,#low errortab

    mov  a,r6           ; lsb error
    movc a,@a+dptr      ; read unsigned speed from error table

    cjne a,#window,re0
    ret

re0:
    cjne a,maxerror,re1
    ret                 ; a = maxerror, ok
re1:
    jc re2              ; jump if a < maxerror
    mov a,maxerror
re2:
    ret




update_posnx:
    clr a 
    mov r0,#encpos0       ; point to data to change
    mov dptr,#Slave0
    jmp update_posn

update_posny:
    clr  a 
    mov r0,#encpos1       ; point to data to change
    mov dptr,#Slave1
    jmp update_posn

update_posnz:
    mov a,#2
    mov r0,#encpos2       ; point to data to change
    mov dptr,#Slave0
    jmp update_posn

update_posnr:
    mov a,#2
    mov r0,#encpos3       ; point to data to change
    mov dptr,#Slave1
    jmp update_posn


update_posn:
    ;dptr will be either Slave0 or Slave1
    
    movx @dptr,a        ; Request data from encoder slave
    jb p3.3,$           ; wait for slave to respond
    movx a,@dptr        ; Read data from slave
    
    mov change,a
    jb acc.7,neg_change
    
    add a,@r0
    mov @r0,a
    jnc nomodmsb

    inc r0              ; Increment pointer, to point to msb
    inc @r0             ; increment msb
    ajmp nomodmsb

neg_change:
    cpl a
    inc a
    mov change,a
    xch a,@r0
    clr c
    subb a,@r0
    mov @r0,a
    
    inc r0
    mov a,@r0
    subb a,#0
    mov @r0,a

nomodmsb:
    

    ret


home:
    setb  zeroing           ; show IBM we're zeroing
    
    clr   enable_servo0
    clr   enable_servo1
    clr   enable_servo2
    clr   enable_servo3
    
    lcall homez    
    lcall homer
    lcall homey
    lcall homex

    clr   zeroing             ; show IBM we're ready

    ret


homex:
    mov   a,#0             ; Stop X
    lcall send_control0x

    lcall waitpower
    
    mov   a,#27             ; send X towards usw asap
    lcall send_control0x

    mov r0,#0
    mov r1,#0
    
usw0a:    
    mov dptr,#0ce00h
    movx a,@dptr
    jnb acc.7,homex         ; if power off restart

    lcall readx_us          ; read and debounce
    jb    acc.0,usw0a
    
    mov   a,#30
    lcall send_control0x
    

    mov a,#4                ; very slow away 
    lcall send_control0x

usw0b:    
    lcall readx_us          ; read and debounce
    jnb   acc.0,usw0b


    mov a,#30
    lcall send_control0x
    
    mov  dptr,#slave0   ; clear encoder counter
    clr  a
    
    clr   tcon.4        ; pause servo timer
    movx @dptr,a        
    jb   p3.3,$           
    movx a,@dptr        

    mov   encpos0,#low 1024
    mov   encpos0+1,#high 1024

    mov   despos0,  #low 1024
    mov   despos0+1,#high 1024

    setb  enable_servo0

    setb  tcon.4        ; restart servo timer
    
    ret


homey:    
    mov   a,#0             ; Stop Y
    lcall send_control1x

    lcall waitpower

    mov   a,#26             ; send Y towards usw asap
    lcall send_control1x

    mov r2,#0
    mov r3,#0
    
usw1a:    
    mov dptr,#0ce00h
    movx a,@dptr
    jnb acc.7,homey         ; if power off restart

    lcall readY_us          ; read and debounce
    jb    acc.1,usw1a
    
    mov   a,#30
    lcall send_control1x
    

    mov a,#5                ; very slow away 
    lcall send_control1x

usw1b:    
    lcall readY_us          ; read and debounce
    jnb   acc.1,usw1b


    mov a,#30
    lcall send_control1x
    
    mov  dptr,#slave1   ; clear encoder counter
    clr  a
    
    clr  tcon.4        ; pause servo timer
    movx @dptr,a        
    jb   p3.3,$           
    movx a,@dptr        

    mov encpos1,#low 1024
    mov encpos1+1,#high 1024

    mov despos1,  #low 1024
    mov despos1+1,#high 1024
    
    setb  enable_servo1
    
    setb  tcon.4        ; restart servo timer

    ret

homez:
    mov   a,#0             ; Stop Z
    lcall send_control2x

    lcall waitpower
    
    
    mov   a,#26             ; send Z towards usw asap
    lcall send_control2x

    mov r0,#0
    mov r1,#0
    
usw2a:    
    mov dptr,#0ce00h
    movx a,@dptr
    jnb acc.7,homez         ; if power off restart

    lcall readZ_us          ; read and debounce
    jb    acc.2,usw2a
    
    mov   a,#30
    lcall send_control2x
    

    mov a,#5                ; very slow away 
    lcall send_control2x

usw2b:    
    lcall readZ_us          ; read and debounce
    jnb   acc.2,usw2b


    mov a,#30
    lcall send_control2x

;    lcall update_posnz      ; clear encoder counter

    mov  dptr,#slave0   ; clear encoder counter
    mov  a,#2
    
    clr  tcon.4        ; pause servo timer
    movx @dptr,a        
    jb   p3.3,$           
    movx a,@dptr        


    mov encpos2,#low 25024          ;150*160 + 1024
    mov encpos2+1,#high 25024
    
    mov despos2,  #low 25024          ;150*160 + 1024
    mov despos2+1,#high 25024 

    setb  enable_servo2

    setb  tcon.4        ; restart servo timer

    ret


homer:
    mov   a,#0             ; Stop R
    lcall send_control3x

    lcall waitpower
    
    
    mov   a,#4             ; send R towards usw asap
    lcall send_control3x

    mov r2,#0
    mov r3,#0
    
usw3a:    
    mov dptr,#0ce00h
    movx a,@dptr
    jnb acc.7,homer         ; if power off restart

    lcall readR_us          ; read and debounce
    jb    acc.3,usw3a
    
    mov   a,#30
    lcall send_control3x
    

    mov a,#5                ; very slow away 
    lcall send_control3x

usw3b:    
    lcall readR_us          ; read and debounce
    jnb   acc.3,usw3b


    mov a,#30
    lcall send_control3x

;    lcall update_posnR      ; clear encoder counter

    mov  dptr,#slave1   ; clear encoder counter
    mov  a,#2
    
    clr  tcon.4        ; pause servo timer
    movx @dptr,a        
    jb   p3.3,$           
    movx a,@dptr        

    mov encpos3,#low 1024
    mov encpos3+1,#high 1024
    
    mov despos3,  #low 1024
    mov despos3+1,#high 1024

    setb  enable_servo3

    setb  tcon.4        ; restart servo timer

    ret




waitpower:
    mov dptr,#0ce00h
    movx a,@dptr
    jnb acc.7,waitpower
    ret


send_control0x:    
    clr   tcon.4
    lcall send_control0
    setb  tcon.4 
    ret

send_control1x:    
    clr   tcon.4
    lcall send_control1
    setb  tcon.4 
    ret

send_control2x:    
    clr   tcon.4
    lcall send_control2
    setb  tcon.4 
    ret

send_control3x:    
    clr   tcon.4
    lcall send_control3
    setb  tcon.4 
    ret




send_control0:    
send_controlx:
    jz   send0_0            ; always let stop through
    jnb  stalled0,send0_1
    ret
send0_0:
    jb   axis0_inpos,send0_3    ; already stopped?  

    mov  a,#28              ; Sending stop - check if inposition
    orl  a,#AXIS0_SPD    
    mov  dptr,#Slave2
    movx @dptr,a            ; send control byte to slave
    jb   p3.3,$             ; wait for slave to respond
    movx a,@dptr            ; read back slaves state
    jnz  send0_4            ; is it stopped?
    setb axis0_inpos        ; Its stopped - set inposition
    ret

send0_4:
    clr  a
send0_1:
    orl  a,#AXIS0_SPD    
    cjne a,control0,send0_2 ; only bother to send new control words
    ret
send0_2:
    mov  dptr,#Slave2
    movx @dptr,a            ; send control byte to slave
    jb   p3.3,$             ; wait for slave to respond
    movx a,@dptr            ; read back slaves state
    mov  control0,a         ; save as control byte
    anl  a,#11110b
    jz   send0_3            ; is slave stopping ?
    
    clr  axis0_inpos        ; no, so axis not inposition

send0_3:    
    ret


send_control1:    
send_controly:    
    jz   send1_0            ; always let stop through
    jnb  stalled1,send1_1
    ret
send1_0:
    jb   axis1_inpos,send1_3    ; already stopped?  

    mov  a,#28              ; Sending stop - check if inposition
    orl  a,#AXIS1_SPD    
    mov  dptr,#Slave3
    movx @dptr,a            ; send control byte to slave
    jb   p3.3,$             ; wait for slave to respond
    movx a,@dptr            ; read back slaves state
    jnz  send1_4            ; is it stopped?
    setb axis1_inpos        ; Its stopped - set inposition
    ret

send1_4:
    clr  a
send1_1:
    orl  a,#AXIS1_SPD    
    cjne a,control1,send1_2 ; only bother to send new control words
    ret
send1_2:
    
    mov  dptr,#Slave3
    movx @dptr,a            ; send control byte to slave
    jb   p3.3,$             ; wait for slave to respond
    movx a,@dptr            ; read back slaves state
    mov  control1,a         ; save as control byte
    
    anl  a,#11110b
    jz   send1_3            ; is slave stopping ?
    clr  axis1_inpos        ; no, so axis not inposition


send1_3:    
    ret



send_control2:    
send_controlz:    
    jz   send2_0            ; always let stop through
    jnb  stalled2,send2_1
    ret
send2_0:
    jb   axis2_inpos,send2_3    ; already stopped?  

    mov  a,#28              ; Sending stop - check if inposition
    orl  a,#AXIS2_SPD    
    mov  dptr,#Slave2
    movx @dptr,a            ; send control byte to slave
    jb   p3.3,$             ; wait for slave to respond
    movx a,@dptr            ; read back slaves state
    jnz  send2_4            ; is it stopped?
    setb axis2_inpos        ; Its stopped - set inposition
    ret

send2_4:
    clr  a
send2_1:
    orl  a,#AXIS2_SPD    
    cjne a,control2,send2_2 ; only bother to send new control words
    ret
send2_2:
    mov  dptr,#Slave2
    movx @dptr,a            ; send control byte to slave
    jb   p3.3,$             ; wait for slave to respond
    movx a,@dptr            ; read back slaves state
    mov  control2,a         ; save as control byte
    anl  a,#11110b
    jz   send2_3            ; is slave stopping ?
    
    clr  axis2_inpos        ; no, so axis not inposition



send2_3:    
    ret

send_control3:    
send_controlr:    

;    jnb ti,$
;    clr ti
;    mov sbuf,a


    jz   send3_0            ; always let stop through
    jnb  stalled3,send3_1
    ret
send3_0:
    jb   axis3_inpos,send3_3    ; already stopped?  

    mov  a,#28              ; Sending stop - check if inposition
    orl  a,#AXIS3_SPD    
    mov  dptr,#Slave3
    movx @dptr,a            ; send control byte to slave
    jb   p3.3,$             ; wait for slave to respond
    movx a,@dptr            ; read back slaves state
    jnz  send3_4            ; is it stopped?
    setb axis3_inpos        ; Its stopped - set inposition
    ret

send3_4:
    clr  a
send3_1:
    orl  a,#AXIS3_SPD    
    cjne a,control3,send3_2 ; only bother to send new control words
    ret
send3_2:
    mov  dptr,#Slave3
    movx @dptr,a            ; send control byte to slave
    jb   p3.3,$             ; wait for slave to respond
    movx a,@dptr            ; read back slaves state
    mov  control3,a         ; save as control byte
    anl  a,#11110b
    jz   send3_3            ; is slave stopping ?
    
    clr  axis3_inpos        ; no, so axis not inposition
    
send3_3:    
    ret





readx_us:
        mov  dptr,#microsws     ; read micro-switch states
rg3:
        movx a,@dptr
        jnb  acc.0,rg2
        
        mov  r0,#0
        inc  r1
        cjne r1,#0,rg3
        dec  r1
        ret
rg2:
        mov  r1,#0
        inc  r0
        cjne r0,#0,rg3
        dec  r0
        ret


ready_us:
        mov  dptr,#microsws     ; read micro-switch states
rg6:        
        movx a,@dptr
        jnb  acc.1,rg5
        
        mov  r2,#0
        inc  r3
        cjne r3,#0,rg6
        dec  r3
        ret
rg5:
        mov  r3,#0
        inc  r2
        cjne r2,#0,rg6
        dec  r2
        ret


readz_us:
        mov  dptr,#microsws     ; read micro-switch states
rg9:
        movx a,@dptr
        jnb  acc.2,rg8
        
        mov  r0,#0
        inc  r1
        cjne r1,#0,rg9
        dec  r1
        ret
rg8:
        mov  r1,#0
        inc  r0
        cjne r0,#0,rg9
        dec  r0
        ret

readR_us:
        mov  dptr,#microsws     ; read micro-switch states
rg16:        
        movx a,@dptr
        jnb  acc.3,rg15
        
        mov  r2,#0
        inc  r3
        cjne r3,#0,rg16
        dec  r3
        ret
rg15:
        mov  r3,#0
        inc  r2
        cjne r2,#0,rg16
        dec  r2
        ret



readpen0:
        mov  dptr,#0ce00h
rp12:      
        movx a,@dptr
        cjne a,pen_copy0,rp1 ; pendant latch changed?
        inc  r3
        cjne r3,#0,rp12
        dec  r3
        ret
rp11:
        mov r3,#0           ; clear debounce count
        mov pen_copy0,a
        jmp rp12


readpen1:
        mov  dptr,#0ca00h
rp2:      
        movx a,@dptr
        cjne a,pen_copy1,rp1 ; pendant latch changed?
        inc  r2
        cjne r2,#0,rp2
        dec  r2
        ret
rp1:
        mov r2,#0           ; clear debounce count
        mov pen_copy1,a
        jmp rp2

        




timer0:    
    mov th0,#250        ;250 gives changes of 53 on X

    push psw
    push acc
    push dph
    push dpl

    mov  dptr,#0ce00h       ; send gripper data
    mov  a,despos_grip
    mov  encpos_grip,a
    movx @dptr,a
    
    mov   a,timer0func
    jnz   t0_2
    
    inc   timer0func
    lcall update_posnx

    jnb   enable_servo0,t0_1a
    jnb   stop0,t0_5
    
    clr   a                   ;Send stop
    lcall send_control0
    
    mov   a,change
    jnz   t0_1a
    clr   stop0
    clr   pen0
    mov   despos0,  encpos0
    mov   despos0+1,encpos0+1
t0_1a:
    jmp   t0_1
t0_5:
    mov  a,control0
    anl  a,#1eh         ; mask off axis/direction info
    jz   h1
    
    cjne  a,#28,$+3
    jnc   h1

    djnz stall0,h2      ; only look every 25 ints
    
    mov a,encpos0
    cjne a,lastencpos0,h1
    
    mov a,encpos0+1
    cjne a,lastencpos0+1,h1
    
    mov   a,#30         ; no change 
    lcall send_control0
    setb  stalled0


h1:
    mov stall0,#STALLPOLL
    mov lastencpos0,encpos0
    mov lastencpos0+1,encpos0+1

h2:
    lcall seterror1
    lcall seterror2
    lcall seterror3
    lcall servox
    jmp   t0_1

t0_2:
    cjne  a,#1,t0_3
    inc   timer0func
    lcall update_posny
    
    jnb   enable_servo1,t0_1a
    
    jnb   stop1,t0_6

    clr   a                   ;Send stop
    lcall send_control1
    
    mov   a,change
    jnz   t0_1a
    clr   stop1
    clr   pen1
    mov   despos1,  encpos1
    mov   despos1+1,encpos1+1

    jmp   t0_1
t0_6:
    mov  a,control1
    anl  a,#1eh         ; mask off axis/direction info
    jz   h3
    
    cjne  a,#28,$+3
    jnc   h3


    djnz stall1,h4      ; only look every 25 ints
    
    mov a,encpos1
    cjne a,lastencpos1,h3
    
    mov a,encpos1+1
    cjne a,lastencpos1+1,h3
    
    mov   a,#30         ; no change 
    lcall send_control1
    setb  stalled1

h3:
    mov stall1,#STALLPOLL
    mov lastencpos1,encpos1
    mov lastencpos1+1,encpos1+1

h4:
    
    lcall seterror0
    lcall seterror2
    lcall seterror3
    lcall servo1
    jmp   t0_1


t0_3:
    cjne  a,#2,t0_4
    inc   timer0func
    lcall update_posnz

    
    jnb   enable_servo2,t0_1b


    jnb   stop2,t0_7

    clr   a                   ;Send stop
    lcall send_control2
    
    mov   a,change
    jnz   t0_1b
    clr   stop2
    clr   pen2
    mov   despos2,  encpos2
    mov   despos2+1,encpos2+1
t0_1b:
    jmp   t0_1
t0_7:

    mov  a,control2
    anl  a,#1eh         ; mask off axis/direction info
    jz   h5
    
    cjne  a,#28,$+3
    jnc   h5

    djnz stall2,h6      ; only look every 25 ints
    
    mov a,encpos2
    cjne a,lastencpos2,h5
    
    mov a,encpos2+1
    cjne a,lastencpos2+1,h5
    
    mov   a,#30         ; no change 
    lcall send_control2
    setb  stalled2

h5:
    mov stall2,#STALLPOLL
    mov lastencpos2,encpos2
    mov lastencpos2+1,encpos2+1
h6:


    lcall seterror0
    lcall seterror1
    lcall seterror3
    lcall servo2

    jmp   t0_1

t0_4:
    mov   timer0func,#0
    lcall update_posnr
    
    jnb   enable_servo3,t0_1

    jnb   stop3,t0_8

    clr   a                   ;Send stop
    lcall send_control3
    
    mov   a,change
    jnz   t0_1
    clr   stop3
    clr   pen3
    mov   despos3,  encpos3
    mov   despos3+1,encpos3+1

    jmp   t0_1
t0_8:

    mov  a,control3
    anl  a,#1eh         ; mask off axis/direction info
    jz   h7
    
    cjne  a,#28,$+3
    jnc   h7

    djnz stall3,h8      ; only look every 25 ints
    
    mov a,encpos3
    cjne a,lastencpos3,h7
    
    mov a,encpos3+1
    cjne a,lastencpos3+1,h7
    
    mov   a,#30         ; no change 
    lcall send_control3
    setb  stalled3

h7:
    mov stall3,#STALLPOLL
    mov lastencpos3,encpos3
    mov lastencpos3+1,encpos3+1
h8:


    lcall seterror0
    lcall seterror1
    lcall seterror2
    lcall servo3

t0_1:

    pop dpl
    pop dph
    pop acc
    pop psw
    
    reti






include ibmint.inc              ; External interrupt 0 service

include timer2.inc              ; Timer 2

;include runin.inc
runseq:

include errtab.inc
end
