    PAGE
    SUBTTL "Timer 0 ISR"


;Modified 19/4/95
; Timer0 runs at twice step freq
;   First interrupt sets step low
;   Second interrupt returns step high




;r0     - lsb steps (x or z)    
;r1     - msb steps (x or z)

;r2     - lsb steps (y)         
;r3     - msb steps (y)         

;r4     - timer 0 function call
;r5     - timer 1 function call


timer0: 
        mov th0,th0_seed        ;2  reseed timer
        mov tl0,tl0_seed        ;2 = 12 cycles before timer restarts
        

;Functions
;Dummy

;Step x asap (up/flat/down) entry is up
;Step z asap (up/flat/down) entry is up

;Step XZ fixed (7.6.5.4.3.2.1.0) entry is 7

;Step x-ve at jog feedrate
;Step x+ve at jog feedrate

;Step z-ve at jog feedrate
;Step z+ve at jog feedrate
        
        jnb sentxstep,t0_0a
        clr sentxstep
        setb XSTP
        setb intused
t0_0a:
        jnb sentzstep,t0_0c
        clr sentzstep
        setb ZSTP
        setb intused
t0_0c:
        jnb intused,t0_0d
        clr intused
        reti
t0_0d:
        
        
        push psw                    ; time critical section
        push acc                    ; operating as level 1 interrupt
        push dpl                    ; so can't be interrupted
        push dph                

        jnb sendxstep,t0_2a
        jb  stepper_power,t0_power    ; can't step until power is on

t0_1a:
        clr XSTP                    ; set step for x low
        setb sentxstep
        jb ignore_step,t0_2a
        
        jnb xdir,t0_1b          ; Steps count - inc or dec position?
        
        mov  a,xpos             ; increment position
        add  a,#1
        mov  xpos,a
        mov  a,xpos+1
        addc a,#0
        mov  xpos+1,a
        ajmp  t0_2a

t0_1b:
        clr  c                  ; decrement position
        mov  a,xpos
        subb a,#1
        mov  xpos,a
        
        mov  a,xpos+1
        subb a,#0
        mov  xpos+1,a


t0_2a:
        jnb sendzstep,t0_3a
        jb  stepper_power,t0_power  ; switch power on
        
        clr ZSTP               ;set step for z low
        setb sentzstep
        jb  ignore_step,t0_3a
        jnb zdir,t0_2b
        
        mov  a,zpos             ; increment position
        add  a,#1
        mov  zpos,a
        
        mov  a,zpos+1
        addc a,#0
        mov  zpos+1,a
        ajmp t0_3a

t0_2b:
        clr  c                      ; decrement position
        mov  a,zpos
        subb a,#1
        mov  zpos,a
        
        mov  a,zpos+1
        subb a,#0
        mov  zpos+1,a


t0_3a:


t0_4a:  

;rest of sub is not timing critical - other interrupts can be serviced 
        
        clr et0                 ; disable  timer 0 interrupts
        lcall end_timer0int     ; clear the interrupt-in-progress latch
                                ; the rest of the "ISR" is interruptable
                                ; this bodgery from p5-20 Intel



        clr sendxstep
        clr sendzstep
        

        mov dptr,#timer0_tab
        mov a,r4                ;set function
        jmp @a+dptr             ;do timer function


t0_power:
        jb suspend,t0_nopower

;        mov dptr,#manual1
;        movx a,@dptr
;        mov c,acc.7
;        mov spindle_on,c
;        jnc t0_end
        
        clr  stepper_power      ; switch power on
        mov  dptr,#step_vice    ; step(s) will be sent next interrupt
        mov  a,os_stat
        movx @dptr,a
        jmp  t0_end

t0_nopower:
        clr  spindle_on

t0_end:
        pop dph 
        pop dpl
        pop acc
        pop psw
        reti

        
        
;        org high $ + 1 * 256    ;force table to start on page boundary 




timer0_tab:
        ajmp t0_exit            ;0
        
        ajmp t0_xasap_up        ;2        
        ajmp t0_xasap_flat      ;4
        ajmp t0_xasap_down      ;6
        
        ajmp t0_zasap_up        ;8
        ajmp t0_zasap_flat      ;10
        ajmp t0_zasap_down      ;12

        ajmp t0_stepxz          ;14 - send bit 7    
        ajmp t0_stepxz          ;16 - send bit 6    
        ajmp t0_stepxz          ;18 - send bit 5    
        ajmp t0_stepxz          ;20 - send bit 4    
        ajmp t0_stepxz          ;22 - send bit 3    
        ajmp t0_stepxz          ;24 - send bit 2    
        ajmp t0_stepxz          ;26 - send bit 1    
        ajmp t0_stepxz          ;28 - send bit 0    
        ajmp t0_stepxz_last     ;30 - last call - dummy 
        
        ajmp t0_exit            ;32     - manual - NULL
        ajmp t0_mstepx          ;34     - 32 + 2
        ajmp t0_mstepz          ;36     - 32 + 4
        ajmp t0_mstepxz         ;38     - 32 + 2 + 4

        ajmp t0_stepx_feed      ;40     - step x at feedrate
        ajmp t0_exit            ;42     - step y at feedrate
        ajmp t0_stepz_feed      ;44     - step z at feedrate


        ajmp t0_purge           ;46 sets NULL fuction, after any steps

        ajmp t0_delay           ;48

        ajmp t_z_home           ;50


t0_purge:
        mov tl0_seed,#0
        mov th0_seed,#0
        mov r4,#NULL
        ajmp t0_exit


t0_mstepx:                       ;used for manual control to
        lcall set_sendxstep      ;send pulse stream
        ajmp t0_exit

t0_mstepz:
        lcall set_sendzstep
        ajmp t0_exit

t0_mstepxz:
        lcall set_sendxstep
        lcall set_sendzstep
        ajmp t0_exit


t0_exit:
    jnb want_all,t0_exit_1
    
    cjne r4,#2,t0_ack_1
    ajmp t0_exit_1
t0_ack_1:    
    cjne r4,#4,t0_ack_2
    ajmp t0_exit_1
t0_ack_2:    
    cjne r4,#6,t0_ack_3
    ajmp t0_exit_1
t0_ack_3:
    cjne r4,#8,t0_ack_4
    ajmp t0_exit_1
t0_ack_4:    
    cjne r4,#10,t0_ack_5
    ajmp t0_exit_1
t0_ack_5:    
    cjne r4,#12,t0_ack_6
    ajmp t0_exit_1
t0_ack_6:    
    setb ack_want_all

t0_exit_1:        
        pop dph
        pop dpl
        pop acc
        pop psw
        setb et0
        ret


end_timer0int:                    ; called at end of timer critical section
        reti                      ; clears the interrupt-in-progress flip-flop






t0_zasap_up:                      ; t0_func=8
;expects r0/r1 to hold number of steps to ramp up 
        cjne r0,#0,z_asap3a
        cjne r1,#0,z_asap3b
        
        mov r0,z_flat             ; lsb
        mov r1,z_flat+1           ; msb
        inc r4                    ; set function call for next interrupt
        inc r4
        ajmp t0_zasap_flat

z_asap3a:
        dec r0        
        ajmp z_asap3c
z_asap3b:
        dec r0                  ;will set r0 to ff
        dec r1
z_asap3c:      
        lcall set_sendzstep          ;send step at start of next interrupt
        lcall zramp_to_t0seed   ; read data from table into timer registers

        clr c                       ; point to next word in table    
        mov a,ramp_z
        addc a,#2
        mov ramp_z,a   
        
        mov a,ramp_z+1
        addc a,#0
        mov ramp_z+1,a   
        

        ajmp t0_exit


t0_zasap_flat:                    ;t0_func=10
        cjne r0,#0,z_asap5a
        cjne r1,#0,z_asap5b

        mov r0,z_updown             ;lsb
        mov r1,z_updown+1           ;msb
        inc r4                  ;set function call for next interrupt
        inc r4
        ajmp t0_zasap_down

z_asap5a:
        dec r0
        ajmp z_asap5c
z_asap5b:            
        dec r0
        dec r1
z_asap5c:
        lcall set_sendzstep     ; send step at start of next interrupt
        lcall zramp_to_t0seed   ; read data from table into timer registers
        ajmp t0_exit



t0_zasap_down:
        cjne r0,#0,z_asap7a
        cjne r1,#0,z_asap7b
        
        mov r4,#PURGE            ;set null function
        ajmp t0_exit

z_asap7a:
        dec r0
        ajmp z_asap7c
z_asap7b:        
        dec r0
        dec r1
z_asap7c:      
        lcall set_sendzstep          ; send step at start of next interrupt        
        lcall zramp_to_t0seed   ; read data from table into timer registers

        clr c                   ; point to previous word in table    
        mov a,ramp_z
        subb a,#2
        mov ramp_z,a   
        
        mov a,ramp_z+1
        subb a,#0
        mov ramp_z+1,a   
        
        ajmp t0_exit



t0_xasap_up:        ;2        
        cjne r0,#0,x_asap3a
        cjne r1,#0,x_asap3b
        
        mov r0,x_flat             ;lsb
        mov r1,x_flat+1           ;msb

        inc r4                  ;set next function
        inc r4
        ajmp t0_xasap_flat


x_asap3a:
        dec r0        
        ajmp x_asap3c
x_asap3b:
        dec r0                  ;will set r0 to ff
        dec r1
x_asap3c:      
        lcall set_sendxstep          ; send step at start of next interrupt        
        lcall xramp_to_t0seed    ; read data from table into timer registers

        clr c                       ; point to next word in table    
        mov a,ramp_x
        addc a,#2
        mov ramp_x,a   
        
        mov a,ramp_x+1
        addc a,#0
        mov ramp_x+1,a   
        
        ajmp t0_exit


t0_xasap_flat:      ;4
        cjne r0,#0,x_asap5a
        cjne r1,#0,x_asap5b
        
        mov r0,x_updown             ;lsb
        mov r1,x_updown+1           ;msb

        inc r4                     ;set next function
        inc r4
        ajmp t0_xasap_down

x_asap5a:
        dec r0
        ajmp x_asap5c
x_asap5b:            
        dec r0
        dec r1
x_asap5c:
        lcall set_sendxstep          ; send step at start of next interrupt        
        lcall xramp_to_t0seed   ; read data from table into timer registers
        
        ajmp t0_exit




t0_xasap_down:      ;6
        cjne r0,#0,x_asap7a
        cjne r1,#0,x_asap7b
        
        mov r4,#PURGE             ; set null function     
        ajmp t0_exit

x_asap7a:
        dec r0
        ajmp x_asap7c
x_asap7b:        
        dec r0
        dec r1
x_asap7c:      
        lcall set_sendxstep          ; send step at start of next interrupt                
        lcall xramp_to_t0seed   ; read data from table into timer registers

        clr c                   ; point to previous word in table    
        mov a,ramp_x
        subb a,#2
        mov ramp_x,a   
        
        mov a,ramp_x+1
        subb a,#0
        mov ramp_x+1,a   

        ajmp t0_exit


t0_stepxz:      ;r4=14 to 
                ;look at bits 7 of x_pattern and z_pattern
        mov b,#0
        mov a,x_pattern
        rl a
        mov x_pattern,a

        jnb acc.7,stepxz1
        lcall set_sendxstep
        inc b
stepxz1:        
        mov a,z_pattern
        rl a
        mov z_pattern,a
        
        jnb acc.7,stepxz2
        lcall set_sendzstep
        inc b
stepxz2:
        mov a,b                 ;b will be 0,1 or 2        
        jz stepxz3
        cjne a,#2,stepxz3
        
        mov tl0_seed,feedrate+2 ;both axes moving use 2nd feedrate
        mov th0_seed,feedrate+3
        ajmp stepxz3a

stepxz3:
        mov tl0_seed,feedrate
        mov th0_seed,feedrate+1
stepxz3a:        
        
        
        inc r4
        inc r4          ;set next timer function
        ajmp t0_exit


t0_stepxz_last:         ;final call to XZ steps
        jb sentXstep,t0_stepxz_last1
        jb sentZstep,t0_stepxz_last1
;        mov tl0_seed,#0
;        mov th0_seed,#0
        mov r4,#NULL
t0_stepxz_last1:
        ajmp t0_exit




t0_stepx_feed:      ;40     - step x at feedrate
        cjne r0,#0,xf1
        cjne r1,#0,xf2
        mov r4,#PURGE            ; Done
        ajmp t0_exit     

xf1:
        dec r0
        ajmp xf3
xf2:            
        dec r0
        dec r1
xf3:
        lcall set_sendxstep          ; send step at start of next interrupt
        ajmp t0_exit



t0_stepz_feed:      ;44     - step z at feedrate
        cjne r0,#0,zf1
        cjne r1,#0,zf2
        mov r4,#PURGE         ; Done
        ajmp t0_exit     

zf1:
        dec r0
        ajmp zf3
zf2:            
        dec r0
        dec r1
zf3:
        lcall set_sendzstep          ; send step at start of next interrupt
        ajmp t0_exit






t0_delay:
        cjne r0,#0,t0del1
        mov r4,#NULL
        ajmp t0_exit     

t0del1:
        dec r0
        ajmp t0_exit     



t_z_home:
        ajmp t0_exit     



xramp_to_t0seed:
        mov dph,ramp_x+1
        mov dpl,ramp_x
        
        clr  a
        movc a,@a+dptr          ; read byte from table, msb of timer set value
        mov  th0_seed,a

        inc  dptr

        clr  a
        movc a,@a+dptr          ; read second byte, lsb of timer set value
        mov  tl0_seed,a
        ret


zramp_to_t0seed:
        mov dph,ramp_z+1
        mov dpl,ramp_z
        
        clr  a
        movc a,@a+dptr          ; read byte from table, msb of timer set value
        mov  th0_seed,a

        inc  dptr

        clr  a
        movc a,@a+dptr          ; read second byte, lsb of timer set value
        mov  tl0_seed,a
        ret


set_sendxstep:
        jb   init,sx1           ;If initilaising then set step
        jb   xdir,sx2
        
        mov  a,xpos
        jnz  sx1                 ;not at 0 so set step
        mov  a,xpos+1
        jnz  sx1                 ;not at 0 so set step
        ret
sx2:
        mov a,xpos
        cjne a,xmax,sx1    ;not @ max so set step   #low XMAX
        
        mov a,xpos+1
        cjne a,xmax+1,sx1   ;not @ max so set step  #high XMAX
        ret

sx1:
        setb sendxstep
        mov  power_cnt,#powerlen       ; switch stepper power on
        ret


set_sendzstep:
        jb   init,sz1           ;If initilaising then set step
        jb   zdir,sz2
        
        mov  a,zpos
        jnz  sz1                 ;not at 0 so set step
        mov  a,zpos+1
        jnz  sz1                 ;not at 0 so set step
        ret
sz2:
        mov a,zpos
        cjne a,zmax,sz1    ;not @ max so set step #low ZMAX
        mov a,zpos+1
        cjne a,zmax+1,sz1   ;not @ max so set step #high ZMAX
        ret
sz1:
        setb sendzstep
        mov  power_cnt,#powerlen
        ret
