SUBTTL Constants
TITLE  **  Control Program for 8032 MILL - Version 1.0 3/12/94 **

PLEN 100        ;set page length suitable for Arial 6pt in works

;LOAD ; to send code to PC's serial port

LIST ; to allow 'L' switch at command line


XCALIB  equ 4 * 400 / 3
YCALIB  equ 4 * 400 / 3
ZCALIB  equ 0


NULL        equ 0     ;timer 0/1 function call - set r4/5

XASAP       equ 2     ;timer 0 function call - set r4
ZASAP       equ 8     ;timer 0 function call - set r4

XYSTEP      equ 14    ;timer 0 function call - set r4
XYSTEP_END  equ 30


MAN         equ 32    ;timer 0 function call - set r4

XSTEP       equ 34    ;timer 0 function call - set r4
ZSTEP       equ 36    ;timer 0 function call - set r4
XZSTEP      equ 38    ;timer 0 function call - set r4

XFEED       equ 40
YFEED       equ 42
ZFEED       equ 44

PURGE       equ 46

delay       equ 48


YASAP       equ 2     ;timer 1 function call - set r5
YSTEP       equ 30    ;timer 1 function call - set r5

PURGE_Y     equ 32

PAUSE       equ 6     ; IBM function call
CONTINUE    equ 8     ; IBM function call  



;Feedrates 50mm/min  E89F
;         100mm/min  F457
;         150mm/min  F83F
;         200mm/min  FA33
;         250mm/min  FB5F
;         300mm/min  FC27
;         350mm/min  FCB6
;         400mm/min  FD21
;         450mm/min  FD74
;         500mm/min  FDB7


LOWFEED     equ 0E89Fh  ; =  50 mm/min
MAXFEED     equ 0FDB7h  ; = 500 mm/min



KHz1        equ 0Fbc2h  ; timer2 load value for 1KHz
maxdelay    EQU 0       ; Maximum seed value for timer2 




EPROM       equ 0ffh   ; IBM peek EPROM page
INTERNAL    equ 0feh   ; IBM peek internal RAM page

tableadd    equ 0200h  ; Start address of control table  


powerlen    equ 3


; HARDWARE ADDRESSES


lsblatch        equ     0a000h  ; latch for lsb address
intlatch        equ     0a001h  ; interrupt from IBM, Ext0
datalatch       equ     0a002h  ; 

;p1.0           is      X direction, positive high
;p1.1           is      X step, pulse low
;p1.2           is      Y direction      
;p1.3           is      Y step
;p1.4           is      Z direction
;p1.5           is      Z step

;p3.0           is      clear 
;p3.1           is      led 

step_vice       equ     0c200h  ; stepper motors/motor control  - w
microsws        equ     0c200h  ; micro-switches                - r
spindle         equ     0c600h  ; spindle (cutter) motor        - w
workcell_i      equ     0c600h  ; inputs from workcell          - r
workcell_o      equ     0ca00h  ; outputs to workcell           - w
manual1         equ     0ce00h  ; manual controls to motors     - r
opto_darl       equ     0ce00h  ; opto-isolated and darlingtons - w
manual2         equ     0ca00h  ; manual controls (unused)      - r

datastrobe      equ     0d200h  ; write strobe at RJ45 interface
dataoen         equ     0d600h  ; data out enable



; INTERNAL RAM MAP



rbank0          equ     00h      ; register bank 0 - 
rbank1          equ     08h      ; register bank 1 - Timer 0/1
rbank2          equ     010h     ; register bank 2 - Timer 2/IBM interrupt
rbank3          equ     018h     ; register bank 3  



                                        ;Bit addressable 20-2f
dir_nbits       equ     020h            ; direction of motor commands
dir_nbitsZ      bit     dir_nbits.0     ; Z bit, set positive
dir_nbitsY      bit     dir_nbits.1     ; Y bit
dir_nbitsX      bit     dir_nbits.2     ; X bit

sentxstep       bit     dir_nbits.3
sentystep       bit     dir_nbits.4
sentzstep       bit     dir_nbits.5
sentystep_XY    bit     dir_nbits.6
intused         bit     dir_nbits.7

step_bits       equ     dir_nbits+1
sendxstep       bit     step_bits.0
xdir            bit     step_bits.1
sendystep       bit     step_bits.2
ydir            bit     step_bits.3
sendzstep       bit     step_bits.4
zdir            bit     step_bits.5
sendystep_XY    bit     step_bits.6 
init            bit     step_bits.7 

pagebits        equ     step_bits+1                                        
pagebits0       bit     pagebits.0      ; marker for page @ 200 used/updated
pagebits1       bit     pagebits.1      ;                   300
pagebits2       bit     pagebits.2      ;                   400
pagebits3       bit     pagebits.3      ;                   500
pagebits4       bit     pagebits.4      ;                   600
pagebits5       bit     pagebits.5      ;                   700
pagebits6       bit     pagebits.6      ;                   800
pagebits7       bit     pagebits.7      ;                   900

wco_stat        equ     pagebits+1      ; states of workcell outputs
wco_stat0       bit     wco_stat.0
wco_stat1       bit     wco_stat.1
wco_stat2       bit     wco_stat.2
wco_stat3       bit     wco_stat.3
wco_stat4       bit     wco_stat.4
wco_stat5       bit     wco_stat.5
wco_stat6       bit     wco_stat.6
wco_stat7       bit     wco_stat.7

flags           equ     wco_stat+1
;abort           bit     flags.0         ; set if IBM wants to end table
last_power      bit     flags.1         ; used to force coninue with power switch
want_all        bit     flags.2
ack_want_all    bit     flags.3
ignore_step     bit     flags.4
no_back         bit     flags.5



os_stat          equ     flags+1         ; current states for odds and sods
stepper_reset    bit     os_stat.0
stepper_xy_power bit     os_stat.1
vice_1           bit     os_stat.2
vice_2           bit     os_stat.3
finger_1         bit     os_stat.4
finger_2         bit     os_stat.5
stepper_z_power  bit     os_stat.6


opto_stat       equ     os_stat+1       ; current states for opto/darlingtons

SpinDir         bit     opto_stat.4
air             bit     opto_stat.5



executing       equ     opto_stat+1     ; read only byte for IBM 
exe_table       bit     executing.0     ; executing table  
exe_pause       bit     executing.1     ; paused executing table  
exe_home        bit     executing.2     ; homing
suspend         bit     executing.3
abort           bit     executing.4
dooropen        bit     executing.5
calibration     bit     executing.6
spindle_on      bit     executing.7     ; spindle power on/off
                                        ; will ignore continue until power is on


ramp_x          equ     executing+1     ; pointers to acceleration ramp table
x_updown        equ     ramp_x+2        ; x and z at same address
x_flat          equ     x_updown+2
x_pattern       equ     x_flat+2
tl0_seed        equ     x_pattern+1
th0_seed        equ     tl0_seed+1

ramp_z          equ     ramp_x
z_updown        equ     x_updown
z_flat          equ     x_flat


ramp_y          equ     th0_seed+1
y_updown        equ     ramp_y+2
y_flat          equ     y_updown+2    
y_pattern       equ     y_flat+2
tl1_seed        equ     y_pattern+1    
th1_seed        equ     tl1_seed+1


control_byte    equ     th1_seed+1      ; current control byte

led_cnt         equ     control_byte+1
led_cnt_seed    equ     led_cnt+1



feedrate        equ     led_cnt_seed+1      ; feed rates set from miscellaneous

power_xy_cnt      equ     feedrate+4
power_z_cnt       equ     power_xy_cnt+1


x_backlash      equ power_z_cnt+1

y_backlash      equ x_backlash+1
        
z_backlash      equ y_backlash+1

r4_seed         equ z_backlash+2

t2_intcnt       equ r4_seed+1

Door_cnt        equ t2_intcnt+1
;os_stat_copy    equ Door_cnt+1
;opto_stat_copy  equ os_stat_copy+1

mcntrl          equ     050H            ; 
wcntrl          equ     mcntrl+1        ; workcell control
scntrl          equ     wcntrl+1
Xpos            equ     scntrl+1        ; current X position from home
Ypos            equ     Xpos+2
Zpos            equ     Ypos+2
curr_line       equ     Zpos+2          ; 'miscellaneous' statement of current line
rs232_mess      equ     curr_line+2     ; Byte to send as "RS232" via workcell
pulses_no       equ     rs232_mess+1    ; number of workcell pulses to be sent
pulses_rate     equ     pulses_no+2     ; rate to send workcell pulses
jog_feed_rate   equ     pulses_rate+2   ; feed rate set direct   
spindle_speed   equ     jog_feed_rate+2 ; spindle motor speed set direct


table_marker    equ     spindle_speed+1 ; control pages used by 8031, bitmap
update_marker   equ     table_marker+1  ; bitmap, control pages IBM updated 
spare           equ     update_marker+1 



mcntrl_copy     equ     spare+2     ; master control byte returned

x_calib         equ     mcntrl_copy+1
y_calib         equ     x_calib+2
z_calib         equ     y_calib+2


jogcnt          equ     z_calib+2


timer2_func     equ     jogcnt+2
wco_low         equ     timer2_func+1
wco_high        equ     wco_low +1
wco_next        equ     wco_high  +1
pulse_width     equ     wco_next +1




scratch         equ     080H        ; only indirect addressable

stack           equ     090H



; EXTERNAL RAM MAP


shared          equ     0h          ; shared for IBM communications
prom_idsp       equ     shared      ; identification copied from PROM
version         equ     prom_idsp+4 ; version copied from PROM
copyright       equ     version+4
csum            equ     copyright+64 


        



workspace       equ     100h

off_sets        equ     workspace   ; XYZ offsets from miscellaneous
xyz_steps       equ     off_sets+6  ; asap moves
ret_addr        equ     xyz_steps+6 ; return address from misc functions


; Mill mechanical constants

XMAX equ 27093  ;(400*203.2/3)  nominal 200mm, Americans prefer 8 inches
YMAX equ 13547  ;(400*101.6/3) 
ZMAX equ 10000  ;6773   ;(400*50.8/3) 
 
;ZMAX  equ 13547
;YFLAT equ YMAX-$800



; PROGRAM MEMORY MAP

PAGE
SUBTTL Interrupt Vectors

;base            equ     0a000h  ; alternatives for RAM and ROM versions
base           equ     0h

ORG     base
PAD     0ffh                    ; to fill unallocated locations


; SET-UP INTERRUPT VECTORS AT START OF PROGRAM SPACE


ORG     base+0h                 ; reset vector address
        jmp     begin           ; start of initialisation

ORG     base+03h                ; external interrupt 0 vector address
        jmp     ibmint          ; used for IBM communications

ORG     base+0bh                ; timer0 interrupt vector
        jmp     timer0

ORG     base+13h                ; external interrupt 1 vector address
        reti                    ; may use for software interrupts

ORG     base+1bh                ; timer1 interrupt vector
        jmp     timer1

ORG     base+23h                ; serial port vector address
        reti

org     base+2bh                ; timer2 interrupt vector
        jmp     timer2

ORG     base+30h                ; to be tidy


; PROM IDENTIFICATION

prom_id:

;        db      "MILL","1.00"     ; mill program, version 1
;        db      "18/12/94 (C) Cybernetic Applications 1994",0    ; Null terminated

;        db      "MILL","1.01"     ; mill program, version 1.01
;        db      "19/04/95 (C) Cybernetic Applications 1994-5",0    ; Null terminated

        db      "MILL","1.02"     ; mill program, version 1.02
        db      "01/11/96 (C) Cybernetic Applications 1994-6",0    ; Null terminated



PAGE
SUBTTL INITIALISATION / RESET

;nb. uses register set 0, as does Baseloop

begin:
        mov  sp,#stack          ; initialise stack pointer 
        mov  psw,#0             ; set register set 0

        mov  dptr,#intlatch     ; clear any interrupt states
        movx a,@dptr





;        mov  dptr,#step_vice    ; reset stepper motor controller chip
;ramp:
;        movx @dptr,a
;        inc acc
;        jmp ramp

        
        clr  a
        lcall set_spindle_speed ; - switch off spindle
        
        mov r0,#255
clear_internalram:
        mov @r0,a
        dec r0
        cjne r0,#8,clear_internalram
        
        
;        mov dptr,#0             ; cosmetic - added 12/11/94 
;        clr a                   ; Clear all RAM (32K) to 0
;        mov r0,#0
;        mov r1,#80h
;clearram:
;        movx @dptr,a
;        inc dptr
;        djnz r0,clearram
;        djnz r1,clearram



; Set-up any values        

        mov jog_feed_rate,#LOW MAXFEED      ; set default jog rate
        mov jog_feed_rate+1,#HIGH MAXFEED

        mov pulses_rate  ,#low  KHz1         ; set default pulse rate
        mov pulses_rate+1,#high KHz1

        
        mov  dptr,#step_vice    ; reset stepper motor controller chip
        mov  os_stat,#0ffh
        
        ;setb stepper_reset      ; Stepper reset high
        ;setb stepper_xy_power   ; power save
        ;setb stepper_z_power    ; power save


        mov  a,os_stat
        movx @dptr,a            ; Write to control port

        mov r0,#10
        djnz r0,$
        
        clr  stepper_reset      ; Stepper reset low
        mov  a,os_stat
        movx @dptr,a            ; Write to control port
        
        mov r0,#10
        djnz r0,$
        
        setb stepper_reset      ; Stepper reset high
        mov  a,os_stat
        movx @dptr,a            ; Write to control port

        mov  opto_stat,#0
        mov mcntrl,#0           ; clear master control byte
        mov update_marker,#0

        mov table_marker,#0ffh  ; mark all table pages used/invalid
        mov  wco_stat,#0ffh     ; Set all workcell outputs high
        
        mov  a,wco_stat         ; send to workcell outputs
        mov  dptr,#workcell_o
        movx @dptr,a            


        mov a,#15               ; set default number of backlash
        mov x_backlash,a        ; compensation steps
        mov y_backlash,a
        mov z_backlash,a


        mov x_calib,  #low  XCALIB    ; set default calibration steps
        mov x_calib+1,#high XCALIB
        
        mov y_calib,  #low  YCALIB
        mov y_calib+1,#high YCALIB
        
        mov z_calib,  #low  ZCALIB
        mov z_calib+1,#high ZCALIB


; Copy Identification for IBM to read

                                ; Copy Null terminated PROM id to RAM
        mov  dptr,#prom_id      ; Start address in PROM
        mov  r0,#prom_idsp      ; Destination address in RAM
copy1:
        clr a
        movc a,@a+dptr   
;        jz copy2                ; found null
        
        mov p2,#0               ; reset p2 
        movx @r0,a
        
        inc dptr
        inc r0
;        ajmp copy1 
        jnz copy1 


copy2:


        mov dptr,#0         ; Calculate EPROM checksum
        mov r0,#0
        mov r1,#0
gc1:
        clr a
        movc a,@a+dptr

        add a,r0
        xch a,r0

        clr a
        addc a,r1
        xch a,r1

        inc dptr
        mov a,dph
        cjne a,#20h,gc1     ; done all EPROM?

        mov dptr,#csum      ; save for IBM
        mov a,r0
        movx @dptr,a

        inc dptr
        mov a,r1
        movx @dptr,a



; Set-up Interrupts and their Priorities

        
        mov wcntrl,#0
        mov timer2_func,#NULL

        mov led_cnt,#8          ; set led flash freq    
        mov led_cnt_seed,#8
        mov rcap2l,#low  maxdelay
        mov rcap2h,#high maxdelay

        mov t2con,#00000100b    ; timer2, 16bit autoreload timer
        
        mov  tmod,#00010001b    ; timer0, timer1 both 16 bit timers
        mov  tcon,#00000100b    ; ext0 state, ext1 edge triggered
        mov  ie  ,#10101011b    ; enable ext0, timer0,timer1, and timer2  
        mov  ip  ,#00001011b    ; ext0, timer0,timer1 high priority




; Move Mill to HOME position
        
        setb calibration        ; request calibration data
        
        lcall home              ; sets X,Y and X positions to zero
        
        mov  executing,#0       ; Show IBM we're ready to do work

        ;mov  mcntrl,#0


    page
    SUBTTL Baseloop


BASELOOP:
        mov  a,mcntrl           ; read master control byte in shared memory
        jz   toggleled          ; nothing else to do

        push acc
        clr  a
        mov  mcntrl,a           ; clear master control byte

        mov  dptr,#ret_addr     ; set up return address
        mov  a,#low set_slave
        movx @dptr,a
        
        inc  dptr
        mov  a,#high set_slave
        movx @dptr,a

        pop  acc
        mov  mcntrl_copy,a      ; record master control byte to return
        
        jb   acc.0,m_error      ; control byte should be even
        jb   acc.7,m_error      ; and less than 128
        
        mov  dptr,#master_decode
        jmp  @a+dptr            ; decode master control byte via jump table 
                                ; returns are all indirect ajmp SET_SLAVE
        
m_error:                        ; could maybe add counter for number of errors caught           
        
set_slave:
        mov a,mcntrl_copy       ; retrieve master control byte, work done
        mov scntrl,a            ; and set to slave control byte - protocol
        

toggleled:
        
        mov  dptr,#manual1      ; read manual control buttons
        movx a,@dptr
        mov c,acc.7
        mov spindle_on,c


        setb acc.7              ; mask OFF switch
        cpl  a                  ; for normal logic
        
        jz   baseloop
        
        lcall manual_control    ; button(s) have been pressed service
        
        ajmp  baseloop


        org high $ + 1 * 256    ; not needed but makes addresses tidy

master_decode:                  

        ajmp toggleled      ;0  should never be obeyed, but need to fill
        
        ajmp m_dowork       ;2  Execute      
        ajmp set_slave      ;4  abort - not executing - so ignore
        ajmp set_slave      ;6  Pause - not executing - so ignore
        ajmp set_slave      ;8  continue - not executing - so ignore

        ajmp m_stepx_h      ;10 move axis to and from home at jog feedrate
        ajmp m_stepx_a      ;12
        ajmp m_stepy_h      ;14
        ajmp m_stepy_a      ;16
        ajmp m_stepz_h      ;18
        ajmp m_stepz_a      ;20
        ajmp set_slave      ;22 other axes, such as for robots
        ajmp set_slave      ;24
        ajmp set_slave      ;26
        ajmp set_slave      ;28
        ajmp set_slave      ;30
        ajmp set_slave      ;32
        ajmp set_slave      ;34
        ajmp set_slave      ;36
        ajmp set_slave      ;38
        ajmp set_slave      ;40

        ajmp m_cancel       ;42 cancels functions 10 to 40
       
        ajmp m_open_vice    ;44
        ajmp m_close_vice   ;46
        
        ajmp m_spindle_cw   ;48
        ajmp m_spindle_ccw  ;50
        
        ajmp m_home         ;52
        ajmp m_reset        ;54 this WILL NOT RETURN TO set_slave

        ajmp m_hardhome     ;56
        ajmp m_sethome      ;58
        
        ajmp m_jogx_p       ;60
        ajmp m_jogy_p       ;62
        ajmp m_jogz_p       ;64
        
        ajmp m_jogx_m       ;66
        ajmp m_jogy_m       ;68
        ajmp m_jogz_m       ;70
        
        
        ajmp m_error        ;70 rest of list should never be called
        ajmp m_error        ;72
        ajmp m_error        ;74
        ajmp m_error        ;76
        ajmp m_error        ;78
        ajmp m_error        ;80
        ajmp m_error        ;82
        ajmp m_error        ;84
        ajmp m_error        ;86
        ajmp m_error        ;88
        ajmp m_error        ;90
        ajmp m_error        ;92
        ajmp m_error        ;94
        ajmp m_error        ;96
        ajmp m_error        ;98
        ajmp m_error        ;100
        ajmp m_error        ;102
        ajmp m_error        ;104
        ajmp m_error        ;106
        ajmp m_error        ;108
        ajmp m_error        ;110
        ajmp m_error        ;112
        ajmp m_error        ;114
        ajmp m_error        ;116
        ajmp m_error        ;118
        ajmp m_error        ;120
        ajmp m_error        ;122
        ajmp m_error        ;124
        ajmp m_error        ;126
        







        page
        SUBTTL Obey IBM

        ;Called when IBM wants something done whilst processing table
        ;returns to table when continue or abort requested


setup_obey_ibm:
        cjne r4,#NULL,$

        push psw                ; save current register set
        push acc                ; Acc = IBM command 
        
        clr  rs0
        setb rs1                ; set register bank 2       
        
        mov r4,#NULL            
        mov r5,#NULL

        setb exe_pause          ; Show IBM we're paused
        mov led_cnt_seed,#4     ; Show the world by flashing LED at double speed


        mov dptr,#ret_addr      ; set up return address
        mov a,#low dummy        ; in pause state
        movx @dptr,a
        inc dptr
        mov a,#high dummy
        movx @dptr,a
        pop acc                 ; recover IBM command

        setb last_power
        
        lcall obey_ibm

        clr exe_pause           ; Show IBM we're not paused
        mov led_cnt_seed,#8


        pop psw                 ; recover register set
        ret



dummy:                          ; re-entry
        
        mov  dptr,#manual1      ; read manual control buttons
        movx a,@dptr

        mov c,acc.7
        mov spindle_on,c

        jb   spindle_on,dummy2  ; look at OFF switch - will be 1 if power on
        clr  last_power         ; power is off this time
        jmp  dummy3

dummy2:
        jb last_power,dummy3    ; power if on this time and was last time

        mov a,#continue         ; power has just been switched on - continue
        jmp dummy4

dummy3:
        setb acc.7              ; mask OFF switch
        cpl  a                  ; for normal logic
        
        jz   dummy1
        
        lcall manual_control    ; button(s) have been pressed service
        
dummy1:
        mov a,mcntrl            ; get next IBM order

obey_ibm:
        
        mov  mcntrl,#0           ; clear master control byte

dummy4:       
        
        mov  mcntrl_copy,a       ; record master control byte to return
        jb   acc.0,ibm_error     ; control byte should be even
        jb   acc.7,ibm_error     ; <128 else its an error   
        
        mov  dptr,#ibm_decode
        jmp  @a+dptr            ; decode master control byte via jump table 
                                ; RETURNS ARE by indirect ajmp dummy
ibm_error:
        ajmp dummy


        org high $ + 1 * 256    ; not needed but makes addresses tidy

ibm_decode:                     
        ajmp dummy          ;0
        ajmp dummy          ;2
        ajmp ibm_abort      ;4  abort set flag and return
        ajmp dummy          ;6  pause - already are
        ajmp ibm_cont       ;8  continue
                        
        ajmp m_stepx_h      ;10 move axis to and from home at jog feedrate
        ajmp m_stepx_a      ;12
        ajmp m_stepy_h      ;14
        ajmp m_stepy_a      ;16
        ajmp m_stepz_h      ;18
        ajmp m_stepz_a      ;20
        ajmp dummy          ;22
        ajmp dummy          ;24
        ajmp dummy          ;26
        ajmp dummy          ;28
        ajmp dummy          ;30
        ajmp dummy          ;32
        ajmp dummy          ;34
        ajmp dummy          ;36
        ajmp dummy          ;38
        ajmp dummy          ;40
        ajmp dummy          ;42 cancels functions 10 to 40
       
        ajmp m_open_vice    ;44
        ajmp m_close_vice   ;46

        ajmp m_spindle_cw   ;48
        ajmp m_spindle_ccw  ;50 
        
        ajmp m_home         ;52

        ajmp m_reset        ;54 this WILL NOT RETURN TO tab_return

        ajmp m_hardhome     ;56
        ajmp m_sethome      ;58
        
        ajmp m_jogx_p       ;60
        ajmp m_jogy_p       ;62
        ajmp m_jogz_p       ;64
        
        ajmp m_jogx_m       ;66
        ajmp m_jogy_m       ;68
        ajmp m_jogz_m       ;70
        
        
        ajmp ibm_error        ;72
        ajmp ibm_error        ;74
        ajmp ibm_error        ;76
        ajmp ibm_error        ;78
        ajmp ibm_error        ;80
        ajmp ibm_error        ;82
        ajmp ibm_error        ;84
        ajmp ibm_error        ;86
        ajmp ibm_error        ;88
        ajmp ibm_error        ;90
        ajmp ibm_error        ;92
        ajmp ibm_error        ;94
        ajmp ibm_error        ;96
        ajmp ibm_error        ;98
        ajmp ibm_error        ;100
        ajmp ibm_error        ;102
        ajmp ibm_error        ;104
        ajmp ibm_error        ;106
        ajmp ibm_error        ;108
        ajmp ibm_error        ;110
        ajmp ibm_error        ;112
        ajmp ibm_error        ;114
        ajmp ibm_error        ;116
        ajmp ibm_error        ;118
        ajmp ibm_error        ;120
        ajmp ibm_error        ;122
        ajmp ibm_error        ;124
        ajmp ibm_error        ;126
        



ibm_abort:
        setb abort
        
        clr  a
        lcall set_spindle_speed ; - switch off spindle
        clr air
        Setb vice_1
        Setb vice_2
        Setb Finger_1
        Setb Finger_2

ibm_cont:
        jb spindle_on,ibm_cont_1   ; don't continue if power is off    
        ajmp dummy
ibm_cont_1:
        ret


tab_return:
        setb  no_back           ; don't send backlash compensation
        lcall setdirns          ; reset direction bits    
        clr   no_back
        
        mov dptr,#ret_addr      ; retrieve return address
        movx a,@dptr             
        push acc
        inc dptr
        movx a,@dptr
        mov dph,a
        pop dpl
        clr a
        jmp @a+dptr         ; jump to return address



    page
    SUBTTL Baseloop/Obey IBM Functions

; When called via baseloop will be using register bank 0
; When called via Obey IBM will be using register bank 2


m_dowork:
        mov   scntrl,a
        setb  exe_table       ; Show IBM we're doing work
        clr   exe_pause
        clr   abort             
        
        lcall dowork
        
        ;mov   executing,#0   ; Show IBM we're done work
        clr   exe_table
        clr   exe_pause

        ajmp   set_slave     ; This function always returns to baseloop
                            ; because its only ever called from there
                            


m_sethome:      ;58
        mov x_calib,  xpos
        mov x_calib+1,xpos+1
        
        mov y_calib,  ypos
        mov y_calib+1,ypos+1
        
        mov z_calib,  zpos
        mov z_calib+1,zpos+1

        ajmp m_home


m_hardhome:     ;56
        mov x_calib,  #0
        mov x_calib+1,#0
        
        mov y_calib,  #0
        mov y_calib+1,#0
        
        mov z_calib,  #0
        mov z_calib+1,#0

m_home:
        lcall home
        ajmp   tab_return


m_reset:
        mov scntrl,a
        ajmp begin

m_jogx_p:
    setb xdir               ; x+ve for sums
    setb p1.0               ; x+ve in hardware
    ajmp m_jogx

m_jogx_m:
    clr xdir               ; x-ve for sums
    clr p1.0               ; x-ve in hardware

m_jogx:
    mov  tl0_seed,#low  maxfeed
    mov  th0_seed,#high maxfeed 
    mov  r0,jogcnt
    mov  r1,jogcnt+1
    mov  r4,#XFEED
    cjne r4,#NULL,$
    ajmp tab_return

m_jogy_p:
    setb ydir               ; y+ve for sums
    clr  p1.2               ; y+ve in hardware
    ajmp m_jogy

m_jogy_m:
    clr  ydir               ; y-ve for sums
    setb p1.2               ; y-ve in hardware

m_jogy:
    mov  tl0_seed,#low  maxfeed
    mov  th0_seed,#high maxfeed 
    mov  r0,jogcnt
    mov  r1,jogcnt+1
    mov  r4,#YFEED
    cjne r4,#NULL,$
    ajmp tab_return

m_jogz_p:
    setb zdir               ; z+ve for sums
    clr  p1.4               ; z+ve in hardware
    ajmp m_jogz

m_jogz_m:
    clr  zdir               ; z-ve for sums
    setb p1.4               ; z-ve in hardware

m_jogz:
    mov  tl0_seed,#low  maxfeed 
    mov  th0_seed,#high maxfeed
    mov  r0,jogcnt
    mov  r1,jogcnt+1
    mov  r4,#ZFEED
    cjne r4,#NULL,$
    ajmp tab_return



m_jog:
        mov tl0_seed,jog_feed_rate 
        mov th0_seed,jog_feed_rate+1 


m_jog1:        
        mov a,mcntrl            ; jogging stopped by another command ?
        jnz m_jog_exit  
        cjne r4,#NULL,m_jog1    ; jogging stopped by limit ?

m_jog_exit:

        mov r4,#PURGE
        cjne r4,#NULL,$
        
        ajmp  tab_return         ; at end of travel or stopped by IBM


m_stepx_h:                      ; move x axis to home at jog feedrate
        mov scntrl,a

        mov r0,xpos             ; set max number of steps to send
        mov r1,xpos+1

        clr  xdir               ; x-ve for sums
        clr  p1.0               ; x-ve in hardware

        cjne r0,#0,m_stepx_h1
        cjne r1,#0,m_stepx_h1
        
        ajmp  tab_return         ; at end of travel

m_stepx_h1:
        
        mov r4,#XFEED           ; timer 0 function call
        ajmp  m_jog              ; setup timer to jog feedrate


m_stepx_a:                      ; move x axis from home at jog feedrate
        
        mov scntrl,a
        
        clr c                   ; set max number of steps to send
        mov  a,xpos
        subb a,#low XMAX
        mov  r0,a
        
        mov  a,xpos+1
        subb a,#high XMAX
        mov  r1,a

        setb xdir               ; x+ve for sums
        setb p1.0                ; x+ve in hardware

        cjne r0,#0,m_stepx_h1
        cjne r1,#0,m_stepx_h1
        
        ajmp  tab_return         ; at end of travel
        


m_stepy_h:                      ; move x axis to home at jog feedrate
        mov scntrl,a
        
        mov r0,ypos             ; set maximum steps to send
        mov r1,ypos+1
        
        clr  ydir                    ; y-ve for sums   
        setb p1.2                    ; y-ve in hardware

        cjne r0,#0,m_stepy_h1
        cjne r1,#0,m_stepy_h1

        ajmp  tab_return         ; at end of travel

m_stepy_h1:
        
        mov r4,#YFEED           ; timer 0 function call
        ajmp  m_jog              ; setup timer to jog feedrate


m_stepy_a:                      ; move y axis from home at jog feedrate
        
        mov scntrl,a
        
        clr  c                  ; set maximum steps to send
        mov  a,ypos
        subb a,#low YMAX
        mov  r0,a
        
        mov  a,ypos+1
        subb a,#high YMAX
        mov  r1,a

        setb ydir                   ; y+ve for sums
        clr  p1.2                   ; y+ve in hardware


        cjne r0,#0,m_stepy_h1
        cjne r1,#0,m_stepy_h1
        
        ajmp  tab_return         ; at end of travel



m_stepz_h:                      ; move x axis to home at jog feedrate
        mov scntrl,a

        mov r0,zpos             ; set maximum steps to send
        mov r1,zpos+1

        clr  zdir                   ; z-ve for sums
        setb p1.4                   ; z-ve in hardware

        cjne r0,#0,m_stepz_h1
        cjne r1,#0,m_stepz_h1
        
        ajmp  tab_return         ; at end of travel

m_stepz_h1:
        
        mov r4,#ZFEED           ; timer 0 function call
        ajmp  m_jog              ; setup timer to jog feedrate


m_stepz_a:                      ; move y axis from home at jog feedrate
        
        mov scntrl,a
        
        clr  c                  ; set maximum steps to send
        mov  a,zpos
        subb a,#low ZMAX
        mov  r0,a
        
        mov  a,zpos+1
        subb a,#high ZMAX
        mov  r1,a


        setb zdir                   ; z+ve for sums
        clr p1.4                    ; z+ve in hardware

        cjne r0,#0,m_stepz_h1
        cjne r1,#0,m_stepz_h1

        ajmp  tab_return         ; at end of travel


        
m_cancel:           ; cancels functions 10 to 40
        mov scntrl,a

        ajmp  tab_return


m_open_vice:
        mov scntrl,a

        lcall open_vice_finger

        ajmp  tab_return


m_close_vice:
        mov scntrl,a

        lcall close_vice_finger

        ajmp  tab_return


m_spindle_cw:
        mov scntrl,a
        
        lcall spindle_cw

        ajmp  tab_return

m_spindle_ccw:
        mov scntrl,a

        lcall spindle_ccw

        ajmp  tab_return
    
    page
    subttl Control Table Interpreter
; OBEY THE INSTRUCTIONS IN THE CONTROL TABLE.  nb. uses register bank 0, also

dowork: 
        mov dptr,#tableadd      ; dptr now has where to start decoding
        mov r6,dph              ; r6,r7 BOOKED !
        mov r7,dpl
        ajmp  decode_loop1

                                ; Decode Control byte from table
decode_loop:                    ; return here after processed control byte

        lcall inctable_a        ; point to next control byte
decode_loop1:

        movx a,@dptr            ; read control byte from table
        mov  control_byte,a     ; preserve it                    
        
        jz  decode_loop1a       ; got more XY data 
        cjne r4,#NULL,$         ; not XY data so make sure previous finished

decode_loop1a:        
        
;        mov dptr,#microsws
;        movx a,@dptr
;        jb acc.3,door1
        
        
        mov  dptr,#manual1      ; read manual buttons, to see if intervention
        movx a,@dptr
        mov c,acc.7
        mov spindle_on,c
        cpl  a                  ; for normal logic
        jz   decode0            ; no intervention

door1:
        mov a,#pause            ; pause function call 
        lcall setup_obey_ibm    ; button(s) pressed service

decode0:
        mov a,mcntrl            ; has there been an abort, meanwhile ?
        jz decode00

        lcall setup_obey_ibm    ; IBM wants something - do it

decode00:
        jb abort,decode01       ; Table was aborted by IBM
        
        mov  a,control_byte     ; retreive control byte - resumption entry

        cjne a,#0ffh,decode1
        
decode01:        
        ret                     ; return to base-loop, finished table

decode1:

        jb   acc.7,decode2
        ajmp  decode3

decode2:
        cjne a,#224,decode2b
        ajmp miscel             ; decode miscellaneous group of commands

decode2b:
        
        cjne a,#192,decode2c

        lcall home              ; return Mill to HOME position
        
        ajmp  decode_loop


decode2c:
        cjne a,#136,decode2d

        lcall inctable_a
        movx a,@dptr            ; read delay in seconds from table

        lcall delay_com         ; invoke a delay
        
        ajmp  decode_loop

decode2d:
        anl  a,#07h             ; extract XYZ direction bits
        mov  dir_nbits,a        ; and save them for use
        
        lcall setdirns
        
        ajmp  decode_loop




decode3:
        jb   acc.6,movez
        jb   acc.4,movexyz

movexy:
        ajmp  movexy_s           ; All these must return to decode_loop
movexyz:
        ajmp movexyz_s
movez:
        jb   acc.0,movex        ; introduced to replace facemill
        jb   acc.1,movey

        ajmp movez_s

movex:
        ajmp movex_s
movey:
        ajmp movey_s

; Decode mumber of steps to move X and Y in a pattern, and do that



movexy_s:
        
        lcall inctable_a
        
        movx a,@dptr            ; byte has pattern for X steps
        mov  x_pattern,a        ; save it for rotation

        lcall inctable_a

        movx a,@dptr            ; followed by pattern for Y steps
        mov  y_pattern,a


        mov r4,#XYSTEP          ; timer 0 function call
        
mxy0:                            ; wait til timer interrupts finished
        cjne r4,#XYSTEP_END,mxy1
        ajmp  decode_loop
mxy1:
        cjne r4,#NULL,mxy0      
        ajmp  decode_loop        ; do next control byte decode



    page
    SUBTTL ASAP Routines

; The asap routines for large movements
z_asap:

; calculate the flat part, if any, after the acceleration transient 


        mov  dptr,#xyz_steps+5
        movx a,@dptr            ; read msb of move
        clr c
        subb a,#8               ; subtract 800h steps for ramp
        jnc  z_asap1            
                                ; all steps will be in ramp
        clr c
        movx a,@dptr            ; re-read msb of move
        rrc  a                  ; divide msb by 2, carry has lsbit
        mov z_updown+1,a
        

        mov  dptr,#xyz_steps+4
        movx a,@dptr            ; read lsb of move
        rrc  a                  ; divide lsb by 2 - carry may be set
        mov  z_updown,a
        
        mov  z_flat,#0
        mov  z_flat+1,#0
        jnc  z_asap2            
        mov  z_flat,#1
        
        ajmp  z_asap2            

z_asap1:
        mov  z_flat+1,a
        mov  dptr,#xyz_steps+4
        movx a,@dptr            ; read lsb of move
        mov  z_flat,a

        mov z_updown,#0
        mov z_updown+1,#4
        

z_asap2:          
        
        mov ramp_z,#low ramptable       ; initialise pointer to acceleration table
        mov ramp_z+1,#high ramptable

        mov r0,z_updown             ;lsb
        mov r1,z_updown+1           ;msb
        
        mov r4,#ZASAP           ; timer 0 function call
        cjne r4,#NULL,$         ; wait til timer interrupts finished

        ret



; Move x and y "simultaneously" asap

xy_asap:
        mov  dptr,#xyz_steps+1
        movx a,@dptr            ; read msb of move
        clr c
        subb a,#8               ; subtract 800h steps for ramp
        jnc  xy_asap1            
                                ; all steps will be in ramp
        clr c
        movx a,@dptr            ; re-read msb of move
        rrc  a                  ; divide msb by 2, carry has lsbit
        mov x_updown+1,a
        

        mov  dptr,#xyz_steps
        movx a,@dptr            ; read lsb of move
        rrc  a                  ; divide lsb by 2 - carry may be set
        mov  x_updown,a
        
        mov  x_flat,#0
        mov  x_flat+1,#0
        jnc  xy_asap2            
        
        mov  x_flat,#1
        ajmp  xy_asap2            

xy_asap1:
        mov  x_flat+1,a
        mov  dptr,#xyz_steps
        movx a,@dptr            ; read lsb of move
        mov  x_flat,a

        mov x_updown,#0
        mov x_updown+1,#4

xy_asap2:
        
        mov  dptr,#xyz_steps+3
        movx a,@dptr            ; read msb of move
        clr c
        subb a,#8               ; subtract 800h steps for ramp
        jnc  xy_asap3            
                                ; all steps will be in ramp
        clr c
        movx a,@dptr            ; re-read msb of move
        rrc  a                  ; divide msb by 2, carry has lsbit
        mov y_updown+1,a
        

        mov  dptr,#xyz_steps+2
        movx a,@dptr            ; read lsb of move
        rrc  a                  ; divide lsb by 2 - carry may be set
        mov  y_updown,a
        
        mov  y_flat,#0
        mov  y_flat+1,#0
        jnc  xy_asap4            

        mov  y_flat,#1
        ajmp  xy_asap4            

xy_asap3:
        mov  y_flat+1,a
        mov  dptr,#xyz_steps+2
        movx a,@dptr            ; read lsb of move
        mov  y_flat,a

        mov y_updown,#0
        mov y_updown+1,#4

xy_asap4:

        mov r0,x_updown             ;lsb
        mov r1,x_updown+1           ;msb
        
        mov r2,y_updown             ;lsb
        mov r3,y_updown+1           ;msb

        mov ramp_x,#low ramptable       ; initialise pointer to acceleration table
        mov ramp_x+1,#high ramptable
        
        mov ramp_y,#low ramptable       ; initialise pointer to acceleration table
        mov ramp_y+1,#high ramptable
        
        jb want_all,$               ; wait for timer 2 
        
        mov  r4,#XASAP              ; timer 0 function 
        mov  r5,#YASAP              ; timer 1 function 

        cjne r4,#NULL,$                ; wait for timer 0                   
        cjne r5,#NULL,$                ; wait for timer 1                 

        ret












; move ASAP the x y z number of steps
; move z independent of x and y (only two 16 bit timers)
; move z first if negative, or up; otherwise do xy first

movexyz_s:
        
        lcall inctable_a        ; copy data from control table
        movx a,@dptr            ; x,y and z, lsb then msb
        mov  dptr,#xyz_steps
        movx @dptr,a

        lcall inctable_a
        movx a,@dptr
        mov  dptr,#xyz_steps+1
        movx @dptr,a

        lcall inctable_a
        movx a,@dptr
        mov  dptr,#xyz_steps+2
        movx @dptr,a
        
        lcall inctable_a
        movx a,@dptr
        mov  dptr,#xyz_steps+3
        movx @dptr,a

        lcall inctable_a
        movx a,@dptr
        mov  dptr,#xyz_steps+4
        movx @dptr,a

        lcall inctable_a
        movx a,@dptr
        mov  dptr,#xyz_steps+5
        movx @dptr,a
        
        jnb  dir_nbitsZ,zfirst

        lcall xy_asap          
        
        lcall z_asap  

        ajmp  xyz_done

zfirst:
        
        lcall z_asap
        
        lcall xy_asap

xyz_done:
        
        ajmp  decode_loop


; move single axis at given feed rate
move_feed:
        
        lcall inctable_a
        
        movx a,@dptr            ; read lsb of number of steps
        mov  r0,a               ; into r0

        lcall inctable_a
        
        movx a,@dptr            ; read msb of number of steps
        mov  r1,a               ; into r1

        
        mov  tl0_seed,feedrate
        mov  tl0,feedrate
        
        mov  th0_seed,feedrate+1
        mov  th0,feedrate+1

        mov r4,r4_seed

move_f1:
        mov a,mcntrl
        jnz move_f2

        mov  dptr,#manual1      ; read manual buttons, to see if intervention
        movx a,@dptr
        mov c,acc.7
        mov spindle_on,c


        setb acc.7              ;
        cpl  a                  ; for normal logic
        jz  move_f3
        mov a,#pause            ; set pause state

move_f2:
        
        mov r4_seed,r4          ; save current function call
        
        mov r4,#PURGE           ; pause current move
        
        
        lcall setup_obey_ibm    ; IBM wants something - do it
        jb abort,move_f4        ; IBM says abort

        mov r4,r4_seed

        mov  tl0_seed,feedrate
        mov  tl0,feedrate
        
        mov  th0_seed,feedrate+1
        mov  th0,feedrate+1

        
move_f3:
        cjne r4,#NULL,move_f1      ;wait til timer interrupts finished

move_f4:        
        ajmp  decode_loop


; Z axis move
movez_s:
        mov r4_seed,#ZFEED           ; timer 0 function call
        ajmp move_feed


; X axis move

movex_s:
        mov r4_seed,#XFEED           ; timer 0 function call
        ajmp move_feed

; Y axis move
movey_s:
        mov r4_seed,#YFEED           ; timer 0 function call
        ajmp move_feed



; decode of miscellaneous commands

miscel:

        lcall inctable_a        ; read miscellaneous context byte

        movx a,@dptr            

miscel1:
        cjne a,#1,miscel2
                
        lcall spindle_cw

        lcall inctable_a

        movx a,@dptr            ; get speed from control table

        lcall set_spindle_speed ; speed from control table
        
        ajmp decode_loop


miscel2:
        cjne a,#2,miscel3

        lcall spindle_ccw

        lcall inctable_a

        movx a,@dptr            ; get speed from control table
        
        lcall set_spindle_speed ; speed from control table
        
        ajmp decode_loop


miscel3:
        cjne a,#3,miscel4

        mov  dptr,#spindle      ; switch-off spindle motor = 0 speed
        clr  a
        movx @dptr,a
        mov spindle_speed,a     ; Copy for IBM
        
        ajmp decode_loop


miscel4:
        cjne a,#4,miscel5

        lcall close_vice_finger
        
        ajmp decode_loop


miscel5:
        cjne a,#5,miscel6

        lcall open_vice_finger
        
        ajmp decode_loop

miscel6:
        cjne a,#6,miscel7

        lcall inctable_a

        movx a,@dptr            ; get speed from control table
        
        lcall set_spindle_speed ; speed from control table
        
        ajmp decode_loop


miscel7:
        cjne a,#7,miscel9
        
        lcall inctable_a
        movx a,@dptr            ; copy table bytes to feedrate
        mov feedrate,a
        
        lcall inctable_a
        movx a,@dptr            ; copy table bytes to feedrate
        mov feedrate+1,a

        lcall inctable_a
        movx a,@dptr            ; copy table bytes to feedrate
        mov feedrate+2,a

        lcall inctable_a
        movx a,@dptr            ; copy table bytes to feedrate
        mov feedrate+3,a


        
;        mov  r1,#feedrate       ; feed rates given in next four bytes of table
;        mov  r0,#4
;miscel7a:
;        
;
;        
;        mov  @r1,a
;        inc  r1
;        djnz r0,miscel7a
        
        ajmp decode_loop





miscel9:
        cjne a,#9,miscel10

        mov  r1,#scratch        ; Off-sets given in next six bytes of table
        mov  r0,#6
miscel9a:
        lcall inctable_a

        movx a,@dptr            ; copy bytes to scratch pad
        mov  @r1,a
        inc  r1
        djnz r0,miscel9a

        mov  dptr,#off_sets
        mov  r1,#scratch
        mov  r0,#6
miscel9b:
        mov  a,@r1              ; copy from scratch pad to offsets record
        movx @dptr,a
        inc  dptr
        inc  r1
        djnz r0,miscel9b

        lcall do_offsets
        
        ajmp decode_loop


miscel10:
        cjne a,#10,miscel11
        
        mov a,#pause            ; pause function call 
        lcall setup_obey_ibm    ; service ibm and/or buttons
        
        ajmp decode_loop


miscel11:
        cjne a,#11,miscel12

        lcall inctable_a        ; ignore tool size
        
        ajmp decode_loop        


miscel12:
        cjne a,#12,miscel13

        lcall inctable_a

        movx a,@dptr            ; read workcell outputs data from table
        mov  r0,a

        mov  a,wco_stat
        orl  a,r0               ; modify with mask to set
        mov  wco_stat,a
        
        mov  dptr,#workcell_o   ; send to workcell outputs
        movx @dptr,a            
        
        ajmp decode_loop


miscel13:
        cjne a,#13,miscel14

        lcall inctable_a

        movx a,@dptr            ; read workcell outputs data from table
        cpl  a
        mov  r0,a

        mov  a,wco_stat
        anl  a,r0               ; modify with mask to clear
        mov  wco_stat,a
        
        mov  dptr,#workcell_o   ; send to workcell outputs
        movx @dptr,a            
        
        ajmp decode_loop

        


miscel14:
        cjne a,#14,miscel15

        lcall inctable_a
        
        movx a,@dptr            ; which workcell input to test from table
        mov  r0,a               ; 
        
        
miscel14a:
        mov a,mcntrl
        jz miscel14b

        lcall setup_obey_ibm    ; IBM wants something - do it
        jb abort,miscel14C      ; Table was aborted by IBM

miscel14b:
        mov  dptr,#workcell_i
        movx a,@dptr            ; read workcell inputs
        anl  a,r0               ; test bit set
        jz   miscel14a          ; wait loop till bit set HIGH
miscel14C:
        ajmp decode_loop




miscel15:
        cjne a,#15,miscel16
        lcall inctable_a
        movx a,@dptr            ; workcell input to test from table
        mov  r0,a
miscel15a:
        mov a,mcntrl
        jz miscel15b
        lcall setup_obey_ibm    ; IBM wants something - do it
        jb abort,miscel15C      ; Table was aborted by IBM
miscel15b:
        mov  dptr,#workcell_i
        movx a,@dptr            ; read workcell inputs
        anl  a,r0               ; test bit low
        jnz  miscel15a          ; wait loop till bit set LOW
miscel15C:
        ajmp decode_loop



miscel16:
        cjne a,#16,miscel17

        lcall inctable_a        ; next two bytes on table are current line
        movx a,@dptr
        mov curr_line,a
        
        lcall inctable_a        
        movx a,@dptr
        mov curr_line+1,a

        ajmp decode_loop


miscel17:
        cjne a,#17,miscel18

        lcall air_on
        
        ajmp decode_loop


miscel18:
        cjne a,#18,miscel19

        lcall air_off
        
        ajmp decode_loop
         
miscel19:
        
        ajmp decode_loop        ; end of miscellaneous command definitions



; Action subroutines

close_vice_finger:
        jnb Vice_1  ,cvf0          ; check if vice and 
        jb  Vice_2  ,cvf0    
        jb  finger_1,cvf0          ; finger are already closed
        jnb Finger_2,cvf0
        ret
cvf0:
        setb Vice_1             ; Close Vice
        clr  Vice_2
        clr  Finger_1           ; Close Finger        
        setb Finger_2

;        mov  dptr,#step_vice
;        mov  a,os_stat
;        movx @dptr,a 
        
        mov   a,#8              ; 4 seconds delay while both move in
        lcall delay_com         ; invoke a delay

        clr  Vice_1              ; open vice
        setb Vice_2
        
;        mov  dptr,#step_vice
;        mov  a,os_stat
;        movx @dptr,a 

        
        mov   a,#1              ; Open main vice a bit
        lcall delay_com         ; finger still moving in
        
        clr  Vice_1              ; stop vice
        clr  Vice_2
        
;        mov  dptr,#step_vice
;        mov  a,os_stat
;        movx @dptr,a 
        
        mov   a,#9              ; total of 9 seconds for finger
        lcall delay_com         
        
        setb Finger_1           ; open finger
        clr  Finger_2
        
        setb Vice_1             ; Close Vice
        clr  Vice_2
        
;        mov  dptr,#step_vice
;        mov  a,os_stat
;        movx @dptr,a
        

        
        mov   a,#1              ; Open finger a bit
        lcall delay_com         
        
        
        
        clr  Finger_1           ; Close Finger        
        setb Finger_2
        
;        mov  dptr,#step_vice
;        mov  a,os_stat
;        movx @dptr,a

        ret



open_vice_finger:
        jb Vice_1,ovf0           ; check if vice and 
        jb Vice_2,ovf0
        jb Finger_1,ovf0           ; finger are already open           
        jb Finger_2,ovf0
        ret
ovf0:
        clr   Vice_1              ; open vice
        setb  Vice_2
        setb  Finger_1            ; open finger
        clr   Finger_2

;        mov   dptr,#step_vice
;        mov   a,os_stat
;        movx  @dptr,a

        mov   a,#18             ; 9 seconds delay while finger and vice move
        lcall delay_com         ; invoke a delay
        
        clr   Vice_1             ; stop vice
        clr   Vice_2
        clr   Finger_1           ; stop finger
        clr   Finger_2
        
;        mov   dptr,#step_vice
;        mov   a,os_stat
;        movx  @dptr,a

        ret



air_on: 
        setb air               ; air blast ON
        
;        mov  dptr,#opto_darl
;        mov  a,opto_stat
;        movx @dptr,a
        ret

air_off: 
        clr  air               ; air blast OFF
        
;        mov  dptr,#opto_darl
;        mov  a,opto_stat
;        movx @dptr,a
        ret

spindle_ccw:
        Setb SpinDir           ; switch on spindle motor, counter-clockwise
        
;        mov  dptr,#opto_darl    
;        mov  a,opto_stat
;        movx @dptr,a
        ret

spindle_cw:
        Clr  SpinDir
        
;        mov  dptr,#opto_darl    ; switch on spindle motor, clockwise
;        mov  a,opto_stat
;        movx @dptr,a
        ret

set_spindle_speed:
        
        mov spindle_speed,a     ; copy for IBM
        Jz  send_spindle_speed
        jb  dooropen,ignore_spindle_speed
        
send_spindle_speed:        
        
        mov  dptr,#spindle
        movx @dptr,a            ; set required spindle motor speed

ignore_spindle_speed:
        ret



delay_com:                      ; enter a delayed state

; delay of 1 second = 11 059 200/12 = 921600  cycles        
; timer interrupt set to 65536 cycles
; - 1 sec = 14 interrupts


; delay of 1 second = 16 000 000/12 = 1333333 cycles        
; timer interrupt set to 65536 cycles
; - 0.5 sec = 10 interrupts



        jz delay_2              ; finished delay ?

        mov tl0_seed,#0
        mov th0_seed,#0
        mov r0,#10
        mov r4,#delay           ; Set timer 0 delay function

        cjne r4,#NULL,$         ; wait til timer interrupts finished

        dec a
        jmp delay_com
delay_2:                            
        ret





do_offsets:                     ; move X,Y and Z by given offsets as steps
                                ; but dont change recorded xyz positions
                                ; done at the 1kh rate, ie. timer set fbc2h
        
        mov  dptr,#off_sets+4   ; do z offset first
        movx a,@dptr            ; lsb
        
        inc  dptr
        movx a,@dptr            ; msb

test_yoff:

        mov  dptr,#off_sets+2   ; test y offset for value
        movx a,@dptr            ; lsb
        
        inc  dptr
        movx a,@dptr            ; msb
        

test_xoff:

        mov  dptr,#off_sets     ; test x offset for value
        movx a,@dptr            ; lsb
        
        inc  dptr
        movx a,@dptr            ; msb

        ret                     ; all offsets done


        page
        subttl Manual Control


manual_control:                 ; decode buttons for what to do

; Entry with acc loaded with the state of the buttons.

; bit0  move X to    home, negative movement
; bit1   ..  Y  .     ..     ..       ..
; bit2   ..  X from  home, positive movement
; bit3   ..  Y  ..    ..     ..       ..
; bit4   ..  Z up to home, negative movement
; bit5   ..  Z down, positive movement
; bit6  fast feed rate of 10mm/sec, normal feed rate 2mm/sec  

; combinations for special functions set with both Z buttons pressed

; Self test, plus X to home.  Will loop indefinately till spindle power off
; Close vice and index finger, plus Y from home.
; Open vice and index finger, plus X from home.
; Air blast, plus Y to home.  Will continue till released.

        ;set timers going/idle
        
        mov  r4,#MAN       ;set timer0 Manual NULL function
        mov  r5,#NULL      ;set timer1 NULL function


man1:
        mov  dptr,#manual1      ; do the read again
        movx a,@dptr
        mov c,acc.7
        mov spindle_on,c

        
        ;setb acc.7              ; ROGERED - power is ON - REMOVE
        cpl  a                  ; for normal logic
        
        jnz man1a

        
        mov  r4,#PURGE     ;set timer0 NULL function
        mov  r5,#NULL      ;set timer1 NULL function
        cjne r4,#NULL,$

        ret
        
man1a:
        jnb   acc.6,fast_manual
        mov  th0_seed,#high LOWFEED         
        mov  tl0_seed,#low  LOWFEED
        mov  th1_seed,#high LOWFEED         
        mov  tl1_seed,#low  LOWFEED
        ajmp  manual01

fast_manual:
        mov  th0_seed,#high MAXFEED     
        mov  tl0_seed,#low  MAXFEED
        mov  th1_seed,#high MAXFEED     
        mov  tl1_seed,#low  MAXFEED

manual01:
        jnb  acc.0,manual4      ; X to home
        clr  xdir
        clr  p1.0
        
        push acc
        mov  a,r4                ; Manual X mode
        setb acc.1
        mov  r4,a
        pop  acc
        
        ajmp  manual02

manual4:
        jnb  acc.2,manual4a      ; X from home
        setb xdir
        setb p1.0
        push acc
        mov  a,r4                ; Manual X mode
        setb acc.1
        mov  r4,a
        pop  acc
        ajmp  manual02

manual4a:
        push acc
        mov a,r4                ; NOT manual X mode
        clr acc.1
        mov r4,a
        pop acc

manual02:
        
        jnb  acc.1,manual8      ; Y to home
        
        clr  ydir
        setb  p1.2
        mov r5,#YSTEP
        ajmp  manual16

manual8:
        
        jnb  acc.3,manual8a     ; Y from home

        setb ydir
        clr  p1.2
        mov r5,#YSTEP
        ajmp  manual16

manual8a:
        mov r5,#NULL

        

manual16:
        
        jnb  acc.4,manual32     ; Z to home, up
        
        clr  zdir
        setb p1.4
        
        push acc
        mov a,r4                ;Set Z manual mode
        setb acc.2
        mov r4,a
        pop acc
        
        ajmp  manual64

manual32:
        
        jnb  acc.5,manual32a     ; Z from home, down

        setb zdir
        clr  p1.4

        push acc
        mov a,r4                ;Set Z manual mode
        setb acc.2
        mov r4,a
        pop acc
        ajmp  manual64

manual32a:

        push acc
        mov a,r4                ;Not Z manual mode
        clr acc.2
        mov r4,a
        pop acc


manual64:
        ajmp man1


        page
        SUBTTL UTILITY ROUTINES

inctable_a:                     ; Increment current address in table

        mov  dpl,r7             ; recover current address
        mov  dph,r6
        inc  dptr               ; increment current address, ready for use
        mov  r6,dph
        mov  r7,dpl             ; save it for next request, ie, r6,r7 BOOKED!
        
        cjne r7,#0,inctable_ax  ; passed a page boundary in control table ?
        ajmp $+3 

inctable_ax:          
        ret

        clr  ea                 ; IBM must not butt in

        mov  a,table_marker     ; read current state of table
        mov  pagebits,a         ; copy for bit marking

        cjne r6,#3,inctable_a1  ; finished first page
        setb pagebits0          ; mark it used
        ajmp  inctable_ac

inctable_a1:
        
        cjne r6,#4,inctable_a2  ; finished second page
        setb pagebits1          ; mark it used
        ajmp  inctable_ac         

inctable_a2:
        
        cjne r6,#5,inctable_a3  ; finished third page
        setb pagebits2          ; mark it used
        ajmp  inctable_ac

inctable_a3:
        
        cjne r6,#6,inctable_a4  ; finished fourth page
        setb pagebits3          ; mark it used
        ajmp  inctable_ac

inctable_a4:
        
        cjne r6,#7,inctable_a5  ; finished fifth page
        setb pagebits4          ; mark it used
        ajmp  inctable_ac

inctable_a5:
        
        cjne r6,#8,inctable_a6  ; finished sixth page
        setb pagebits5          ; mark it used
        ajmp  inctable_ac

inctable_a6:
        
        cjne r6,#9,inctable_a7  ; finished seventh page
        setb pagebits6          ; mark it used
        ajmp  inctable_ac

inctable_a7:
                                ; finished eigth page
        setb pagebits7          ; mark it used
        mov  r6,#2              ; reset to beginning of table
        

inctable_ac:

        
        mov  a,pagebits
        mov  table_marker,a     ; restate state of table after changes
        
        setb ea                 ; let IBM back in
                                
                                
                                
                                ;Now check to see if page were on is valid
inctable_waitvalid:             ; bit will be clear
        
        mov a,table_marker           ; read current state of table

        cjne r6,#2,$+6
        jnb acc.0,inctable_exit
        
        cjne r6,#3,$+6
        jnb acc.1,inctable_exit
        
        cjne r6,#4,$+6
        jnb acc.2,inctable_exit
        
        cjne r6,#5,$+6
        jnb acc.3,inctable_exit
        
        cjne r6,#6,$+6
        jnb acc.4,inctable_exit
        
        cjne r6,#7,$+6
        jnb acc.5,inctable_exit
        
        cjne r6,#8,$+6
        jnb acc.6,inctable_exit
        
        jnb acc.7,inctable_exit  ;r6 will be 9 - no need to check

        mov a,mcntrl
        jz inctable_w

        lcall setup_obey_ibm    ; IBM wants something - do it

inctable_w:
        mov led_cnt_seed,#2     ; flash LED FAST    
        ajmp inctable_waitvalid
        
inctable_exit:        
        
        mov led_cnt_seed,#8     ; flash LED normal
        mov dpl,r7              ; r7 will be 0
        mov dph,r6              ; r6 will be 2,3,4,5,6,7,8 or 9
        ret                     ; Caller expects both R6/R7 and dptr


setdirns:                           ; Set direction bits    
        setb ignore_step            ; Don't count step for position
        jnb dir_nbitsX,sd1
        jb  xdir,sd2
        setb xdir                   ; x+ve for sums
        setb p1.0                   ; x+ve in hardware
        acall send_x_backlash
        ajmp sd2
sd1:
        jnb xdir,sd2
        clr  xdir                   ; x-ve for sums
        clr  p1.0                   ; x-ve in hardware
        acall send_x_backlash

sd2:
        jnb dir_nbitsY,sd3
        jb   ydir,sd4
        setb ydir                   ; y+ve for sums
        clr  p1.2                   ; y+ve in hardware
        acall send_y_backlash
        ajmp sd4
sd3:
        jnb ydir,sd4
        clr ydir                    ; y-ve for sums   
        setb p1.2                    ; y-ve in hardware
        acall send_y_backlash

sd4:
        jnb dir_nbitsZ,sd5
        jb zdir,sd6
        setb zdir                   ; z+ve for sums
        clr p1.4                    ; z+ve in hardware
        acall send_z_backlash
        ajmp sd6

sd5:
        jnb zdir,sd6 
        clr  zdir                   ; z-ve for sums
        setb p1.4                   ; z-ve in hardware
        acall send_z_backlash
sd6:
        clr ignore_step
        ret

send_x_backlash:
        jb  no_back,xb1
        mov r0,x_backlash
        mov r1,#0
        mov th0_seed,#high maxfeed
        mov tl0_seed,#low  maxfeed
        mov r4,#XFEED
        cjne r4,#NULL,$
xb1:
        ret

send_y_backlash:
        jb  no_back,yb1
        mov r0,y_backlash
        mov r1,#0
        mov th0_seed,#high maxfeed
        mov tl0_seed,#low  maxfeed
        mov r4,#YFEED
        cjne r4,#NULL,$
yb1:
        ret

send_z_backlash:
        jb  no_back,zb1
        mov r0,z_backlash
        mov r1,#0
        mov th0_seed,#high maxfeed
        mov tl0_seed,#low  maxfeed
        mov r4,#ZFEED
        cjne r4,#NULL,$
zb1:
        ret


        page
        subttl Home


home:                           ; return X,Y and Z to the "zero" position
                                ; at start-up or on command.

                                ; additionally :-
        
        
       ; mov r4,#NULL
       ; setb tcon.4             ; enable timer0 - vice may need it   
        
        
        setb exe_home
        mov scntrl,#183         ; indicate to IBM we're homing
        

        clr  a
        lcall set_spindle_speed ; - switch off spindle

                           
        lcall spindle_cw        ; - set spindle CW
        lcall air_off           ; - switch off air blaster



        clr  a
        MOV curr_line,a
        MOV curr_line+1,a
        
        mov t2_intcnt,#0
        jb Vice_1,home_vice     ; check if  vice and 
        jb Vice_2,home_vice
        jb Finger_1,home_vice   ; finger are already open           
        jb Finger_2,home_vice
        ajmp home_vice1

home_vice:
        clr  Vice_1              ; open vice
        setb Vice_2

        setb Finger_1              ; open finger
        clr  Finger_2
        
;        mov  dptr,#step_vice
;        mov  a,os_stat
;        movx @dptr,a 


        mov t2_intcnt,#180      ; 20 per second - 9 seconds

home_vice1:



                                ; Z axis moved first, for safety, 
                                ; then X and Y "simultaneously" at 1khz rate

        

        setb init               ; allow moves past limits
        setb ignore_step        ; don't bother to count steps 

        clr a
        mov zpos,a              ; set recorded position to zero
        mov zpos+1,a
        
        mov ypos,a             ; set recorded Y position to zero
        mov ypos+1,a
        
        mov xpos,a             ; set recorded X position to zero
        mov xpos+1,a






        
        mov r0,#0               ;reset uswitch debounce counters
        mov r1,#0
        
        mov r4,#ZSTEP           ; set timer0 function send z steps
        mov r5,#NULL            ; set timer1 function NULL
        
        mov  th0_seed,#high MAXFEED
        mov  tl0_seed,#low  MAXFEED
        
        mov  th1_seed,#high MAXFEED          
        mov  tl1_seed,#low  MAXFEED

        mov  th0,th0_seed          ; set timer intervals 
        mov  tl0,tl0_seed

        mov  th1,th1_seed
        mov  tl1,tl1_seed


        setb p1.4                ;z-ve
        clr  zdir
        
        setb tcon.4                  ; enable timer0    


homez:
        lcall readz_us          ; read micro-switch states

        jb   acc.2,backoffz     ; ie. switch set
        
        ajmp  homez              ; continue till micro-switch is set


backoffz:        

        mov r4,#PURGE           ; set timer0 NULL function
        cjne r4,#NULL,$
        
        
        mov  th0_seed,#HIGH LOWFEED
        mov  tl0_seed,#low  LOWFEED     ; change rate to 50 mm/min
        
        clr p1.4                ; change z direction
        setb zdir                 
        

homez1:
        lcall readz_us          ; read micro-switch states

        jnb  acc.2,zthere       ; ie. not switch set

        lcall set_sendzstep
        
        jb sendzstep,$          ; wait til previous step sent
        
        ajmp  homez1             ; continue till micro-switch not set



zthere:
        clr tcon.4

        mov r0,z_calib
        mov r1,z_calib+1
        mov th0_seed,#high maxfeed
        mov tl0_seed,#low  maxfeed
        mov r4,#ZFEED
        setb tcon.4
        cjne r4,#NULL,$
        clr tcon.4

;        mov zpos,#0             ; set recorded position to zero
;        mov zpos+1,#0



        

; now do the x-y home move, simultaneously

        
        mov r0,#0               ;reset uswitch debounce counters
        mov r1,#0               ;for X

        mov r2,#0               ;reset uswitch debounce counters
        mov r3,#0               ;for Y
        
        mov  r4,#XSTEP
        mov  r5,#YSTEP

        mov  th0_seed,#high MAXFEED     ; set timer intervals 
        mov  tl0_seed,#low  MAXFEED
        
        mov  th0,th0_seed
        mov  tl0,tl0_seed
        

        clr  p1.0                ;x-ve
        clr  xdir
        
        setb p1.2                ;y-ve
        clr  ydir
        
        
        setb tcon.4                  ; enable timer0    
        setb tcon.6                  ; enable timer0    

homexy:
        lcall readx_us          ; read micro-switch states

        jnb  acc.0,homexy_1     ; ie. switch set

        clr tcon.4
homexy_1:
        
        lcall ready_us          ; read micro-switch states

        jnb  acc.1,homexy_2     ; ie. switch set

        clr tcon.6
homexy_2:
        jb tcon.4,homexy        
        jb tcon.6,homexy        ; continue till both micro-switch is set


backoffx:        
        mov r4,#NULL            ; set timer0 NULL function -  timer OFF
        
        mov  th0_seed,#high LOWFEED
        mov  tl0_seed,#low  LOWFEED    ; change rate to 50 mm/min
        MOV th0,th0_seed
        MOV tl0,tl0_seed
        
        setb p1.0                ; change x direction
        setb xdir                 

        setb tcon.4

homex1:
        lcall readx_us          ; read micro-switch states

        jnb  acc.0,xthere       ; ie. not switch set

        lcall set_sendxstep
        
        jb sendxstep,$          ; wait til previous step sent
        
        ajmp  homex1             ; continue till micro-switch not set

        clr tcon.4

xthere:

        mov r0,x_calib
        mov r1,x_calib+1
        mov th0_seed,#high maxfeed
        mov tl0_seed,#low  maxfeed
        mov r4,#XFEED
        setb tcon.4
        cjne r4,#NULL,$

;        clr tcon.4

;        mov xpos,#0             ; set recorded X position to zero
;        mov xpos+1,#0




backoffy:        
        mov r5,#NULL             ; set timer1 NULL function - timer OFF
        
        mov  th1_seed,#high LOWFEED
        mov  tl1_seed,#low  LOWFEED    ; change rate to 50 mm/min
        MOV  th1,th1_seed
        MOV  tl1,tl1_seed
        
        clr  p1.2                ; change y direction
        setb ydir                

        setb tcon.6

homey1:
        lcall ready_us          ; read micro-switch states

        jnb  acc.1,ythere       ; ie. not switch set

        lcall set_sendystep
        
        jb sendystep,$          ; wait til previous step sent
        
        ajmp  homey1             ; continue till micro-switch not set

ythere:
        
        mov r0,y_calib
        mov r1,y_calib+1
        mov th0_seed,#high maxfeed
        mov tl0_seed,#low  maxfeed
        mov r4,#YFEED
        setb tcon.4
        cjne r4,#NULL,$

;        clr tcon.4

;        mov ypos,#0             ; set recorded Y position to zero
;        mov ypos+1,#0

        
        
wait_vice:        
        mov a,t2_intcnt
        jnz wait_vice

        clr  Vice_1              ; stop vice
        clr  Vice_2
        clr  Finger_1              ; stop finger
        clr  Finger_2
;        mov  dptr,#step_vice
;        mov  a,os_stat
;        movx @dptr,a 

        
        clr exe_home
        clr init                ; limits are now set - use them
        clr ignore_step
        
        ret





readx_us:
        mov  dptr,#microsws     ; read micro-switch states
rg3:
        movx a,@dptr
        cpl  a                  ; for normal logic

        jb acc.0,rg2
        
        mov r0,#0
        inc r1
        cjne r1,#0,rg3
        dec r1
        ret
rg2:
        mov r1,#0
        inc r0
        cjne r0,#0,rg3
        dec r0
        ret


ready_us:
        mov  dptr,#microsws     ; read micro-switch states
rg6:        
        movx a,@dptr
        cpl  a                  ; for normal logic
        
        jb acc.1,rg5
        
        mov r2,#0
        inc r3
        cjne r3,#0,rg6
        dec r3
        ret
rg5:
        mov r3,#0
        inc r2
        cjne r2,#0,rg6
        dec r2
        ret


readz_us:
        mov  dptr,#microsws     ; read micro-switch states
rg9:
        movx a,@dptr
        cpl  a                  ; for normal logic
        jb acc.2,rg8
        
        mov r0,#0
        inc r1
        cjne r1,#0,rg9
        dec r1
        ret
rg8:
        mov r1,#0
        inc r0
        cjne r0,#0,rg9
        dec r0
        ret



include ibmint.inc              ; External interrupt 0 service

include timer0.inc              ; Timer 0

include timer1.inc              ; Timer 1

include timer2.inc              ; Timer 2


        page
        subttl Ramp Table
        
        org high $ + 1 * 256

ramptable:                      ; 16 bit entries for timer delay settings
    
        nolist

include ramp.inc
        list
        db "END"
        nolist

END





