    PAGE
    SUBTTL "Timer 0 ISR"


;Modified 19/4/95
; Timer0 runs at twice step freq
;   First interrupt sets step low
;   Second interrupt returns step high




;r0     - lsb steps (x or z)    
;r1     - msb steps (x or z)

;r2     - lsb steps (y)         
;r3     - msb steps (y)         

;r4     - timer 0 function call
;r5     - timer 1 function call


timer0: 
        mov th0,th0_seed        ;2  reseed timer
        mov tl0,tl0_seed        ;2 = 12 cycles before timer restarts
        

;Functions
;Dummy

;Step x asap (up/flat/down) entry is up
;Step z asap (up/flat/down) entry is up

;Step XY fixed (7.6.5.4.3.2.1.0) entry is 7

;Step x-ve at jog feedrate
;Step x+ve at jog feedrate

;Step z-ve at jog feedrate
;Step z+ve at jog feedrate
        
        jnb sentxstep,t0_0a
        clr sentxstep
        setb p1.1
        setb intused
t0_0a:
        jnb sentystep_XY,t0_0b
        clr sentystep_XY
        setb p1.3
        setb intused
t0_0b:
        jnb sentzstep,t0_0c
        clr sentzstep
        setb p1.5
        setb intused
t0_0c:
        jnb intused,t0_0d
        clr intused
        reti
t0_0d:
        
        
        push psw                ; time critical section
        push acc                ; operating as level 1 interrupt
        push dpl                ; so can't be interrupted
        push dph                

        jnb sendxstep,t0_2a
        jnb stepper_xy_power,t0_1a  ; can't step until power is on
        ajmp t0_power             ; so switch power on
        
t0_1a:
        clr p1.1                ; set step for x low
        setb sentxstep
        jb ignore_step,t0_2a
        
        jnb xdir,t0_1b          ; Steps count - inc or dec position?
        
        mov  a,xpos             ; increment position
        add  a,#1
        mov  xpos,a
        mov  a,xpos+1
        addc a,#0
        mov  xpos+1,a
        ajmp  t0_2a

t0_1b:
        clr  c                  ; decrement position
        mov  a,xpos
        subb a,#1
        mov  xpos,a
        
        mov  a,xpos+1
        subb a,#0
        mov  xpos+1,a


t0_2a:
        jnb sendzstep,t0_3a
        jb  stepper_z_power,t0_z_power  ; switch power on
        
        clr p1.5               ;set step for z low
        setb sentzstep
        jb  ignore_step,t0_3a
        jnb zdir,t0_2b
        
        mov  a,zpos             ; increment position
        add  a,#1
        mov  zpos,a
        
        mov  a,zpos+1
        addc a,#0
        mov  zpos+1,a
        ajmp t0_3a

t0_2b:
        clr  c                      ; decrement position
        mov  a,zpos
        subb a,#1
        mov  zpos,a
        
        mov  a,zpos+1
        subb a,#0
        mov  zpos+1,a


t0_3a:
        jnb sendystep_XY,t0_4a
        jb  stepper_xy_power,t0_power  ; switch power on
        
        clr p1.3                ; set step for y low
        setb sentystep_XY
        jb ignore_step,t0_4a
        
        jnb ydir,t0_3b
        mov  a,ypos             ;increment position
        add  a,#1
        mov  ypos,a
        
        mov  a,ypos+1
        addc a,#0
        mov  ypos+1,a
        ajmp  t0_4a

t0_3b:
        clr  c                  ;decrement position
        mov  a,ypos
        subb a,#1
        mov  ypos,a
        
        mov  a,ypos+1
        subb a,#0
        mov  ypos+1,a


t0_4a:  

;rest of sub is not timing critical - other interrupts can be serviced 
        
        clr et0                 ; disable  timer 0 interrupts
        lcall end_timer0int     ; clear the interrupt-in-progress latch
                                ; the rest of the "ISR" is interruptable
                                ; this bodgery from p5-20 Intel



        clr sendxstep
        clr sendzstep
        clr sendystep_XY        ; y step during XY stepping

        mov dptr,#timer0_tab
        mov a,r4                ;set function
        jmp @a+dptr             ;do timer function


t0_power:
;Can only switch on if doors are closed 1/11/96
;        mov dptr,#microsws
;        movx a,@dptr
;        jb acc.3,t0_nopower
        
        jb suspend,t0_nopower

        clr  stepper_xy_power      ; switch power on
        mov  dptr,#step_vice    ; step(s) will be sent next interrupt
        mov  a,os_stat
        movx @dptr,a

;        mov  a,spindle_speed    ;restart spindle 
;        mov  dptr,#spindle
;        movx @dptr,a

t0_nopower:
        pop dph 
        pop dpl
        pop acc
        pop psw
        reti

t0_z_power:
;        mov dptr,#microsws
;        movx a,@dptr
;        jb acc.3,t0_z_nopower

        jb suspend,t0_nopower
        
        clr  stepper_z_power    ; switch power on
        mov  dptr,#step_vice    ; step(s) will be sent next interrupt
        mov  a,os_stat
        movx @dptr,a

;        mov  a,spindle_speed    ;restart spindle
;        mov  dptr,#spindle
;        movx @dptr,a


t0_z_nopower:
        pop dph 
        pop dpl
        pop acc
        pop psw
        reti

        
        
        org high $ + 1 * 256    ;force table to start on page boundary 




timer0_tab:
        ajmp t0_exit            ;0
        
        ajmp t0_xasap_up        ;2        
        ajmp t0_xasap_flat      ;4
        ajmp t0_xasap_down      ;6
        
        ajmp t0_zasap_up        ;8
        ajmp t0_zasap_flat      ;10
        ajmp t0_zasap_down      ;12

        ajmp t0_stepxy          ;14 - send bit 7    
        ajmp t0_stepxy          ;16 - send bit 6    
        ajmp t0_stepxy          ;18 - send bit 5    
        ajmp t0_stepxy          ;20 - send bit 4    
        ajmp t0_stepxy          ;22 - send bit 3    
        ajmp t0_stepxy          ;24 - send bit 2    
        ajmp t0_stepxy          ;26 - send bit 1    
        ajmp t0_stepxy          ;28 - send bit 0    
        ajmp t0_stepxy_last     ;30 - last call - dummy 
        
        ajmp t0_exit            ;32     - manual - NULL
        ajmp t0_stepx           ;34     - 32 + 2
        ajmp t0_stepz           ;36     - 32 + 4
        ajmp t0_stepxz          ;38     - 32 + 2 + 4

        ajmp t0_stepx_feed      ;40     - step x at feedrate
        ajmp t0_stepy_feed      ;42     - step y at feedrate
        ajmp t0_stepz_feed      ;44     - step z at feedrate


        ajmp t0_purge           ;46 sets NULL fuction, after any steps

        ajmp t0_delay           ;48

        ajmp t_z_home           ;50


t0_purge:
        mov tl0_seed,#0
        mov th0_seed,#0
        mov r4,#NULL
        ajmp t0_exit


t0_stepx:                       ;used for manual control to
        lcall set_sendxstep      ;send pulse stream
        ajmp t0_exit

t0_stepz:
        lcall set_sendzstep
        ajmp t0_exit

t0_stepxz:
        lcall set_sendxstep
        lcall set_sendzstep
        ajmp t0_exit


t0_exit:
        pop dph
        pop dpl
        pop acc
        pop psw
        setb et0
        ret


end_timer0int:                    ; called at end of timer critical section
        reti                      ; clears the interrupt-in-progress flip-flop






t0_zasap_up:                      ; t0_func=8
;expects r0/r1 to hold number of steps to ramp up 
        jb abort,t0_zasap_end
        cjne r0,#0,z_asap3a
        cjne r1,#0,z_asap3b
        
        mov r0,z_flat             ; lsb
        mov r1,z_flat+1           ; msb
        inc r4                    ; set function call for next interrupt
        inc r4
        ajmp t0_zasap_flat

z_asap3a:
        dec r0        
        ajmp z_asap3c
z_asap3b:
        dec r0                  ;will set r0 to ff
        dec r1
z_asap3c:      
        lcall set_sendzstep          ;send step at start of next interrupt
        lcall zramp_to_t0seed   ; read data from table into timer registers

        clr c                       ; point to next word in table    
        mov a,ramp_z
        addc a,#2
        mov ramp_z,a   
        
        mov a,ramp_z+1
        addc a,#0
        mov ramp_z+1,a   
        

        ajmp t0_exit


t0_zasap_flat:                    ;t0_func=10
        jb abort,t0_zasap_end
        cjne r0,#0,z_asap5a
        cjne r1,#0,z_asap5b

        mov r0,z_updown             ;lsb
        mov r1,z_updown+1           ;msb
        inc r4                  ;set function call for next interrupt
        inc r4
        ajmp t0_zasap_down

z_asap5a:
        dec r0
        ajmp z_asap5c
z_asap5b:            
        dec r0
        dec r1
z_asap5c:
        lcall set_sendzstep     ; send step at start of next interrupt
        lcall zramp_to_t0seed   ; read data from table into timer registers
        ajmp t0_exit



t0_zasap_down:
        jb abort,t0_zasap_end
        cjne r0,#0,z_asap7a
        cjne r1,#0,z_asap7b
        
t0_zasap_end:
        mov r4,#PURGE            ;set null function
        ajmp t0_exit

z_asap7a:
        dec r0
        ajmp z_asap7c
z_asap7b:        
        dec r0
        dec r1
z_asap7c:      
        lcall set_sendzstep          ; send step at start of next interrupt        
        lcall zramp_to_t0seed   ; read data from table into timer registers

        clr c                   ; point to previous word in table    
        mov a,ramp_z
        subb a,#2
        mov ramp_z,a   
        
        mov a,ramp_z+1
        subb a,#0
        mov ramp_z+1,a   
        
        ajmp t0_exit



t0_xasap_up:        ;2        
        jb abort,t0_xasap_end
        cjne r0,#0,x_asap3a
        cjne r1,#0,x_asap3b
        
        mov r0,x_flat             ;lsb
        mov r1,x_flat+1           ;msb

        inc r4                  ;set next function
        inc r4
        ajmp t0_xasap_flat


x_asap3a:
        dec r0        
        ajmp x_asap3c
x_asap3b:
        dec r0                  ;will set r0 to ff
        dec r1
x_asap3c:      
        lcall set_sendxstep          ; send step at start of next interrupt        
        lcall xramp_to_t0seed    ; read data from table into timer registers

        clr c                       ; point to next word in table    
        mov a,ramp_x
        addc a,#2
        mov ramp_x,a   
        
        mov a,ramp_x+1
        addc a,#0
        mov ramp_x+1,a   
        
        ajmp t0_exit


t0_xasap_flat:      ;4
        jb abort,t0_xasap_end
        cjne r0,#0,x_asap5a
        cjne r1,#0,x_asap5b
        
        mov r0,x_updown             ;lsb
        mov r1,x_updown+1           ;msb

        inc r4                     ;set next function
        inc r4
        ajmp t0_xasap_down

x_asap5a:
        dec r0
        ajmp x_asap5c
x_asap5b:            
        dec r0
        dec r1
x_asap5c:
        lcall set_sendxstep          ; send step at start of next interrupt        
        lcall xramp_to_t0seed   ; read data from table into timer registers
        
        ajmp t0_exit




t0_xasap_down:      ;6
        jb abort,t0_xasap_end
        cjne r0,#0,x_asap7a
        cjne r1,#0,x_asap7b
t0_xasap_end:        
        mov r4,#PURGE             ; set null function     
        ajmp t0_exit

x_asap7a:
        dec r0
        ajmp x_asap7c
x_asap7b:        
        dec r0
        dec r1
x_asap7c:      
        lcall set_sendxstep          ; send step at start of next interrupt                
        lcall xramp_to_t0seed   ; read data from table into timer registers

        clr c                   ; point to previous word in table    
        mov a,ramp_x
        subb a,#2
        mov ramp_x,a   
        
        mov a,ramp_x+1
        subb a,#0
        mov ramp_x+1,a   

        ajmp t0_exit


t0_stepxy:      ;r4=14 to 
                ;look at bits 7 of x_pattern and y_pattern
        mov b,#0
        mov a,x_pattern
        rl a
        mov x_pattern,a

        jnb acc.7,stepxy1
        lcall set_sendxstep
        inc b
stepxy1:        
        mov a,y_pattern
        rl a
        mov y_pattern,a
        
        jnb acc.7,stepxy2
        lcall set_sendystep_XY
        inc b
stepxy2:
        mov a,b                 ;b will be 0,1 or 2        
        jz stepxy3
        cjne a,#2,stepxy3
        
        mov tl0_seed,feedrate+2 ;both axes moving use 2nd feedrate
        mov th0_seed,feedrate+3
        ajmp stepxy3a

stepxy3:
        mov tl0_seed,feedrate
        mov th0_seed,feedrate+1
stepxy3a:        
        
        
        inc r4
        inc r4          ;set next timer function
        ajmp t0_exit


t0_stepxy_last:         ;final call to XY steps
;        mov tl0_seed,#0
;        mov th0_seed,#0
        mov r4,#NULL
        ajmp t0_exit




t0_stepx_feed:      ;40     - step x at feedrate
        cjne r0,#0,xf1
        cjne r1,#0,xf2
        mov r4,#PURGE            ; Done
        ajmp t0_exit     

xf1:
        dec r0
        ajmp xf3
xf2:            
        dec r0
        dec r1
xf3:
        lcall set_sendxstep          ; send step at start of next interrupt
        ajmp t0_exit


t0_stepy_feed:      ;42     - step y at feedrate
        cjne r0,#0,yf1
        cjne r1,#0,yf2
        mov r4,#PURGE         ; Done
        ajmp t0_exit     

yf1:
        dec r0
        ajmp yf3
yf2:            
        dec r0
        dec r1
yf3:
        lcall set_sendystep_XY      ; send step at start of next interrupt
        ajmp t0_exit


t0_stepz_feed:      ;44     - step z at feedrate
        cjne r0,#0,zf1
        cjne r1,#0,zf2
        mov r4,#PURGE         ; Done
        ajmp t0_exit     

zf1:
        dec r0
        ajmp zf3
zf2:            
        dec r0
        dec r1
zf3:
        lcall set_sendzstep          ; send step at start of next interrupt
        ajmp t0_exit






t0_delay:
        cjne r0,#0,t0del1
        mov r4,#NULL
        ajmp t0_exit     

t0del1:
        dec r0
        ajmp t0_exit     



t_z_home:
        ajmp t0_exit     
