    PAGE
    SUBTTL "Timer 1 ISR"


;r0     - lsb steps (x or z)
;r1     - msb steps (x or z)

;r2     - lsb steps (y)
;r3     - msb steps (y)

;r4     - timer 0 function call
;r5     - timer 1 function call


timer1: 
    mov th1,th1_seed
    mov tl1,tl1_seed        ; reseed timer
    
    jnb  sentystep,t1_0
    setb p1.3               ;return step for y high
    clr  sentystep
    reti 

t1_0:
        
;        jnb ack_want_all,timer1_1 
;        reti
timer1_1: 


;Functions
;Dummy

;Step y asap (up/flat/down) entry is up
;Step y asap (up/flat/down) entry is up

;Step y-ve at jog feedrate
;Step y+ve at jog feedrate

        push psw
        push acc
        push dpl
        push dph

        jnb sendystep,t1_2a
        jb  stepper_xy_power,t1_power   ;if power is off switch it on
        
        clr p1.3                ; set step for y low
        setb sentystep          ; set flag to clear step next interrupt
        
        jb ignore_step,t1_2a
        
        jnb ydir,t1_1b
        
        mov  a,ypos         ; increment position
        add  a,#1
        mov  ypos,a
        mov  a,ypos+1
        addc a,#0
        mov  ypos+1,a
        ajmp  t1_2a

t1_1b:
        clr c               ;decrement position
        mov a,ypos
        subb a,#1
        mov ypos,a
        mov a,ypos+1
        subb a,#0
        mov ypos+1,a


t1_2a:

;rest of sub is not timing critical - other interrupts can be serviced 
        
        clr et1                 ; disable timer 1 interrupts
        lcall end_timer1int     ; clear the interrupt-in-progress latch
                                ; the rest of the "ISR" is interruptable
                                ; this bodgery from p5-20 Intel


        clr sendystep

        mov dptr,#timer1_tab
        mov a,r5                ;set function
        jmp @a+dptr             ;do timer function


t1_power:
;Can only power up if doors are closed        

        jb suspend,t1_nopower

        clr stepper_xy_power       ;switch power on
        mov dptr,#step_vice     ;step will be sent next interrupt
        mov a,os_stat
        movx @dptr,a

t1_nopower:
        pop dph
        pop dpl
        pop acc
        pop psw
        reti


        org high $ + 1 * 256    ; start table on page boundary

timer1_tab:
        ajmp t1_exit            ;0
        
        ajmp t1_yasap_up        ;2        
        ajmp t1_yasap_flat      ;4
        ajmp t1_yasap_down      ;6

        ajmp t1_exit            ;8
        ajmp t1_exit            ;10
        ajmp t1_exit            ;12
        ajmp t1_exit            ;14
        ajmp t1_exit            ;16
        ajmp t1_exit            ;18
        ajmp t1_exit            ;20
        ajmp t1_exit            ;22
        ajmp t1_exit            ;24
        ajmp t1_exit            ;26
        ajmp t1_exit            ;28
        
        ajmp t1_sendy           ;30

        ajmp t1_purge           ;32

t1_purge:
        mov r5,#NULL
        mov tl1_seed,#low  maxdelay
        mov th1_seed,#high maxdelay



t1_exit:
        jnb want_all,t1_exit_1
        cjne r5,#2,t1_ack_1
        ajmp t1_exit_1        
t1_ack_1:        
        cjne r5,#4,t1_ack_2
        ajmp t1_exit_1
t1_ack_2:        
        cjne r5,#6,t1_ack_3
        ajmp t1_exit_1       
t1_ack_3:        
        
        cjne r4,#2,t1_ack_4     ;x up
        ajmp t1_exit_1        
t1_ack_4:        
        cjne r4,#4,t1_ack_5     ;x flat
        ajmp t1_exit_1
t1_ack_5:        
        cjne r4,#6,t1_ack_6     ;x down
        ajmp t1_exit_1       
t1_ack_6:        

        cjne r4,#8,t1_ack_7     ;z up
        ajmp t1_exit_1        
t1_ack_7:        
        cjne r4,#10,t1_ack_8    ;z flat
        ajmp t1_exit_1
t1_ack_8:        
        cjne r4,#12,t1_ack_9    ;z down
        ajmp t1_exit_1       
t1_ack_9:        
        setb ack_want_all
        
t1_exit_1:
        
        pop dph
        pop dpl
        pop acc
        pop psw
        setb et1
        ret
        
end_timer1int:        
        reti                    



t1_ack_want:            
; Timer 2 sets flag want_all when it needs good timing
; if timer0 and timer1 are not ramping acknowledge











t1_sendy:
        lcall set_sendystep
        ajmp t1_exit



t1_yasap_up:                      ;r5=2
        jb abort,t1_yasap_End
        cjne r2,#0,y_asap3a
        cjne r3,#0,y_asap3b
        
        mov r2,y_flat             ;lsb
        mov r3,y_flat+1           ;msb
        
        inc r5
        inc r5
        ajmp  t1_yasap_flat            ;finished going up ramp

y_asap3a:
        dec r2        
        ajmp y_asap3c
y_asap3b:
        dec r2                  ;will set r2 to ff
        dec r3
y_asap3c:      
        lcall set_sendystep 
        lcall yramp_to_t1seed    ; read data from table into timer registers

        clr c                       ; point to next word in table    
        mov a,ramp_y
        addc a,#2
        mov ramp_y,a   
        
        mov a,ramp_y+1
        addc a,#0
        mov ramp_y+1,a   
        
        ajmp  t1_exit_1



t1_yasap_flat:                    ;r5=4
        jb abort,t1_yasap_End
        cjne r2,#0,y_asap5a
        cjne r3,#0,y_asap5b
        
        mov r2,y_updown             ;lsb
        mov r3,y_updown+1           ;msb
        inc r5
        inc r5
        ajmp  t1_yasap_down            ;finished flat section    

y_asap5a:
        dec r2
        ajmp y_asap5c
y_asap5b:            
        dec r2
        dec r3
y_asap5c:
        lcall set_sendystep 
        lcall yramp_to_t1seed    ; read data from table into timer registers
        
        ajmp t1_exit_1


t1_yasap_down:                  ;r5=6
        jb abort,t1_yasap_End
        cjne r2,#0,y_asap7a
        cjne r3,#0,y_asap7b
        
t1_yasap_End:
        mov r5,#PURGE_Y           ;set NULL function                       
        ajmp t1_exit               ;finished going down ramp

y_asap7a:
        dec r2
        ajmp y_asap7c
y_asap7b:        
        dec r2
        dec r3
y_asap7c:      
        lcall set_sendystep 
        lcall yramp_to_t1seed        ; read data from table into timer registers

        clr c                       ; point to previous word in table    
        mov a,ramp_y
        subb a,#2
        mov ramp_y,a   
        
        mov a,ramp_y+1
        subb a,#0
        mov ramp_y+1,a   
        
        ajmp  t1_exit_1




xramp_to_t0seed:
        mov dph,ramp_x+1
        mov dpl,ramp_x
        
        clr  a
        movc a,@a+dptr          ; read byte from table, msb of timer set value
        mov  th0_seed,a

        inc  dptr

        clr  a
        movc a,@a+dptr          ; read second byte, lsb of timer set value
        mov  tl0_seed,a
        ret


yramp_to_t1seed:
        mov dph,ramp_y+1
        mov dpl,ramp_y

        clr  a
        movc a,@a+dptr          ; read byte from table, msb of timer set value
        mov  th1_seed,a

        inc  dptr

        clr  a
        movc a,@a+dptr          ; read second byte, lsb of timer set value
        mov  tl1_seed,a
        ret



zramp_to_t0seed:
        mov dph,ramp_z+1
        mov dpl,ramp_z
        
        clr  a
        movc a,@a+dptr          ; read byte from table, msb of timer set value
        mov  th0_seed,a

        inc  dptr

        clr  a
        movc a,@a+dptr          ; read second byte, lsb of timer set value
        mov  tl0_seed,a
        ret




set_sendxstep:
        jb   init,sx1           ;If initilaising then set step
        jb   xdir,sx2
        
        mov a,xpos
        jnz  sx1                 ;not at 0 so set step
        mov a,xpos+1
        jnz  sx1                 ;not at 0 so set step
        ret
sx2:
        mov a,xpos
        cjne a,#low XMAX,sx1    ;not @ max so set step 
        
        mov a,xpos+1
        cjne a,#high XMAX,sx1   ;not @ max so set step 
        ret

sx1:
        setb sendxstep
        mov  power_xy_cnt,#powerlen       ; switch stepper power on
        ret



set_sendystep:
        jb   init,sy1           ;If initilaising then set step
        jb   ydir,sy2
        
        mov a,ypos
        jnz  sy1                 ;not at 0 so set step
        mov a,ypos+1
        jnz  sy1                 ;not at 0 so set step
        ret
sy2:
        mov a,ypos
        cjne a,#low YMAX,sy1    ;not @ max so set step 
        
        mov a,ypos+1
        cjne a,#high YMAX,sy1   ;not @ max so set step 
        ret
sy1:
        setb sendystep
        mov  power_xy_cnt,#powerlen
        ret



set_sendystep_XY:
        jb   init,sy1_XY           ;If initilaising then set step
        jb   ydir,sy2_XY
        
        mov a,ypos
        jnz  sy1_XY                 ;not at 0 so set step
        mov a,ypos+1
        jnz  sy1_XY                 ;not at 0 so set step
        ret
sy2_XY:
        mov a,ypos
        cjne a,#low YMAX,sy1_XY    ;not @ max so set step 
        mov a,ypos+1
        cjne a,#high YMAX,sy1_XY   ;not @ max so set step 
        ret
sy1_XY:
        setb sendystep_XY
        mov  power_xy_cnt,#powerlen
        ret





set_sendzstep:
        jb   init,sz1           ;If initilaising then set step
        jb   zdir,sz2
        
        mov a,zpos
        jnz  sz1                 ;not at 0 so set step
        mov a,zpos+1
        jnz  sz1                 ;not at 0 so set step
        ret
sz2:
        mov a,zpos
        cjne a,#low ZMAX,sz1    ;not @ max so set step 
        mov a,zpos+1
        cjne a,#high ZMAX,sz1   ;not @ max so set step 
        ret
sz1:
        setb sendzstep
        mov  power_z_cnt,#powerlen
        ret
