include cyber.inc

.code

wait25 macro
	mov cx,2
	.repeat
		timer2wait
	.untilcxz
	endm


wait1000 macro
	mov cx,800
	.repeat
		timer2wait
	.untilcxz
	endm
	
waiteoc macro
comment *
	conversion time max 25usec - according to data sheet
*
	mov si,robot[bx].read
	dlwait
	mov al,es:[si]
	dlwait
	mov al,es:[si]
	and al,1
	
	.if al==0               ;EOC should be high
		call putoffline
	.else
		wait25              ; Wait for ADC to do conversion
		mov al,es:[si]
		dlwait
		mov al,es:[si]
		and al,1
		.if al==1           ;EOC should now be low
			call putoffline
		.else
			call putonline
		.endif
	.endif


;    call putonline
	
	endm


readadc macro channel,Bits
	mov di,robot[bx].write
	mov al,080h
	add al,channel
	dlwait
	mov es:[di+14],al
	
	mov al,channel
	dlwait
	mov es:[di+14],al
	
	mov al,080h
	add al,channel
	dlwait
	mov es:[di+14],al
	
	waiteoc
	mov si,robot[bx].read
	dlwait

	if Bits eq 8
		mov al,es:[si+1]
		dlwait
		mov al,es:[si+1]
		mov ah,0
	else
		mov al,es:[si]
		dlwait
		mov al,es:[si]
		
		dlwait
		mov ah,es:[si+1]
		dlwait
		mov ah,es:[si+1]

		shr ax,4
	endif
		push ax


	mov al,08fh             ;Set mux to unused channel
	dlwait                  ;reduces interchannel noise?
	mov es:[di+14],al

	pop ax


	endm


service_neptune1_pulses macro
	mov di,robot[bx].read
	add di,2
	add di,robot[bx].pulseop     ;1 to 7
	dlwait
	mov al,es:[di]
	endm


service_neptune1 proc

	mov robot[bx].SimPres,1         ;Assume Simulator connected


	mov ax,es                               ; Check if turbo board exists
	.if ax != 0
		readadc 7,12                        ; Read Workcell input
		mov robot[bx].WC.ANA_inputs[0],ax   ; checks if neptune1 is online
	.endif
	
	.IF robot[bx].online==DEV_ONLINE
		.if robot[bx].pulsecount == 1
			service_neptune1_pulses
			mov robot[bx].pulsecount,0
		.endif

		.if (robot[bx].teachmode==INIT) || (robot[bx].teachmode==EmerStop)

			MOV robot[bx].stop[0],1
			MOV robot[bx].stop[2],1
			MOV robot[bx].stop[4],1
			MOV robot[bx].stop[6],1
			MOV robot[bx].stop[8],1
			MOV robot[bx].stop[10],1

			mov robot[bx].teachmode,KEYBOARD
		
		.elseif robot[bx].teachmode==SIMULATOR
			readadc 8,8
			invoke filter ,8,ax,robot[bx].learn[0],2
			mov robot[bx].learn[0],ax
			mov robot[bx].despos[0],ax  ;Axis 0
							
			readadc 9,8
			invoke filter ,9,ax,robot[bx].learn[2],2
			mov robot[bx].learn[2],ax
			mov robot[bx].despos[2],ax  ;Axis 1
							
			readadc 10,8
			invoke filter ,10,ax,robot[bx].learn[4],2
			mov robot[bx].learn[4],ax
			mov robot[bx].despos[4],ax  ;Axis 2

			readadc 11,8
			invoke filter ,11,ax,robot[bx].learn[6],2
			mov robot[bx].learn[6],ax
			mov robot[bx].despos[6],ax   ;Axis 3
							
			readadc 13,8
			invoke filter ,13,ax,robot[bx].learn[8],2
			mov robot[bx].learn[8],ax
			mov robot[bx].despos[8],ax   ;Axis 4

			readadc 14,8
			invoke filter ,14,ax,robot[bx].learn[10],2
			mov robot[bx].learn[10],ax
			mov robot[bx].despos[10],ax   ;Axis 5
		.endif

		readadc 0,8
		.if robot[bx].stop[0] != FALSE
			DEC robot[BX].stop[0]
			mov robot[bx].despos[0],AX 
		.ENDIF
		invoke filter ,0,ax,robot[bx].feedback[0],2
		mov robot[bx].feedback[0],AX    ;Axis 0
					
		readadc 1,8
		.if robot[bx].stop[2] != FALSE      
			dec robot[BX].stop[2]
			mov robot[bx].despos[2],AX 
		.ENDIF
		invoke filter ,1,ax,robot[bx].feedback[2],2
		mov robot[bx].feedback[2],AX    ;Axis 1
					
		readadc 2,8
		.if robot[bx].stop[4] != false
			dec robot[BX].stop[4]
			mov robot[bx].despos[4],AX 
		.ENDIF
		invoke filter ,2,ax,robot[bx].feedback[4],2
		mov robot[bx].feedback[4],AX    ;Axis 2
					
		readadc 3,8
		.if robot[bx].stop[6] != false
			dec robot[BX].stop[6]
			mov robot[bx].despos[6],AX 
		.ENDIF
		invoke filter ,3,ax,robot[bx].feedback[6],2
		mov robot[bx].feedback[6],AX    ;Axis 3
					
		readadc 5,8
		.if robot[bx].stop[8] != false
			dec robot[BX].stop[8]
			mov robot[bx].despos[8],AX 
		.ENDIF
		invoke filter ,4,ax,robot[bx].feedback[8],2
		mov robot[bx].feedback[8],AX    ;Axis 4
					
		readadc 6,8
		.if robot[bx].stop[10] != false
			dec robot[BX].stop[10]
			mov robot[bx].despos[10],AX 
		.ENDIF
		invoke filter ,5,ax,robot[bx].feedback[10],2
		mov robot[bx].feedback[10],AX   ;Axis 5,Gripper

			
			mov robot[bx].inpos,True ;assume there

			mov si,0
			.repeat
				mov ax,robot[bx].despos[si]
				.if ax>robot[bx].Feedback[si]
					sub ax,robot[bx].Feedback[si]
				.else
					mov ax,robot[bx].Feedback[si]
					sub ax,robot[bx].despos[si]
				.endif
				.if ax>5
					mov robot[bx].inpos,False
					.break
				.endif
				add si,2
			.until si==10

			.if robot[bx].inpos!=False
				mov si,0
				.repeat
					invoke AxisChanged ,si
					.if ax!=0
						mov robot[bx].inpos,False
						.break
					.endif
					inc si
				.until si==5
			.endif



			

			mov di,robot[bx].write
			
			mov ax,robot[bx].despos[0]
			dlwait
			mov es:[di+0],al
			
			mov ax,robot[bx].despos[2]
			dlwait
			mov es:[di+2],al
			
			mov ax,robot[bx].despos[4]
			dlwait
			mov es:[di+4],al

			mov ax,robot[bx].despos[6]
			dlwait
			mov es:[di+6],al

			
			mov ax,128              ; Axis doen't really exist
			dlwait                  ; but this allows Nep II to simulate
			mov es:[di+8],al
			
			
			mov ax,robot[bx].despos[8]
			dlwait
			mov es:[di+10],al

			mov ax,robot[bx].despos[10]
			dlwait
			mov es:[di+12],al

			


			mov si,robot[bx].read
			dlwait
			mov al,es:[si]
			dlwait
			mov al,es:[si]
			test ax,2
			.if zero?
				mov robot[bx].WC.DIG_inputs[0],0
			.else
				mov robot[bx].WC.DIG_inputs[0],1
			.endif
			test ax,4
			.if zero?
				mov robot[bx].WC.DIG_inputs[2],0
			.else
				mov robot[bx].WC.DIG_inputs[2],1
			.endif
			test ax,8
			.if zero?
				mov robot[bx].WC.DIG_inputs[4],0
			.else
				mov robot[bx].WC.DIG_inputs[4],1
			.endif


		;.endif
	
	.else
		call service_neptune1_offline
	.endif


	
	mov robot.pulsecount[bx],0
	mov robot.rs232flag[bx],false
	mov robot.Serialflag[bx],false


	ret
service_neptune1 endp


service_neptune1_offline proc
	.if robot[bx].teachmode == init
		mov robot[bx].despos[0],128
		mov robot[bx].despos[2],128
		mov robot[bx].despos[4],128
		mov robot[bx].despos[6],128
		mov robot[bx].despos[8],128
		mov robot[bx].despos[10],128
		mov si,0
		.repeat
			mov ax,robot[bx].despos[si]
			mov robot[bx].lastdespos[si],ax
			mov robot[bx].feedback[si],ax
			add si,2
		.until si==12
	.endif
	
	mov robot[bx].teachmode,keyboard
	mov robot.pulsecount[bx],0
	mov robot.rs232flag[bx],false
	mov robot.Serialflag[bx],false
	mov robot.inpos[bx],TRUE        

	Simulate 0,2
	Simulate 1,2
	Simulate 2,2
	Simulate 3,2
	Simulate 4,2
	Simulate 5,4
	ret
service_neptune1_offline endp


;service_neptune1_pulses proc
;    service_neptune1_pulses
;    ret
;service_neptune1_pulses endp


end

