include cyber.inc

.code

ReadAna Macro HandShake
	
	if HandShake
		.if robot[bx].teachmode != SIMULATOR    ;Analogue inputs will be read with simulator    
			invoke peek_8032, 021H      ; simulator ready?
			bt ax,0
			.if carry?
	endif

				invoke deek_8032, 080H+2*6  ;Read workcell ip 1
				mov robot[bx].WC.ANA_inputs[0],ax
		
				invoke deek_8032, 080H+2*7  ;Read workcell ip 2
				mov robot[bx].WC.ANA_inputs[2],ax

	if HandShake
			.endif
			invoke poke_8032, 021H,0
		.endif
	endif
	endm

ReadAdc macro axis,change,range
	invoke deek_8032, 080H+2*axis
	invoke Filter,axis, ax ,robot[bx].rawlearn[axis*2],change
	mov robot[bx].rawlearn[axis*2],ax

	mov cx,range
	mul cx
	mov cx,4095
	div cx
	mov robot[bx].learn[2*axis],ax
	endm






service_rj45_serpent proc
	Mov robot[bx].SimPres,1


	.if rj45base==0                      ; serial card not present
		call service_RJ45_serpent_offline    
	.else
		.if robot[bx].online!=DEV_OFFLINE
			invoke set8032page, INTERNAL
			invoke poke_8032, scratch,27
			invoke peek_8032, scratch
			.if al==27
				invoke poke_8032, scratch,72
				invoke peek_8032, scratch
				.if al==72
					.if robot[bx].online !=DEV_ONLINE
						invoke set8032page, EPROM
						
						mov si,030h
						mov di,0
						.repeat
							invoke peek_8032,si
							mov robot[bx].prom.promid[di],al
							inc si
							inc di
						.until di==sizeof Prom_struc

						invoke set8032page, INTERNAL
						invoke deek_8032,07eh
						mov robot[bx].prom.promcsum,ax

						mov robot[bx].teachmode,INIT
					.endif
					
					.if robot[bx].prom.promid == 'S'
						call putonline
					.endif
				.else
					call putoffline ; increments robot[bx].offlinecnt
				.endif
			.else
				call putoffline     ; increments robot[bx].offlinecnt
			.endif
		.endif


		.if robot[bx].online==DEV_ONLINE
			.if robot[bx].offlinecnt==0             ; really available ?
				call service_RJ45_serpent_online
			.endif
		.ELSE
			call service_RJ45_serpent_offline    
		.endif
	.endif
	ret
service_rj45_serpent endp


service_rj45_serpent_online proc
	call service_8032_workcell
	ReadAna True                    ; if not in simulator mode read wc inputs

	
	invoke peek_8032, 027H      ; read in position flags
								; bit 7 = zeroing
								; bit 6 = jammed
								; bit 5 = power
	bt ax,5
	.if carry?
		mov robot[bx].power,true
	.else
		mov robot[bx].power,false
	.endif
	
	
	.if robot[bx].teachmode == zeroing
		bt ax,7
		.if !carry?
			mov robot[bx].teachmode,init
		.endif
	.else
		bt ax,7
		.if carry?
			mov robot[bx].teachmode,zeroing

		.else
			.if robot[bx].teachmode == jammed
				bt ax,6
				.if !carry?
					mov robot[bx].teachmode,init
				.endif
			.else
				bt ax,6
				.if carry?
					mov robot[bx].teachmode,jammed
					invoke peek_8032, 026H  ; read which axis is jammed
					mov ah,0
					mov robot[bx].errors,ax
					ret
				.endif
			.endif
		.endif

		;bt ax,5
		;.if carry?
		;    mov robot[bx].power,true
		;.else
		;    mov robot[bx].power,false
		;.endif


		
		and al,01111b        ;ignore gripper  for now
		.if al==01111b
			mov robot[bx].inpos,true
		.else
			mov robot[bx].inpos,false
		.endif


		invoke peek_8032, 024H      ; read pendant active flag
		and al,0100000b
		.if al!=0
			mov robot[bx].inpos,false
			.if robot[bx].teachmode < jammed
				mov robot[bx].teachmode,pendant
			.ENDIF
		.endif


		.if robot[bx].teachmode == SIMULATOR
			invoke peek_8032, 021H      ; simulator ready?
			bt ax,0
			.if carry?
				ReadAna False       ; read workcell inputs
				
				ReadAdc 0,2,6600   ; axis 0
				ReadAdc 1,2,7900   ; axis 1
				ReadAdc 2,2,1250   ; axis 2

				invoke deek_8032, 080H+2*3  ; axis Z
				.if ax>2048
					mov robot[bx].learn[2*3],1
				.else
					mov robot[bx].learn[2*3],0
				.endif

				invoke deek_8032, 080H+2*4  ; Gripper
				.if ax>2048
					mov robot[bx].learn[2*4],1
				.else
					mov robot[bx].learn[2*4],0
				.endif
				
				mov si,0
				.repeat
					mov ax,robot[bx].learn[si]
					mov robot[bx].despos[si],ax
					add si,2
				.until si==10
				
				invoke poke_8032, 021H,0
			.endif
		.endif



		invoke deek_8032, 070H          ;Axis 0
		.if ax>1024
			sub ax,1024
		.else 
			mov ax,0
		.endif
		mov robot[bx].feedback[0],ax
	
		invoke deek_8032, 072H          ; Axis 1
		.if ax>1024
			sub ax,1024
		.else 
			mov ax,0
		.endif
		mov robot[bx].feedback[2],ax

		invoke deek_8032, 074H          ; Axis 2
		.if ax>1024
			sub ax,1024
		.else 
			mov ax,0
		.endif
		mov robot[bx].feedback[4],ax
		
		
		invoke peek_8032, 06aH              ; current Desired gripper state (was 7a = feedback)
		bt ax,6
		.if carry?
			mov robot[bx].feedback[6],0    ; Up
		.else
			mov robot[bx].feedback[6],1    ; Down
		.endif
		
		bt ax,4
		.if carry?
			mov robot[bx].feedback[8],1    ; Shut
		.else
			mov robot[bx].feedback[8],0    ; Open
		.endif
		
		
		invoke peek_8032, 07bH              ; current speed
		mov robot[bx].feedback[10],ax
								   
		
		
		.if (robot[bx].teachmode==INIT ) || (robot[bx].teachmode==Pendant) || (robot[bx].teachmode==JAMMED)
			mov si,0
			.repeat
				mov ax,robot[bx].feedback[si]
				mov robot[bx].despos[si],ax
				mov robot[bx].lastdespos[si],ax
				add si,2
			.until si==18
			.if robot[bx].teachmode==INIT
				mov robot[bx].teachmode,keyboard
			.endif
		
		.elseif  robot[bx].teachmode==NOSELEAD
			
			mov si,0
			.repeat
				mov ax,robot[bx].feedback[si]
				mov robot[bx].despos[si],ax
				mov robot[bx].lastdespos[si],ax
				add si,2
			.until si==6   ; Z/Gripper/Speed  still set by despos

		.endif


		.if robot[bx].teachmode != zeroing
			mov si,0
			mov di,060h                                 ; start address in serpent
			.repeat
				mov cx,robot[bx].despos[si]             ; desired position
				.if robot[bx].resend==true
					mov robot[bx].lastdespos[si],-1
				.endif

				.if cx!=robot[bx].lastdespos[si]        ; already sent?
					mov robot[bx].lastdespos[si],cx     ; no, take copy
					add cx,1024                         ; shift, saves negative encoders
					invoke doke_8032,di,cx              ; write it out
				.endif
				add di,2
				add si,2
			.until si==8
			mov robot[bx].resend,false
		
			mov cx,robot[bx].despos[6]             ; desired Z
			.if cx==0
				mov dx,64
			.else
				mov dx,32
			.endif
		
			mov cx,robot[bx].despos[8]             ; desired gripper
			.if cx==1
				or dx,16
			.endif
			
			.if dx!=robot[bx].lastdespos[8]        ; already sent?
				mov robot[bx].lastdespos[8],dx
				invoke poke_8032,06ah,dx        ; write it out
			.endif


			mov cx,robot[bx].despos[10]             ; desired speed
			.if cx!=robot[bx].lastdespos[10]        ; already sent?
				mov robot[bx].lastdespos[10],cx     ; no, take copy
				invoke poke_8032,06bh,cx            ; write it out
			.endif
		.endif

		.if (robot[bx].inpos==true) || (robot[bx].power==false)
			mov si,0
			.repeat
				.if robot[bx].stop[si]!=false
					mov robot[bx].stop[si],false 
					mov ax,robot[bx].feedback[si]
					mov robot[bx].despos[si],ax
					mov robot[bx].lastdespos[si],ax
				.endif
				add si,2
			.until si==6        ;do for axes 0,1 and 2


			.if robot[bx].stop[10]!=false
				mov robot[bx].stop[10],false 
				mov ax,robot[bx].feedback[10]
				mov robot[bx].despos[10],ax
				mov robot[bx].lastdespos[10],ax
			.endif

			
			;.repeat
			;    .if robot[bx].stop[si]!=false
			;        mov robot[bx].stop[si],false 
			;    .endif
			;    add si,2
			;.until si==18
		.endif

		.if robot[bx].stop[6]!=false
			mov robot[bx].stop[6],false 
			mov ax,robot[bx].feedback[6]
			mov robot[bx].despos[6],ax
		.ENDIF
					
		.if robot[bx].stop[8]!=false
			mov robot[bx].stop[8],false 
			mov ax,robot[bx].feedback[8]
			mov robot[bx].despos[8],ax
		.ENDIF



	.endif

	ret
service_rj45_serpent_online endp



service_rj45_serpent_offline proc
	
;    .if robot[bx].teachmode == zeroing
;        mov robot[bx].teachmode,keyboard
 ;   .endif

	
	.if robot[bx].teachmode == init
;        mov robot[bx].teachmode,keyboard
		mov robot[bx].despos[0],3000
		mov robot[bx].despos[2],3000
		mov robot[bx].despos[4],1000
		mov robot[bx].despos[6],1
		mov robot[bx].despos[8],1
		mov robot[bx].despos[10],0
		mov robot[bx].despos[12],1
		mov robot[bx].despos[14],1
		mov robot[bx].despos[16],3
		mov si,0
		.repeat
			mov ax,robot[bx].despos[si]
			mov robot[bx].lastdespos[si],ax
			mov robot[bx].feedback[si],ax
			add si,2
		.until si==18

		mov di,0
		.repeat
			mov robot[bx].prom.promid[di],'?'
			inc di
		.until di==sizeof Prom_struc
	.endif
	mov robot[bx].teachmode,keyboard

	
	mov robot.inpos[bx],TRUE        
	
	simulate 0,100          ; simulate movement of axis 0, increment of 100
	simulate 1,100          ; simulate movement of axis 1, increment of 100
	simulate 2,100          ; simulate movement of axis 2, increment of 100
	simulate 3,10
	simulate 4,10
	simulate 5,1
	
	simulate 8,3

	
	
	mov robot.pulsecount[bx],0
	mov robot.rs232flag[bx],false
	mov robot.Serialflag[bx],false
	mov robot.mesgflag[bx],false

	ret
service_RJ45_Serpent_offline endp

				

end

