SUBTTL Constants     
TITLE **  Control Program for 8032 Serpent - Version 1.0 16/05/95 **

PLEN 100

LOAD
    
LIST ; to allow 'L' switch at command line


NULL        equ 0     ;timer 0/1 function call - set r4/5


EPROM       equ 0ffh   ; IBM peek EPROM page
INTERNAL    equ 0feh   ; IBM peek internal RAM page
window      equ 99



FASTEST     equ 12
SLOWEST     equ 9





maxdelay    EQU 0      ; Maximum seed value for timer2 


; HARDWARE ADDRESSES


lsblatch        equ     0a000h  ; latch for lsb address
intlatch        equ     0a001h  ; interrupt from IBM, Ext0
datalatch       equ     0a002h  ; 

workcell_i      equ     0c600h  ; inputs from workcell          - r
workcell_o      equ     0ca00h  ; outputs to workcell           - w

datastrobe      equ     0d200h  ; write strobe at RJ45 interface
dataoen         equ     0d600h  ; data out enable

slave0          equ     0dc00h  ; encoder slave letterbox address r/w
pwm_01          equ     0dc40h  ; PWM axes 0 and 1 speed    
pwm_23          equ     0dc41h  ; PWM axes 2 and 3 speed      
pwm_dir         equ     0dc42h  ; PWM direction



; INTERNAL RAM MAP



rbank0          equ     00h      ; register bank 0 - Baseloop
rbank1          equ     08h      ; register bank 1 - IBM interrupt
rbank2          equ     010h     ; register bank 2 - 
rbank3          equ     018h     ; register bank 3  

lastjam    equ 20h
last_stalled0 bit lastjam.0 
last_stalled1 bit lastjam.1 
last_stalled2 bit lastjam.2 


adccontrol equ 21h
adcvalid    bit adccontrol.0    ; set by 8032, cleared by IBM


wco_stat        equ     023h     ; states of workcell outputs - FIXED LOCATION
wco_stat0       bit     wco_stat.0
wco_stat1       bit     wco_stat.1
wco_stat2       bit     wco_stat.2
wco_stat3       bit     wco_stat.3
wco_stat4       bit     wco_stat.4
wco_stat5       bit     wco_stat.5
wco_stat6       bit     wco_stat.6
wco_stat7       bit     wco_stat.7

flags       equ 024h
pen0        bit flags.0
pen1        bit flags.1
pen2        bit flags.2
pen3        bit flags.3
pen4        bit flags.4

pen_active  bit flags.5
enable_pen  bit flags.6



stallflags equ 025h
stalled0   bit stallflags.0
stalled1   bit stallflags.1
stalled2   bit stallflags.2


stallflags_ibm equ 026h ; copy for IBM to read
    

inpos       equ 027h          ;fixed location
axis0_inpos bit inpos.0
axis1_inpos bit inpos.1
axis2_inpos bit inpos.2


axis5_inpos bit inpos.5
jammed      bit inpos.6
zeroing     bit inpos.7


flags2 equ inpos+1

grip_press bit flags2.0
ibm_mode   bit flags2.1
enable_jam bit flags2.2
inner0     bit flags2.3
inner1     bit flags2.4
inner2     bit flags2.5
inner3     bit flags2.6
inner4     bit flags2.7


AxesDir         equ     flags2+1          ;direction word for axes 0,1 & 2
Dir0    bit AxesDir.0
Dir1    bit AxesDir.1
Dir2    bit AxesDir.2



timer2_func     equ     axesdir+1
wco_low         equ     timer2_func+1
wco_high        equ     wco_low +1
wco_next        equ     wco_high  +1
pulse_width     equ     wco_next +1

control01       equ     pulse_width+1       ;Control word for axes 0 and 1
control2        equ     control01+1         ;control word for axis 2



stall0  equ control2+1
stall1  equ stall0+1
stall2  equ stall1+1



last_encpos0 equ stall2+1    
last_encpos1 equ last_encpos0+2
last_encpos2 equ last_encpos1+2



led_cnt         equ     last_encpos2+2
led_cnt_seed    equ     led_cnt+1

Simspd0         equ     led_cnt_seed+1
Simspd1         equ     Simspd0+1


mcntrl          equ     050H            ; FIXED LOCATION
wcntrl          equ     mcntrl+1        ; workcell control
scntrl          equ     wcntrl+1
mcntrl_copy     equ     scntrl+1   
pen_copy        equ     mcntrl_copy+1

rs232_mess      equ     05bh            ; Byte to send as "RS232" via workcell
pulses_no       equ     rs232_mess+1    ; number of workcell pulses to be sent
pulses_rate     equ     pulses_no+2     ; rate to send workcell pulses


despos0         equ 060h
despos1         equ 062h
despos2         equ 064h


despos_grip     equ 06ah        ; Bit 4=gripper, Bits 5/6 = Z axis
despos_speed    equ 06bh
maxerror        equ 06ch
seqcnt          equ 06dh

encpos0         equ 070h
encpos1         equ 072h
encpos2         equ 074h


encpos_grip     equ 07ah
encpos_speed    equ 07bh

seqptr          equ 07ch

csum            equ 07eh      


rawadc          equ 080h            ;Result from ADC chip


stack           equ     090H


runcnt          equ 0f0h
stallcnt0       equ runcnt+2
stallcnt1       equ stallcnt0+1
stallcnt2       equ stallcnt1+1



scratch         equ 0ffh        ; used only by IBM    



; PROGRAM MEMORY MAP

PAGE 
SUBTTL Interrupt Vectors

base           equ     08000h  ; alternatives for RAM and ROM versions
;base           equ     0h

ORG     base
PAD     0ffh                    ; to fill unallocated locations


; SET-UP INTERRUPT VECTORS AT START OF PROGRAM SPACE


ORG     base+0h                 ; reset vector address
        jmp     begin           ; start of initialisation

ORG     base+03h                ; external interrupt 0 vector address
        jmp     ibmint          ; used for IBM communications

ORG     base+0bh                ; timer0 interrupt vector
        jmp timer0

ORG     base+13h                ; external interrupt 1 vector address
        reti

ORG     base+1bh                ; timer1 interrupt vector
        RETI

ORG     base+23h                ; serial port vector address
        reti

org     base+2bh                ; timer2 interrupt vector
        jmp     timer2
        ;reti

ORG     base+30h                ; to be tidy


; PROM IDENTIFICATION


        db      "SERP","1.00"     ; Gryphon program, version 1

        db      "16/05/95 ",169," Cybernetic Applications 1995",0    ; Null terminated




PAGE
     
SUBTTL INITIALISATION / RESET

;nb. uses register set 0, as does Baseloop

begin:
        mov  sp,#stack          ; initialise stack pointer 
        
        mov  dptr,#intlatch     ; clear possible IBM interrupt states
        movx a,@dptr

        mov dptr,#Slave0        ; Clear possible interrupt state on encoder slave
        movx a,@dptr
        
        
        clr a                   ; Clear all internal RAM
        mov r0,#255
clearram:
        mov @r0,a
        djnz r0,clearram
        
        
        mov  wco_stat,#0ffh     ; Set all workcell outputs high
        mov  a,wco_stat         ; send to workcell outputs
        mov  dptr,#workcell_o
        movx @dptr,a            

        mov  dptr,#0ce00h       ; send gripper/Z data
        mov  a,despos_grip
        mov  encpos_grip,a
        movx @dptr,a

;        lcall initadc
        
        
        
        
        mov dptr,#0         ; Calculate EPROM checksum
        mov r0,#0
        mov r1,#0
gc1:
        clr a
        movc a,@a+dptr

        add a,r0
        mov r0,a

        clr a
        addc a,r1
        mov r1,a

        inc dptr
        mov a,dph
        cjne a,#20h,gc1     ; done all EPROM?

        mov csum,r0      ; save for IBM
        mov csum+1,r1

        mov led_cnt,#8          ; set led flash freq    
        mov led_cnt_seed,#8



; Set-up Interrupts and their Priorities


        mov rcap2l,#low  maxdelay
        mov rcap2h,#high maxdelay
        mov t2con,#00000100b    ; timer2, 16bit autoreload timer

        ; To enable serial debugger:        
        ; replace following

;       mov t2con,#00110100b    ; serial generat
;       mov rcap2l,#low  65484  ; gives 9615 baud
;       mov rcap2h,#high 65484
;       mov scon,#01000000b
;       mov sbuf,a
        
        mov  tmod,#00010001b    ; timer0, timer1 both 16 bit timers
        mov  tcon,#00010000b    ; ext0 and ext1 level triggered, timer 0 running
        
        mov  ie  ,#10100011b    ; enable ext0 timer2,timer0  
        mov  ip  ,#00000001b    ; ext0 high priority


    clr  jammed
    setb enable_jam

    mov despos_speed,#3         ; Default to fastest
    acall setmaxerror

    acall home
;    clr enable_jam


    page
     
     
    SUBTTL Baseloop

;    lcall startadc
    
    setb enable_pen    
    

    

BASELOOP:
    ;DEBUG CODE
    
;    clr   a 
;    mov   r0,#encpos0    
;    lcall update_posn
;    clr a
;    
;    lcall sendcontrol
;    mov maxerror,#FASTEST
    
    
    mov a,despos_speed    
    mov encpos_speed,a
    mov maxerror,a
    Lcall servoarm_1
    
    
    JMP baseloop


    ;DEBUG CODE





    lcall readadc

    mov a,mcntrl
    jz bl1

    mov mcntrl,#0
    mov mcntrl_copy,a
    mov dptr,#master_decode
    jmp @a+dptr             ; will return to bl0
bl0:
    mov scntrl,mcntrl_copy


bl1:
    
    mov dptr,#0ce00h        ; read pendant
                            ; bits 0,1,2,3 = function
                            ; bit 4 = gripper
                            ; bit 5 = speed
    movx a,@dptr
;    jb acc.7,bl1a           ; power on?
;    acall nopower
;    jmp baseloop
;
;
;bl1a:
    jnb enable_pen,bl2
    anl a,#63
    mov pen_copy,a
    cjne a,#63,bl4          ; any key pressed
    clr pen_active          ; no - speed will be set by IBM
    jmp bl5
bl4:
    setb pen_active         ; yes - put in charge
    mov maxerror,#FASTEST
    jb  acc.5,bl5           
    mov maxerror,#SLOWEST

bl5:
    
    jnb acc.4,bl6           ; gripper switch down
    clr grip_press          ; no
    jmp bl7
bl6:    
    jb   grip_press,bl7     ; yes, but it was last time as well
    setb grip_press         ; mark pressed, for next time
    mov a,despos_grip       ; toggle the gripper
    cpl acc.4
    mov despos_grip,a
    
bl7:
    
    anl a,#15 
    rl  a
    mov dptr,#pen_decode
    jmp @a+dptr             ; will return to bl2
bl2:
    
    lcall testjams_noread
    
    jb pen_active,bl3
    
    acall setmaxerror

bl3:    
    
    mov   a,stallflags      ; any axis jammed?
    jz    bl8

    mov  stallflags_ibm,a   ; tell ibm which axes are jammed
    mov  stallflags,#0      ; re-enable servo
                            ; want to be able to use pendant to back off
    
;    mov a,stalledcnt
;    jz bl9
;    djnz stalledcnt,bl8     ; only report multiple jams
;                            ; stalledcnt incremented by timer
;bl9:    
    setb jammed             ; tell IBM

emer_stop:
    orl flags,#01011111b     ; == setb pen0 etc   
    ;setb  pen0              ; causes despos=encpos when stopped
    ;setb  pen1              ; causes despos=encpos when stopped
    ;setb  pen2              ; causes despos=encpos when stopped
    ;setb  pen3              ; causes despos=encpos when stopped
     
     
    ;setb  pen4              ; causes despos=encpos when stopped
    ;setb  enable_pen
    JMP BASELOOP


bl8:
    acall servoarm
    JMP BASELOOP


nopower:
    clr   a 
    mov   r0,#encpos0       ; point to data to change
    lcall update_posn
    clr a
    lcall send_control0
    
    mov   a,#2
    mov   r0,#encpos1       ; point to data to change
    lcall update_posn
    clr a
    lcall send_control1
    
    mov   a,#4
    mov   r0,#encpos2         ; point to data to change
    lcall update_posn
    clr a
    lcall send_control2

    

    mov  dptr,#0ce00h       ; send gripper/Z data
    mov  a,despos_grip
    mov  encpos_grip,a
    movx @dptr,a

    ret




servoarm:
    mov dptr,#0ce00h
    movx a,@dptr
    jb acc.7,servoarm_1     ; power on?
    acall nopower
    ret

servoarm_1:
    clr   a 
    mov   r0,#encpos0       ; point to data to change
    lcall update_posn
    acall servo0
    
    mov   a,#2
    mov   r0,#encpos1       ; point to data to change
    lcall update_posn
    acall servo1
    
    mov a,#4
    mov r0,#encpos2         ; point to data to change
    lcall update_posn
    acall servo2


    mov  dptr,#0ce00h       ; send gripper data
    mov  a,despos_grip
    mov  encpos_grip,a
    movx @dptr,a

    ret


pen_decode:
    ajmp pen_0m         ;0
    ajmp pen_0p         ;1
    ajmp pen_1m         ;2
    ajmp pen_1p         ;3
    ajmp pen_2m         ;4
    ajmp pen_2p         ;5
    ajmp bl2            ;6
    ajmp bl2            ;7
    ajmp bl2            ;8
    ajmp bl2            ;9
    ajmp bl2            ;10     
    ajmp pen_runin      ;11
    ajmp bl2            ;12
    ajmp bl2            ;13
    ajmp bl2            ;14
    ajmp pen_end        ;15


master_decode:                   ; jump table for IBM intructions
    ajmp bl0                     ; 0 - unused
    ajmp nojams                  ; 2 - turn arm jam off
    ajmp jams                    ; 4 - turn arm jams back on
    ajmp nopen                   ; 6 - turn pendant off
    ajmp emer_stop               ; 8 
    ajmp jog_0m                  ; 10 - move axis 0 towards 0   
    ajmp jog_0p                  ; 12
    ajmp jog_1m                  ; 14 - move axis 1 towards 0   
    ajmp jog_1p                  ; 16
    ajmp jog_2m                  ; 18 - move axis 2 towards 0   
    ajmp jog_2p                  ; 20
    ajmp jog_end                  ; 22 - move axis 3 towards 0   
    ajmp jog_end                  ; 24
    ajmp jog_end                  ; 26 - move axis 4 towards 0   
    ajmp jog_end                  ; 28
    ajmp jog_5m                  ; 30 - open gripper  
    ajmp jog_5p                  ; 32
    ajmp jog_end                  ; 34 - move axes 3 and 4 towards 0   
    ajmp jog_end                  ; 36
    ajmp jog_end                  ; 38 - move axes 3 towards 0  4 +ve 
    ajmp jog_end                  ; 40
    ajmp jog_sm                  ; 42 - Speed -ve
    ajmp jog_sp                  ; 44 - Speed +ve  
    
    ajmp jog_end                 ; 46 - end of jog, re-enable pendant
    ajmp jog_end                 ; 48
    ajmp jog_clearjam            ; 50
    ajmp jog_home                ; 52
    ajmp jog_reset               ; 54

nopen:
    clr enable_pen
    ajmp bl0

nojams:
    clr  enable_jam
    ajmp bl0

jams:
    setb enable_jam
    ajmp bl0

jog_clearjam:
    clr jammed                  ; clear arm jammed flag
    mov stallflags,#0
    mov stall0,#10
    mov stall1,#10
    mov stall2,#10
    
    


    ajmp bl0

jog_sm:
    mov a,despos_speed
    jz  sm_1
    dec a
sm_1:    
    mov despos_speed,a
    ajmp bl0

jog_sp:
    mov a,despos_speed
    cjne a,#3,sp_1
    ajmp bl0
     
     
sp_1:    
    inc a
    mov despos_speed,a
    ajmp bl0


jog_0m:
    clr enable_pen
    ajmp pen_0m
jog_0p:
    clr enable_pen
    ajmp pen_0p
jog_1m:
    clr enable_pen
    ajmp pen_1m
jog_1p:
    clr enable_pen
    ajmp pen_1p
jog_2m:
    clr enable_pen
    ajmp pen_2m
jog_2p:
    clr enable_pen
    ajmp pen_2p

jog_Zm:
    mov a,despos_grip   
    clr  acc.5 
    Setb acc.6
    mov despos_grip,a
    ajmp bl2 

jog_Zp:
    mov a,despos_grip   
    clr  acc.6 
    Setb acc.5
    mov despos_grip,a
    ajmp bl2 


jog_5m:
    mov a,despos_grip   
    clr acc.4               ; Clear griiper bit
    mov despos_grip,a
    ajmp bl2 

jog_5p:
    mov a,despos_grip   
    Setb acc.4              ; Set griiper bit
    mov despos_grip,a
    ajmp bl2 



jog_end:
    setb enable_pen
    ajmp pen_end
jog_home:
jog_reset:

    jmp bl0


pen_0m:
    mov despos0  , #low 1024
    mov despos0+1, #high 1024
    setb pen0
    ajmp bl2
pen_0p:
    mov despos0  , #low 10024       ; 9000 + 1024 = 10024       
    mov despos0+1, #high 10024
    setb pen0
    ajmp bl2

pen_1m:
    mov despos1  , #low 1024
    mov despos1+1, #high 1024
     
     
    setb pen1
    ajmp bl2
pen_1p:
    mov despos1  , #low 6524        ; 5500 + 1024
    mov despos1+1, #high 6524
    setb pen1
    ajmp bl2

pen_2m:
    mov despos2  , #low 1024
    mov despos2+1, #high 1024
    setb pen2
    ajmp bl2
pen_2p:
    mov despos2  , #low 10974       ; 9950 + 1024 = 10974
    mov despos2+1, #high 10974
    setb pen2
    ajmp bl2 


pen_end:
    jnb pen0,pen_end1
    mov despos0  ,encpos0
    mov despos0+1,encpos0+1
    acall servo0
    jnb axis0_inpos,pen_end1
    clr pen0

pen_end1:
    jnb pen1,pen_end2
    mov despos1  ,encpos1
    mov despos1+1,encpos1+1
    acall servo1
    jnb axis1_inpos,pen_end2
    clr pen1

pen_end2:
    jnb pen2,pen_end5
    mov despos2  ,encpos2
    mov despos2+1,encpos2+1
    acall servo2
    jnb axis2_inpos,pen_end5
    clr pen2


pen_end5:

    ajmp bl2




pen_runin:
    
    clr a 
    mov r0,#runcnt
    
pen_clear:    
    mov @r0,a
    inc r0
    cjne r0,#stallcnt2 + 1,pen_clear


    mov seqptr  ,#low runseq        ; set pointer to start of sequence
    mov seqptr+1,#high runseq
    mov seqcnt,#1
    mov seqcnt+1,#1

    ajmp pen_run2                   ; send first line
    

pen_run0:    
    mov  a,mcntrl
    jz   pen_run1
    ajmp baseloop       ; return to baseloop
    
pen_run1:
    mov stallflags,#0   ; reset jmammed axes

    mov a,inpos
    anl a,#01fh
    cjne a,#01fh,pen_run4
                         ;arm inposition, set up new data

pen_run2:
    djnz seqcnt  ,pen_run4
    djnz seqcnt+1,pen_run4

    mov seqcnt+1,#16
    
    mov dpl,seqptr
    mov dph,seqptr+1
    mov r0,#despos_speed

    clr  a
    movc a,@a+dptr
    inc  dptr
    jnz  pen_run3

    mov r0,#runcnt
    inc @r0
    mov a,@r0
    jnz pen_run2a
    
    inc r0
    inc @r0
    mov a,@r0
    jnz pen_run2a
    
    jmp baseloop

pen_run2a:    
    mov seqptr  ,#low runseq    ;at end of sequence point back to start
    mov seqptr+1,#high runseq
    ajmp pen_run2


pen_run3:
    clr a
    movc a,@a+dptr
    mov @r0,a
    inc dptr            ; inc pointer to source
    dec r0              ; dec pointer to dest
    mov a,r0
    cjne a,#despos0-1,pen_run3

    mov seqptr,dpl
    mov seqptr+1,dph

pen_run4:
    acall servoarm
    lcall testjams_noread
    ajmp pen_run0


     

servo0:
    clr  c
    mov  a,despos0
    subb a,encpos0
    mov  r6,a           ; LSB error
    
    mov  a,despos0+1
    subb a,encpos0+1
    mov  r7,a           ; MSB error
    
    mov dir0,c          ; save direction
    jnc setspd0

    clr  c
    mov  a,encpos0
    subb a,despos0
    mov  r6,a           ; absolute LSB error
    
    mov  a,encpos0+1
    subb a,despos0+1
    mov  r7,a           ; absolute MSB error

setspd0:
    anl a,#11111100b
    jz setspd0_1
    
    mov a,maxerror      ; 15 to 0
    jmp setdir0

setspd0_1:
    
    mov a,r7
    add a,  #HIGH error0
    mov dph,a
    mov dpl,#LOW  error0

    acall readerror     ; pointed to by dptr,r6
    
    cjne a,#window,setdir0 
    mov a,#1
    jnb inner0,setdir0
    clr a

setdir0:
    jz   send0_in       ; stopped? - don't add direction
    clr  inner0             
    
;    mov a,#1            ; force bodged speed
;    mov a,maxerror
    
send0:   
    acall send_control0
    ret

send0_in:
    setb inner0         ; real stop open window
    jmp send0





servo1:
    clr  c
    mov  a,despos1
    subb a,encpos1
    mov  r6,a           ; LSB error
    
    mov  a,despos1+1
    subb a,encpos1+1
    mov  r7,a           ; MSB error
    mov dir1,c           ; save direction
    jnc setspd1

    clr  c
    mov  a,encpos1
    subb a,despos1
    mov  r6,a           ; absolute LSB error
    
    mov  a,encpos1+1
    subb a,despos1+1
    mov  r7,a           ; absolute MSB error

setspd1:
    anl a,#11111100b
    jz setspd1_1
    
    mov a,maxerror      ; = 15 to 0 , 99 
    jmp setdir1

setspd1_1:
    
    mov a,r7
    add a,  #HIGH error1
    mov dph,a
    mov dpl,#LOW  error1
    
    acall readerror     ; pointed to by dptr,r6
    cjne a,#window,setdir1
    mov a,#1
    jnb inner1,setdir1
    clr a

setdir1:
    jz   send1_in       ; stopped? - don't add direction
    clr  inner1
;    mov a,#1            ; force bodged speed
;    mov a,maxerror
    
send1:   
    acall send_control1
    ret

send1_in:  
    setb inner1
    jmp send1



servo2:
    clr  c
    mov  a,despos2
    subb a,encpos2
    mov  r6,a           ; LSB error
    
    mov  a,despos2+1
    subb a,encpos2+1
    mov  r7,a           ; MSB error
    mov dir2,c          ; save direction
    jnc setspd2

    clr  c
    mov  a,encpos2
    subb a,despos2
    mov  r6,a           ; absolute LSB error
    
    mov  a,encpos2+1
    subb a,despos2+1
    mov  r7,a           ; absolute MSB error

setspd2:
    anl a,#11111100b    ; 1024 error entries for axis 2
    jz setspd2_1
    
    mov a,maxerror      ; = 14 for fast
    jmp setdir2

setspd2_1:
    
    mov a,r7
    add a,  #HIGH error2
    mov dph,a
    mov dpl,#LOW  error2
    
    acall readerror     ; pointed to by dptr,r6
    cjne a,#window,setdir2
    mov a,#1
    jnb inner2,setdir2
    clr a

setdir2:
    jz   send2_in       ; stopped? - don't add direction
    clr  inner2
    
send2:   
    lcall send_control2
    ret
send2_in:
    setb inner2
    jmp  send2





setmaxerror:
    mov a,despos_speed
    mov encpos_speed,a
    jnz spd1
    mov maxerror,#SLOWEST
    ret
spd1:
    cjne a,#1,spd2
    mov maxerror,#3
    ret
spd2:
    cjne a,#2,spd3
    mov maxerror,#7
    ret
spd3:
    mov maxerror,#FASTEST
    ret



readerror:
    mov  a,r6           ; lsb error
    movc a,@a+dptr      ; read unsigned speed from error table

    cjne a,#window,re0
    ret

re0:
    cjne a,maxerror,re1
    ret                 ; a = maxerror, ok
re1:
    jc re2              ; jump if a < maxerror
    mov a,maxerror
re2:
    ret



update_posn:
    mov dptr,#Slave0
    
    movx @dptr,a        ; Request data from encoder slave
    jb p3.3,$           ; wait for slave to respond
    movx a,@dptr        ; Read data from slave
    
;    mov change,a
    jb acc.7,neg_change
    
    add a,@r0
    mov @r0,a
    jnc nomodmsb

    inc r0              ; Increment pointer, to point to msb
    inc @r0             ; increment msb
    ajmp nomodmsb

neg_change:
    cpl a
    inc a
;    mov change,a
    xch a,@r0
    clr c
    subb a,@r0
    mov @r0,a
    
    inc r0
    mov a,@r0
    subb a,#0
    mov @r0,a

nomodmsb:
    

    ret





home:
    mov despos0  , #low 1024
    mov despos0+1, #high 1024
    
    mov despos1  , #low 1024
    mov despos1+1, #high 1024
    
    mov despos2  , #low 1024
    mov despos2+1, #high 1024
    
    mov encpos0  , #low 1024
    mov encpos0+1, #high 1024
    
    mov encpos1  , #low 1024
    mov encpos1+1, #high 1024
    
    mov encpos2  , #low 1024
    mov encpos2+1, #high 1024


    ret


    jmp @a+dptr
hometab:
    ajmp home_0m    ;0
    ajmp home_0p    ;1
    ajmp home_1m    ;2
    ajmp home_1p    ;3
    ajmp home_2m    ;4
    ajmp home_2p    ;5
    ajmp home_null    ;6
    ajmp home_null    ;7
    ajmp home_null    ;8
    ajmp home_null    ;9
    ajmp home_end   ;10

    ajmp home_null  ;11
    ajmp home_null  ;12
    ajmp home_null  ;13
    ajmp home_null  ;14
    ajmp home_null  ;15
    

home_0:
home_0m:    
    mov  a,maxerror
    Clr dir0
    lcall send_control0
    jmp home_0

home_0p:    
    mov  a,maxerror
    setb dir0  
    lcall send_control0
    jmp home_0


home_1m:    
    mov  a,maxerror
    clr dir1
    lcall send_control1
    jmp home_0

home_1p:    
    mov  a,maxerror
    Setb dir1
    lcall send_control1
    jmp home_0

home_2m:    
    mov  a,maxerror
    Clr Dir2
    lcall send_control2
    jmp home_0

home_2p:    
    mov  a,maxerror
    setb Dir2
    lcall send_control2
    jmp home_0



home_null:    
    mov stallflags,#0
    
    clr a
    lcall send_control0
    clr a
    lcall send_control1
    clr a
    lcall send_control2
    
    jmp home_0


home_end:
    mov a,pen_copy
    jb acc.4,home_null
    jb acc.5,home_null

home_end_ibm:


; send control byte to send each axis to its endstop - wait til stalled

    setb enable_jam             ; arm jams need to be recognised
    mov stallflags,#0
    
he_0:
    
he_1:
    
    clr a 
    mov r0,#encpos0     ;point to data to change
    lcall update_posn
    
    mov a,#2
    mov r0,#encpos1     ;point to data to change
    lcall update_posn
    
    mov a,#4
    mov r0,#encpos2     ;point to data to change
    lcall update_posn


    mov stallflags,#0
    
    
;   Note: To avoid negative feedback.   0 == 1024
;   The IBM will adjust data to suit
    
    mov encpos0,  #low   974    ; set axis 0 to move away from endstop
    mov encpos0+1,#high  974
    mov despos0,  #low  1024    ; == 0
    mov despos0+1,#high 1024
    
    
    mov encpos1,  #low  6574    ; set axis 1 to move away from endstop
    mov encpos1+1,#high 6574
    mov despos1,  #low  6524    ; == 5500
    mov despos1+1,#high 6524

    mov encpos2,  #low   974    ; set axis 2 to move away from endstop
    mov encpos2+1,#high  974
    mov despos2,  #low  1024    ; == 0
    mov despos2+1,#high 1024


    
he_4:    
    acall servoarm

    jnb axis0_inpos,he_4
    jnb axis1_inpos,he_4
    jnb axis2_inpos,he_4
    
    

    clr zeroing             ; show IBM we're ready

    ret

home_nojams:
    clr enable_jam
    ajmp home_02

home_jams:
    setb enable_jam
    ajmp home_02

home_decode:
    ajmp home_02        ; 0 - unused
    ajmp home_nojams    ; 2 - turn arm jam off
    ajmp home_jams      ; 4 - turn arm jams back on
    ajmp home_02        ; 6 - unused
    ajmp home_02        ; 8 - unused
                        
    ajmp home_0m        ; 10 - move axis 0 towards 0   
    ajmp home_0p        ; 12
    ajmp home_1m        ; 14 - move axis 1 towards 0   
    ajmp home_1p        ; 16
    ajmp home_2m        ; 18 - move axis 2 towards 0   
    ajmp home_2p        ; 20
    
    ajmp home_02        ; 22 - move axis 3 towards 0   
    ajmp home_02        ; 24
    ajmp home_02        ; 26 - move axis 4 towards 0   
    ajmp home_02        ; 28
    ajmp home_02        ; 30 - open gripper  
    ajmp home_02        ; 32
    ajmp home_02        ; 34 - move axes 3 and 4 towards 0   
    ajmp home_02        ; 36
    ajmp home_02        ; 38 - move axes 3 towards 0  4 +ve 
    ajmp home_02        ; 40
    ajmp home_02        ; 42 - Speed -ve
    ajmp home_02        ; 44 - Speed +ve  
    
    ajmp home_02        ; 46 - cancel ibm_mode   
    ajmp home_end_ibm   ; 48 - finish it off
    ajmp home_02        ; 50
    ajmp home_02        ; 52
    ajmp home_02        ; 54


home_02:

waitpower:
    mov dptr,#0ce00h
    movx a,@dptr
    jnb acc.7,waitpower
    ret


send_control0:              
;a = lower nibble for control01    
    
;If A=1 then use timer to simulate slower speed
    
    cjne a,#1,nosim0
    mov a,SimSpd0
    jnz Sc0_0
    
    mov SimSpd0,#1                  ;timer will be used
    anl control01,#11110000b        ; mask off lower nibble

Sc0_0:    
    ret
    

nosim0:    
    mov SimSpd0,#0
    anl control01,#11110000b        ; mask off lower nibble
    orl control01,a
    jmp sendcontrol


send_control1:    
;a = upper nibble for control01    
    cjne a,#1,nosim1
    mov a,SimSpd1
    jnz Sc1_0
    
    mov SimSpd1,#1                  ; timer will be used
    anl control01,#00001111b        ; mask off upper nibble

Sc1_0:    
    ret
    

nosim1:    
    mov SimSpd1,#0
    anl control01,#00001111b        ; mask off upper nibble
    swap a              ; swap upper and lower nibbles
    orl control01,a
    jmp sendcontrol




    

send_control2:    
    anl control2,#11110000b        ; mask off lower nibble
    orl control2,a
    
sendcontrol:
    mov dptr,#pwm_01
    mov a,control01
    movx @dptr,a

    mov dptr,#pwm_23
    mov a,control2
    movx @dptr,a
    
    mov dptr,#pwm_dir
    mov a,axesdir
    movx @dptr,a


    ret




change_inpos:     
;r0 points to encpos
;r1 points to last_encpos
    
    clr c
    mov a,@r0
    subb a,@r1
    mov r2,a
    inc r0  ; point to msb
    inc r1  ; point to msb
    mov a,@r0
    subb a,@r1
    mov r3,a
    jnc h0_1

    dec r0  ; point back to lsb
    dec r1  ; point back to lsb

    clr c
    mov a,@r1
    subb a,@r0
    mov r2,a

    inc r0  ; point to msb
    inc r1  ; point to msb
    
    mov a,@r1
    subb a,@r0
    mov r3,a

h0_1:                   ; r3/r2 = abs change of position


    mov a,@r0       ; save msb encpos in msb last_encpos
    mov @r1,a

    dec r0  ; point to lsb
    dec r1

    mov a,@r0       ; save msb encpos in msb last_encpos
    mov @r1,a

    ret




testjams:
    mov a,#0
    mov r0,#encpos0     ;point to data to change
    lcall update_posn
    
    mov a,#2
    mov r0,#encpos1     ;point to data to change
    lcall update_posn
    
    mov a,#4
    mov r0,#encpos2     ;point to data to change
    lcall update_posn
    

testjams_noread:
    mov dptr,#0ce00h        
    
    movx a,@dptr
    jb acc.7,testjams_1           ; power on?

    mov stallflags,#0
    ret
    

testjams_1:

    mov a,control01
    anl a,#1111b
    cjne a,#2 ,$+3      ; set carry if a < 2
    jc h1               ; stopped or pigging slow
    jb stalled0,h1

    mov a,stall0        ; counter decremented by timer0
    jnz h1b
    mov stall0,#3

    mov r0,#encpos0
    mov r1,#last_encpos0
    lcall change_inpos  ; r3/r2 will be set to absolute change    

    mov a,r3            ; changed by more than 256 ?
    jnz h1
    
    mov a,r2
    cjne a,#10,$+3     ; set carry if a < 10
    jnc h1              ; moving 

    jnb enable_jam,h1
    jnb last_stalled0,h1a

    mov a,#0
    lcall send_control0
    setb stalled0       ; axis 0 is stalled
    mov r0,#stallcnt0
    inc @r0 
h1a:
    setb last_stalled0
    ajmp h1b
h1:
    clr  last_stalled0
h1b:

    mov a,control01
    swap a
    anl a,#1111b
    cjne a,#2,$+3           ; set carry if a < 2
    jc h2                ; stopped or pigging slow
    jb stalled1,h2
    
    mov a,stall1        ; counter decremented by timer0
    jnz h2b
    mov stall1,#4

    mov r0,#encpos1
    mov r1,#last_encpos1
    lcall change_inpos  ; r3/r2 will be set to absolute change    
    
    mov a,r3            ; changed by more than 256 ?
    jnz h2

;    jnb ti,$
;    clr ti
;    mov sbuf,a
    
    mov a,r2
    cjne a,#5,$+3     ; set carry if a < 5     
    jnc h2            ; moving 

;    MOV A,#1

    jnb enable_jam,h2
    jnb last_stalled1,h2a
    
    mov a,#30
    lcall send_control1
    setb stalled1       ; axis 1 is stalled
    mov r0,#stallcnt1
    inc @r0

h2a:
    setb last_stalled1
    ajmp h2b
h2:
    clr  last_stalled1
h2b:

    mov a,control2
    anl a,#1111b
    cjne a,#2,$+3       ; set carry if a < 2
    jc h3                ; stopped or pigging slow
    
    jb stalled2,h3

    mov a,stall2        ; counter decremented by timer0
    jnz h3b
    mov stall2,#3

    mov r0,#encpos2
    mov r1,#last_encpos2
    lcall change_inpos  ; r3/r2 will be set to absolute change    
    
    mov a,r3            ; changed by more than 256 ?
    jnz h3
    
    mov a,r2
    cjne a,#5,$+3     ; set carry if a < 5     
    jnc h3

;    MOV A,#2
;    clr ti
;    mov sbuf,a
    jnb enable_jam,h3
    jnb last_stalled2,h3a
    mov a,#30
    lcall send_control2
    setb stalled2       ; axis 2 is stalled
    mov r0,#stallcnt2
    inc @r0
h3a:
    setb last_stalled2
    ajmp h3b
h3:
    clr  last_stalled2
h3b:

    ret


;include runin.inc
runseq:


include errtab.inc
include ibmint.inc              ; External interrupt 0 service
include adc.inc
include timer0.inc
include timer2.inc              ; Timer 2



end
