SUBTTL Constants
TITLE  **  Control Program for 8032 Lathe - Version 1.0 30/06/95 **

PLEN 100        ;set page length suitable for Arial 6pt in works

;LOAD ; to send code to PC's serial port

LIST ; to allow 'L' switch at command line


SLOWSPIN equ 6          ;Was 3 24/10/95 but motor stalled too easy

MEDSPIN equ 10         

VPOSMAX equ 12          ;Max rotation was 10*256 on 12/7/96

ScrewTable equ 1000h

XCALIB  equ 0       ; default shift from usw to zero
ZCALIB  equ 0


TOOL2MIN  equ 12*400/3   
TOOL3MIN  equ 3*400/3   


NULL        equ 0     ;timer 0/1 function call - set r4

XASAP       equ 2     ;timer 0 function call - set r4
ZASAP       equ 8     ;timer 0 function call - set r4

XYSTEP      equ 14    ;timer 0 function call - set r4
XYSTEP_END  equ 30


MAN         equ 32    ;timer 0 function call - set r4

XSTEP       equ 34    ;timer 0 function call - set r4
ZSTEP       equ 36    ;timer 0 function call - set r4
;XZSTEP      equ 38    ;timer 0 function call - set r4

XFEED       equ 40
YFEED       equ 42
ZFEED       equ 44

PURGE       equ 46

delay       equ 48



PAUSE       equ 6     ; IBM function call
CONTINUE    equ 8     ; IBM function call  



;Feedrates 50mm/min  E89F
;         100mm/min  F457
;         150mm/min  F83F
;         200mm/min  FA33
;         250mm/min  FB5F
;         300mm/min  FC27
;         350mm/min  FCB6
;         400mm/min  FD21
;         450mm/min  FD74
;         500mm/min  FDB7


LOWFEED     equ 0E89Fh  ; =  50 mm/min
MAXFEED     equ 0FDB7h  ; = 500 mm/min


LOWMANFEED  equ 0E000h  
MAXMANFEED  equ MAXFEED



KHz1        equ 0Fbc2h  ; timer2 load value for 1KHz
maxdelay    EQU 0       ; Maximum seed value for timer2 




EPROM       equ 0ffh   ; IBM peek EPROM page
INTERNAL    equ 0feh   ; IBM peek internal RAM page

tableadd    equ 0200h  ; Start address of control table  


powerlen    equ 3


; HARDWARE ADDRESSES


lsblatch        equ     0a000h  ; latch for lsb address
intlatch        equ     0a001h  ; interrupt from IBM, Ext0
datalatch       equ     0a002h  ; 

;p1.0           is      X direction, positive high
;p1.1           is      X step, pulse low
;p1.2           is      Y direction      
;p1.3           is      Y step
;p1.4           is      Z direction
;p1.5           is      Z step

;p3.0           is      clear 
;p3.1           is      led 

step_vice       equ     0c200h  ; stepper motors/motor control  - w
microsws        equ     0c200h  ; micro-switches                - r
spindle         equ     0c600h  ; spindle (cutter) motor        - w
workcell_i      equ     0c600h  ; inputs from workcell          - r
workcell_o      equ     0ca00h  ; outputs to workcell           - w
manual1         equ     0ce00h  ; manual controls to motors     - r
opto_darl       equ     0ce00h  ; opto-isolated and darlingtons - w
manual2         equ     0ca00h  ; manual controls (unused)      - r

datastrobe      equ     0d200h  ; write strobe at RJ45 interface
dataoen         equ     0d600h  ; data out enable



; INTERNAL RAM MAP



rbank0          equ     00h      ; register bank 0 - 
rbank1          equ     08h      ; register bank 1 - Timer 0/1
rbank2          equ     010h     ; register bank 2 - Timer 2/IBM interrupt
rbank3          equ     018h     ; register bank 3  



XDIRN   bit p1.4
XSTP    bit p1.5
ZDIRN   bit p1.6
ZSTP    bit p1.7

tr2 bit t2con.2




										;Bit addressable 20-2f
dir_nbits       equ     020h            ; direction of motor commands
dir_nbitsZ      bit     dir_nbits.0     ; Z bit, set positive
dir_nbitsX      bit     dir_nbits.1     ; X bit

sentxstep       bit     dir_nbits.3
sentzstep       bit     dir_nbits.5
intused         bit     dir_nbits.7

step_bits       equ     dir_nbits+1
sendxstep       bit     step_bits.0
xdir            bit     step_bits.1
sendzstep       bit     step_bits.4
zdir            bit     step_bits.5
init            bit     step_bits.7 

pagebits        equ     step_bits+1                                        
pagebits0       bit     pagebits.0      ; marker for page @ 200 used/updated
pagebits1       bit     pagebits.1      ;                   300
pagebits2       bit     pagebits.2      ;                   400
pagebits3       bit     pagebits.3      ;                   500
pagebits4       bit     pagebits.4      ;                   600
pagebits5       bit     pagebits.5      ;                   700
pagebits6       bit     pagebits.6      ;                   800
pagebits7       bit     pagebits.7      ;                   900

wco_stat        equ     pagebits+1      ; states of workcell outputs
wco_stat0       bit     wco_stat.0
wco_stat1       bit     wco_stat.1
wco_stat2       bit     wco_stat.2
wco_stat3       bit     wco_stat.3
wco_stat4       bit     wco_stat.4
wco_stat5       bit     wco_stat.5
wco_stat6       bit     wco_stat.6
wco_stat7       bit     wco_stat.7

flags           equ     wco_stat+1
;abort           bit     flags.0         ; set if IBM wants to end table
;last_power      bit     flags.1         ; used to force coninue with power switch
want_all        bit     flags.2
ack_want_all    bit     flags.3
ignore_step     bit     flags.4
no_back         bit     flags.5


door_close      bit flags.0


os_stat         equ     flags+1         ; current states for odds and sods
stepper_reset   bit     os_stat.0
stepper_power   bit     os_stat.1


chuck_A bit os_stat.2   ; Chuck engagment motor
chuck_B bit os_stat.3

key_A bit os_stat.4     ; Chuck key motor
key_B bit os_stat.5

key_Power     bit     os_stat.6
key_reset     bit     os_stat.7








opto_stat       equ     os_stat+1       ; current states for opto/darlingtons


executing       equ     opto_stat+1     ; read only byte for IBM 
exe_table       bit     executing.0     ; executing table  
exe_pause       bit     executing.1     ; paused executing table  
exe_home        bit     executing.2     ; homing
suspend         bit     executing.3
abort           bit     executing.4
door_open       bit     executing.5
calibration     bit     executing.6
spindle_on      bit     executing.7     ; spindle power on/off
										; will ignore continue until power is on




lastp1 equ     executing+1          ;last encoder 0 - 3 states

last0A  bit lastp1.0
last0B  bit lastp1.1

last1A  bit lastp1.2
last1B  bit lastp1.3


Enc0A   bit acc.0
Enc0B   bit acc.1

Enc1A   bit acc.2
Enc1B   bit acc.3


darl_stat equ lastp1+1
ToolA   bit darl_stat.0
ToolB   bit darl_stat.1




ramp_x          equ     darl_stat+1     ; pointers to acceleration ramp table
x_updown        equ     ramp_x+2        ; x and z at same address
x_flat          equ     x_updown+2
x_pattern       equ     x_flat+2
tl0_seed        equ     x_pattern+1
th0_seed        equ     tl0_seed+1

ramp_z          equ     th0_seed+1
z_updown        equ     ramp_z+2
z_flat          equ     z_updown+2
z_pattern       equ     z_flat+2


control_byte    equ     z_pattern+1      ; current control byte

led_cnt         equ     control_byte+1
led_cnt_seed    equ     led_cnt+1



feedrate        equ     led_cnt_seed+1      ; feed rates set from miscellaneous

power_cnt      equ     feedrate+4

x_backlash      equ power_cnt+1

z_backlash      equ x_backlash+1

r4_seed         equ z_backlash+1

t2_intcnt       equ r4_seed+1

Xmax            equ t2_intcnt+1
Zmax            equ Xmax+2


Vpos    equ Zmax+2      ;Vice position

door_cnt equ Vpos+2


mcntrl          equ     050H            ; 
wcntrl          equ     mcntrl+1        ; workcell control
scntrl          equ     wcntrl+1
Xpos            equ     scntrl+1        ; current X position from home
Zpos            equ     Xpos+2
curr_line       equ     Zpos+2          ; 'miscellaneous' statement of current line

rs232_mess      equ     curr_line+4     ; Byte to send as "RS232" via workcell
pulses_no       equ     rs232_mess+1    ; number of workcell pulses to be sent
pulses_rate     equ     pulses_no+2     ; rate to send workcell pulses
jog_feed_rate   equ     pulses_rate+2   ; feed rate set direct   
spindle_speed   equ     jog_feed_rate+2 ; spindle motor speed set direct


table_marker    equ     spindle_speed+1 ; control pages used by 8031, bitmap
update_marker   equ     table_marker+1  ; bitmap, control pages IBM updated 
Tool            equ     update_marker+1 
Spare           equ     Tool+1


mcntrl_copy     equ     spare+1     ; master control byte returned

;x_calib         equ     mcntrl_copy+1
;z_calib         equ     x_calib+2

jogcnt          equ     mcntrl_copy+1       ;z_calib+2


timer2_func     equ     jogcnt+2
wco_low         equ     timer2_func+1
wco_high        equ     wco_low +1
wco_next        equ     wco_high  +1
pulse_width     equ     wco_next +1



tool1_xmax       equ     070h       ;Fixed locations
tool2_xmax       equ     072h
tool3_xmax       equ     074h
tool4_xmax       equ     076h

tool1_zmax       equ     078h
tool2_zmax       equ     07ah
tool3_zmax       equ     07ch
tool4_zmax       equ     07eh


scratch         equ     080H        ; only indirect addressable

stack           equ     090H



; EXTERNAL RAM MAP


shared          equ     0h          ; shared for IBM communications
prom_idsp       equ     shared      ; identification copied from PROM
version         equ     prom_idsp+4 ; version copied from PROM
copyright       equ     version+4
csum            equ     copyright+64 


		



workspace       equ     100h

off_sets        equ     workspace   ; XYZ offsets from miscellaneous
xz_steps        equ     off_sets+6  ; asap moves
ret_addr        equ     xz_steps+4 ; return address from misc functions


; Mill mechanical constants

;XMAX equ 7620   ;(2.25 * 25.4 * 400/3)  
;ZMAX equ 20320  ;(   6 * 25.4 * 400/3)  



; PROGRAM MEMORY MAP

PAGE
SUBTTL Interrupt Vectors

;base            equ     0a000h  ; alternatives for RAM and ROM versions
base           equ     0h

ORG     base
PAD     0ffh                    ; to fill unallocated locations


; SET-UP INTERRUPT VECTORS AT START OF PROGRAM SPACE


ORG     base+0h                 ; reset vector address
		jmp     begin           ; start of initialisation

ORG     base+03h                ; external interrupt 0 vector address
		jmp     ibmint          ; used for IBM communications

ORG     base+0bh                ; timer0 interrupt vector
		jmp     timer0

ORG     base+13h                ; external interrupt 1 vector address
		reti                    ; may use for software interrupts

ORG     base+1bh                ; timer1 interrupt vector
;        jmp     timer1
		reti

ORG     base+23h                ; serial port vector address
		reti

org     base+2bh                ; timer2 interrupt vector
		jmp     timer2

ORG     base+30h                ; to be tidy


; PROM IDENTIFICATION

prom_id:

;    db      "LATH","1.00"     ; lathe program, version 1.00
;    db      "25/03/96 (C) Cybernetic Applications 1996",0    ; Null terminated

;     db      "LATH","1.01"     
;     db      "09/07/96 (C) Cybernetic Applications 1996",0    ; Null terminated


     db      "LATH","1.02"     
     db      "06/02/97 (C) Cybernetic Applications 1996",0    ; Null terminated


;1.01 Fixes - RS232 outputs, chuck motor current reduction
;1.02 Door switches



PAGE
SUBTTL INITIALISATION / RESET

;nb. uses register set 0, as does Baseloop

begin:
		mov  sp,#stack          ; initialise stack pointer 
		mov  psw,#0             ; set register set 0

		mov  dptr,#intlatch     ; clear any interrupt states
		movx a,@dptr
		
		clr   a
		mov   os_stat,a
		lcall set_spindle_speed ; - switch off spindle
		
		mov r0,#255
clear_internalram:
		mov @r0,a
		dec r0
		cjne r0,#8,clear_internalram
		
		
		mov dptr,#0             ; cosmetic - added 12/11/94 
		clr a                   ; Clear all RAM (32K) to 0
		mov r0,#0
		mov r1,#80h
clearram:
		movx @dptr,a
		inc dptr
		djnz r0,clearram
		djnz r1,clearram






; Set-up any values        


;XMAX equ 7620   ;(2.25 * 25.4 * 400/3)  
;ZMAX equ 20320  ;(   6 * 25.4 * 400/3)  
		mov   tool,#1
		lcall Set_Toolmax



		mov jog_feed_rate,#LOW MAXFEED      ; set default jog rate
		mov jog_feed_rate+1,#HIGH MAXFEED

		mov pulses_rate  ,#low  KHz1         ; set default pulse rate
		mov pulses_rate+1,#high KHz1

		
		mov  dptr,#step_vice    ; reset stepper motor controller chip
		mov  os_stat,#0ffh
		
		;setb stepper_reset      ; Stepper reset high
		;setb stepper_xy_power   ; power save
		;setb stepper_z_power    ; power save


		mov  a,os_stat
		movx @dptr,a            ; Write to control port

		mov r0,#10
		djnz r0,$
		
		clr  stepper_reset      ; Stepper reset low
		mov  a,os_stat
		movx @dptr,a            ; Write to control port
		
		mov r0,#10
		djnz r0,$
		
		setb stepper_reset      ; Stepper reset high
		mov  a,os_stat
		movx @dptr,a            ; Write to control port

		mov  opto_stat,#0
		mov mcntrl,#0           ; clear master control byte
		mov update_marker,#0

		mov table_marker,#0ffh  ; mark all table pages used/invalid
		mov  wco_stat,#0ffh     ; Set all workcell outputs high
		
		mov  a,wco_stat         ; send to workcell outputs
		mov  dptr,#workcell_o
		movx @dptr,a            


		mov a,#15               ; set default number of backlash
		mov x_backlash,a        ; compensation steps
		mov z_backlash,a


;        mov x_calib,  #low  XCALIB    ; set default calibration steps
;        mov x_calib+1,#high XCALIB
;        
;        mov z_calib,  #low  ZCALIB
;        mov z_calib+1,#high ZCALIB


; Copy Identification for IBM to read

								; Copy Null terminated PROM id to RAM
		mov  dptr,#prom_id      ; Start address in PROM
		mov  r0,#prom_idsp      ; Destination address in RAM
copy1:
		clr a
		movc a,@a+dptr   
		jz copy2                ; found null
		
		mov p2,#0               ; reset p2 
		movx @r0,a
		
		inc dptr
		inc r0
		ajmp copy1 
copy2:


		mov dptr,#0         ; Calculate EPROM checksum
		mov r0,#0
		mov r1,#0
gc1:
		clr a
		movc a,@a+dptr

		add a,r0
		xch a,r0

		clr a
		addc a,r1
		xch a,r1

		inc dptr
		mov a,dph
		cjne a,#20h,gc1     ; done all EPROM?

		mov dptr,#csum      ; save for IBM
		mov a,r0
		movx @dptr,a

		inc dptr
		mov a,r1
		movx @dptr,a



; Set-up Interrupts and their Priorities

		
		mov wcntrl,#0
		mov timer2_func,#NULL

		mov led_cnt,#8          ; set led flash freq    
		mov led_cnt_seed,#8
		mov rcap2l,#low  maxdelay
		mov rcap2h,#high maxdelay

		mov t2con,#00000100b    ; timer2, 16bit autoreload timer
		
		mov  tmod,#00010001b    ; timer0, timer1 both 16 bit timers
		mov  tcon,#00000100b    ; ext0 state, ext1 edge triggered
		mov  ie  ,#10101011b    ; enable ext0, timer0,timer1, and timer2  
		mov  ip  ,#00001011b    ; ext0, timer0,timer1 high priority




; Move Mill to HOME position
		
		
		
		
;                lcall home
;                jmp debug
		
		
		
		
		setb calibration        ; request calibration data
		
		lcall home              ; sets X,Y and X positions to zero
		lcall Home_Open_Vice


		
		mov  executing,#0       ; Show IBM we're ready to do work
		;mov  mcntrl,#0





	page
	SUBTTL Baseloop


BASELOOP:
		mov  a,mcntrl           ; read master control byte in shared memory
		jz   toggleled          ; nothing else to do

		push acc
		clr  a
		mov  mcntrl,a           ; clear master control byte

		mov  dptr,#ret_addr     ; set up return address
		mov  a,#low set_slave
		movx @dptr,a
		
		inc  dptr
		mov  a,#high set_slave
		movx @dptr,a

		pop  acc
		mov  mcntrl_copy,a      ; record master control byte to return
		
		jb   acc.0,m_error      ; control byte should be even
		jb   acc.7,m_error      ; and less than 128
		
		mov  dptr,#master_decode
		jmp  @a+dptr            ; decode master control byte via jump table 
								; returns are all indirect ajmp SET_SLAVE
		
m_error:                        ; could maybe add counter for number of errors caught           
		
set_slave:
		mov a,mcntrl_copy       ; retrieve master control byte, work done
		mov scntrl,a            ; and set to slave control byte - protocol
		

toggleled:
		
		mov  dptr,#manual1      ; read manual control buttons
		movx a,@dptr
		setb acc.7              ; mask OFF switch
		cpl  a                  ; for normal logic
		
		jz   baseloop
		
		lcall manual_control    ; button(s) have been pressed service
		
		ajmp  baseloop


		org high $ + 1 * 256    ; not needed but makes addresses tidy

master_decode:                  

		ajmp toggleled      ;0  should never be obeyed, but need to fill
		
		ajmp m_dowork       ;2  Execute      
		ajmp set_slave      ;4  abort - not executing - so ignore
		ajmp set_slave      ;6  Pause - not executing - so ignore
		ajmp set_slave      ;8  continue - not executing - so ignore

		ajmp m_stepx_h      ;10 move axis to and from home at jog feedrate
		ajmp m_stepx_a      ;12
		ajmp m_stepz_h      ;14
		ajmp m_stepz_a      ;16
		ajmp set_slave      ;18
		ajmp set_slave      ;20
		ajmp set_slave      ;22 other axes, such as for robots
		ajmp set_slave      ;24
		ajmp set_slave      ;26
		ajmp set_slave      ;28
		ajmp set_slave      ;30
		ajmp set_slave      ;32
		ajmp set_slave      ;34
		ajmp set_slave      ;36
		ajmp set_slave      ;38
		ajmp set_slave      ;40

		ajmp m_cancel       ;42 cancels functions 10 to 40
	   
		ajmp m_open_Vice    ;44
		ajmp m_Close_Vice   ;46
		
		ajmp m_changetool   ;48
		ajmp m_error        ;50
		
		ajmp m_home         ;52
		ajmp m_reset        ;54 this WILL NOT RETURN TO set_slave

		ajmp m_hardhome     ;56
		ajmp m_sethome      ;58
		
		ajmp m_jogx_p       ;60
		ajmp m_error
		ajmp m_jogz_p       ;64
		
		ajmp m_jogx_m       ;66
		ajmp m_error
		ajmp m_jogz_m       ;70
		
		
		ajmp m_error        ;70 rest of list should never be called
		ajmp m_error        ;72
		ajmp m_error        ;74
		ajmp m_error        ;76
		ajmp m_error        ;78
		ajmp m_error        ;80
		ajmp m_error        ;82
		ajmp m_error        ;84
		ajmp m_error        ;86
		ajmp m_error        ;88
		ajmp m_error        ;90
		ajmp m_error        ;92
		ajmp m_error        ;94
		ajmp m_error        ;96
		ajmp m_error        ;98
		ajmp m_error        ;100
		ajmp m_error        ;102
		ajmp m_error        ;104
		ajmp m_error        ;106
		ajmp m_error        ;108
		ajmp m_error        ;110
		ajmp m_error        ;112
		ajmp m_error        ;114
		ajmp m_error        ;116
		ajmp m_error        ;118
		ajmp m_error        ;120
		ajmp m_error        ;122
		ajmp m_error        ;124
		ajmp m_error        ;126



		page
		SUBTTL Obey IBM

		;Called when IBM wants something done whilst processing table
		;returns to table when continue or abort requested


setup_obey_ibm:
		cjne r4,#NULL,$

		push psw                ; save current register set
		push acc                ; Acc = IBM command 
		
		clr  rs0
		setb rs1                ; set register bank 2       
		
		mov r4,#NULL            

		
		cjne a,#10,$+3          ; a < 10 c=1
		jc   dummy0
		setb exe_pause          ; Show IBM we're paused
dummy0:              
		
		mov led_cnt_seed,#4     ; Show the world by flashing LED at double speed


		mov dptr,#ret_addr      ; set up return address
		mov a,#low dummy        ; in pause state
		movx @dptr,a
		inc dptr
		mov a,#high dummy
		movx @dptr,a
		pop acc                 ; recover IBM command
		
		lcall obey_ibm

		clr exe_pause           ; Show IBM we're not paused
		mov led_cnt_seed,#8


		pop psw                 ; recover register set
		ret



dummy:                          ; re-entry
;        jnb  door_open,dummy2
;        jnb  suspend,dummy2
;        ajmp ibm_cont          ;door has been opened
;        
;dummy2:
;        jnb  door_close,dummy3
;        jb   suspend,dummy3
;        ajmp ibm_cont          ;door has been closed
;
;dummy3:

		
		mov  dptr,#manual1      ; read manual control buttons
		movx a,@dptr
		setb acc.7              ; mask OFF switch
		cpl  a                  ; for normal logic
		jz   dummy1
		
		lcall manual_control    ; button(s) have been pressed service
		
dummy1:
		mov a,mcntrl            ; get next IBM order

obey_ibm:
		
		mov  mcntrl,#0           ; clear master control byte

dummy4:       
		
		mov  mcntrl_copy,a       ; record master control byte to return
		jb   acc.0,ibm_error     ; control byte should be even
		jb   acc.7,ibm_error     ; <128 else its an error   
		
		mov  dptr,#ibm_decode
		jmp  @a+dptr            ; decode master control byte via jump table 
								; RETURNS ARE by indirect ajmp dummy
ibm_error:
		ajmp dummy


		org high $ + 1 * 256    ; not needed but makes addresses tidy

ibm_decode:                     
		ajmp dummy          ;0
		ajmp dummy          ;2
		ajmp ibm_abort      ;4  abort set flag and return
		ajmp ibm_pause      ;6  pause 
		ajmp ibm_cont       ;8  continue
						
		ajmp m_stepx_h      ;10 move axis to and from home at jog feedrate
		ajmp m_stepx_a      ;12
		ajmp m_stepz_h      ;14
		ajmp m_stepz_a      ;16
		
		ajmp dummy          ;18
		ajmp dummy          ;20
		ajmp dummy          ;22
		ajmp dummy          ;24
		ajmp dummy          ;26
		ajmp dummy          ;28
		ajmp dummy          ;30
		ajmp dummy          ;32
		ajmp dummy          ;34
		ajmp dummy          ;36
		ajmp dummy          ;38
		ajmp dummy          ;40
		ajmp dummy          ;42 cancels functions 10 to 40
	   
		ajmp m_open_vice    ;44
		ajmp m_Close_vice   ;46

		ajmp m_changetool   ;48
		ajmp ibm_error      ;50 
		
		ajmp m_home         ;52

		ajmp m_reset        ;54 this WILL NOT RETURN TO tab_return

		ajmp m_hardhome     ;56
		ajmp m_sethome      ;58
		
		ajmp m_jogx_p       ;60
		ajmp ibm_error
		ajmp m_jogz_p       ;64
		
		ajmp m_jogx_m       ;66
		ajmp ibm_error
		ajmp m_jogz_m       ;70
		
		
		ajmp ibm_error        ;72
		ajmp ibm_error        ;74
		ajmp ibm_error        ;76
		ajmp ibm_error        ;78
		ajmp ibm_error        ;80
		ajmp ibm_error        ;82
		ajmp ibm_error        ;84
		ajmp ibm_error        ;86
		ajmp ibm_error        ;88
		ajmp ibm_error        ;90
		ajmp ibm_error        ;92
		ajmp ibm_error        ;94
		ajmp ibm_error        ;96
		ajmp ibm_error        ;98
		ajmp ibm_error        ;100
		ajmp ibm_error        ;102
		ajmp ibm_error        ;104
		ajmp ibm_error        ;106
		ajmp ibm_error        ;108
		ajmp ibm_error        ;110
		ajmp ibm_error        ;112
		ajmp ibm_error        ;114
		ajmp ibm_error        ;116
		ajmp ibm_error        ;118
		ajmp ibm_error        ;120
		ajmp ibm_error        ;122
		ajmp ibm_error        ;124
		ajmp ibm_error        ;126

ibm_pause:
	setb exe_pause
	ajmp dummy


ibm_abort:
	setb abort
	clr  a
	lcall set_spindle_speed ; - switch off spindle
	mov r4,#PURGE
	lcall retract_key

ibm_cont:
	ret


tab_return:

		setb  no_back           ; don't send backlash compensation
		lcall setdirns          ; reset direction bits    
		clr   no_back
		
		mov dptr,#ret_addr      ; retrieve return address
		movx a,@dptr             
		push acc
		inc dptr
		movx a,@dptr
		mov dph,a
		pop dpl
		clr a
		jmp @a+dptr         ; jump to return address



	page
	SUBTTL Baseloop/Obey IBM Functions

; When called via baseloop will be using register bank 0
; When called via Obey IBM will be using register bank 2


m_dowork:
		mov   scntrl,a
		setb  exe_table       ; Show IBM we're doing work
		clr   abort             
		
		lcall dowork
		
		mov   executing,#0   ; Show IBM we're done work

		ajmp   set_slave     ; This function always returns to baseloop
							; because its only ever called from there
						     

m_sethome:      ;58
m_hardhome:     ;56
m_home:

	lcall home
	ajmp   tab_return


m_reset:
		mov scntrl,a
		ajmp begin

m_jogx_p:
	setb xdir               ; x+ve for sums
	setb XDIRN               ; x+ve in hardware
	ajmp m_jogx

m_jogx_m:
	clr xdir               ; x-ve for sums
	clr XDIRN               ; x-ve in hardware

m_jogx:
	mov  tl0_seed,#low  maxfeed
	mov  th0_seed,#high maxfeed 
	mov  r0,jogcnt
	mov  r1,jogcnt+1
	mov  r4,#XFEED
	cjne r4,#NULL,$
	ajmp tab_return


m_jogz_p:
	setb zdir               ; z+ve for sums
	setb ZDIRN               ; z+ve in hardware
	ajmp m_jogz

m_jogz_m:
	clr  zdir               ; z-ve for sums
	clr  ZDIRN               ; z-ve in hardware

m_jogz:
	mov  tl0_seed,#low  maxfeed 
	mov  th0_seed,#high maxfeed
	mov  r0,jogcnt
	mov  r1,jogcnt+1
	mov  r4,#ZFEED
	cjne r4,#NULL,$
	ajmp tab_return



m_jog:
		mov tl0_seed,jog_feed_rate 
		mov th0_seed,jog_feed_rate+1 


m_jog1:        
		mov a,mcntrl            ; jogging stopped by another command ?
		jnz m_jog_exit  
		cjne r4,#NULL,m_jog1    ; jogging stopped by limit ?

m_jog_exit:

		mov r4,#PURGE
		cjne r4,#NULL,$
		
		ajmp  tab_return         ; at end of travel or stopped by IBM


m_stepx_h:                      ; move x axis to home at jog feedrate
		mov scntrl,a
		lcall r0r1_xmin
		cjne r0,#0,m_stepx_h1
		cjne r1,#0,m_stepx_h1
		ajmp  tab_return         ; at end of travel

m_stepx_h1:
		mov r4,#XFEED           ; timer 0 function call
		ajmp  m_jog             ; setup timer to jog feedrate


m_stepx_a:                         ; move x axis from home at jog feedrate
		mov scntrl,a
		lcall r0r1_xmax
		cjne r0,#0,m_stepx_h1
		cjne r1,#0,m_stepx_h1
		ajmp  tab_return         ; at end of travel
		



m_stepz_h:                      ; move x axis to home at jog feedrate
		mov scntrl,a
		lcall r0r1_zmin
		cjne r0,#0,m_stepz_h1
		cjne r1,#0,m_stepz_h1
		ajmp  tab_return         ; at end of travel

m_stepz_h1:
		mov r4,#ZFEED            ; timer 0 function call
		ajmp  m_jog              ; setup timer to jog feedrate


m_stepz_a:                      ; move y axis from home at jog feedrate
		mov scntrl,a
		lcall r0r1_zmax
		cjne r0,#0,m_stepz_h1
		cjne r1,#0,m_stepz_h1
		ajmp  tab_return         ; at end of travel


		
m_cancel:           ; cancels functions 10 to 40
		mov scntrl,a

		ajmp  tab_return


m_open_vice:
		mov scntrl,a

		lcall open_vice

		ajmp  tab_return


m_close_vice:
		mov scntrl,a

		lcall close_vice

		ajmp  tab_return


m_Changetool:
		mov scntrl,a
		
		lcall changetool
		
		ajmp  tab_return





	
	page
	subttl Control Table Interpreter
; OBEY THE INSTRUCTIONS IN THE CONTROL TABLE.  nb. uses register bank 0, also

dowork: 
		mov dptr,#tableadd      ; dptr now has where to start decoding
		mov r6,dph              ; r6,r7 BOOKED !
		mov r7,dpl
		ajmp  decode_loop1

								; Decode Control byte from table
decode_loop:                    ; return here after processed control byte

		lcall inctable_a        ; point to next control byte
decode_loop1:

		movx a,@dptr            ; read control byte from table
		mov  control_byte,a     ; preserve it                    
		
		jz  decode_loop1a       ; got more XY data 
		cjne r4,#NULL,$         ; not XY data so make sure previous finished

decode_loop1a:        

		mov  dptr,#manual1      ; read manual buttons, to see if intervention
		movx a,@dptr
		setb acc.7              ; added 6/2/96        
		cpl  a                  ; for normal logic
		jz   decode0            ; no intervention
		
		
		mov a,#pause            ; pause function call 
		lcall setup_obey_ibm    ; button(s) pressed service

decode0:
		mov a,mcntrl            ; has there been an abort, meanwhile ?
		jz decode00

		lcall setup_obey_ibm    ; IBM wants something - do it

decode00:
		jb abort,decode01       ; Table was aborted by IBM
		
		mov  a,control_byte     ; retreive control byte - resumption entry

		cjne a,#0ffh,decode1
		
decode01:        
		clr a
		lcall set_spindle_speed

		ret                     ; return to base-loop, finished table

decode1:

		jb   acc.7,decode2
		ajmp  decode3

decode2:
		cjne a,#224,decode2b
		ajmp miscel             ; decode miscellaneous group of commands

decode2b:
		
		cjne a,#192,decode2c

		lcall home              ; return Mill to HOME position
;        lcall Open_Vice
		
		ajmp  decode_loop


decode2c:
		cjne a,#136,decode2d

		lcall inctable_a
		movx a,@dptr            ; read delay in seconds from table

		lcall delay_com         ; invoke a delay
		
		ajmp  decode_loop

decode2d:
		anl  a,#03h             ; extract XZ direction bits
		mov  dir_nbits,a        ; and save them for use
		
		lcall setdirns
		
		ajmp  decode_loop




decode3:
		jb   acc.6,movez
		jb   acc.4,movexyz

movexy:
		ajmp  movexy_s           ; All these must return to decode_loop

movexyz:
		ajmp movexyz_s




movez:
	cjne a,#64,code65
	ajmp movez_s                ; Move Z at feedrate

code65:
	cjne a,#65,code66
	ajmp movex_s                ; Move X at feedrate

code66:
	cjne a,#66,code67
	ajmp readZ_t                ; Read Z threading table

code67:
	ajmp moveZ_t                ; Move Z at threading rate





; Decode mumber of steps to move X and Y in a pattern, and do that



movexy_s:
		
		lcall inctable_a
		
		movx a,@dptr            ; byte has pattern for X steps
		mov  x_pattern,a        ; save it for rotation

		lcall inctable_a

		movx a,@dptr            ; followed by pattern for Z steps
		mov  z_pattern,a


		mov r4,#XYSTEP          ; timer 0 function call
		
mxy0:                            ; wait til timer interrupts finished
		
		cjne r4,#NULL,$
		
;        cjne r4,#XYSTEP_END,mxy1
;        ajmp  decode_loop
;mxy1:
;        cjne r4,#NULL,mxy0      
		
		ajmp  decode_loop        ; do next control byte decode



	page
	SUBTTL ASAP Routines

; The asap routines for large movements
z_asap:

; calculate the flat part, if any, after the acceleration transient 


		mov  dptr,#xz_steps+3
		movx a,@dptr            ; read msb of move
		clr c
		subb a,#8               ; subtract 800h steps for ramp
		jnc  z_asap1            
								; all steps will be in ramp
		clr c
		movx a,@dptr            ; re-read msb of move
		rrc  a                  ; divide msb by 2, carry has lsbit
		mov z_updown+1,a
		

		mov  dptr,#xz_steps+2
		movx a,@dptr            ; read lsb of move
		rrc  a                  ; divide lsb by 2 - carry may be set
		mov  z_updown,a
		
		mov  z_flat,#0
		mov  z_flat+1,#0
		jnc  z_asap2            
		mov  z_flat,#1
		
		ajmp  z_asap2            

z_asap1:
		mov  z_flat+1,a
		mov  dptr,#xz_steps+2
		movx a,@dptr            ; read lsb of move
		mov  z_flat,a

		mov z_updown,#0
		mov z_updown+1,#4
		

z_asap2:          
		
		mov ramp_z,#low ramptable       ; initialise pointer to acceleration table
		mov ramp_z+1,#high ramptable

		mov r0,z_updown             ;lsb
		mov r1,z_updown+1           ;msb
		
		mov r4,#ZASAP           ; timer 0 function call
		cjne r4,#NULL,$         ; wait til timer interrupts finished
;                lcall WaitDoneAbort

		ret




; The asap routines for large movements
x_asap:

; calculate the flat part, if any, after the acceleration transient 


		mov  dptr,#xz_steps+1
		movx a,@dptr            ; read msb of move
		clr c
		subb a,#8               ; subtract 800h steps for ramp
		jnc  x_asap1            
								; all steps will be in ramp
		clr c
		movx a,@dptr            ; re-read msb of move
		rrc  a                  ; divide msb by 2, carry has lsbit
		mov x_updown+1,a
		

		mov  dptr,#xz_steps
		movx a,@dptr            ; read lsb of move
		rrc  a                  ; divide lsb by 2 - carry may be set
		mov  x_updown,a
		
		mov  x_flat,#0
		mov  x_flat+1,#0
		jnc  x_asap2            
		mov  x_flat,#1
		
		ajmp  x_asap2            

x_asap1:
		mov  x_flat+1,a
		mov  dptr,#xz_steps
		movx a,@dptr            ; read lsb of move
		mov  x_flat,a

		mov x_updown,#0
		mov x_updown+1,#4
		

x_asap2:          
		
		mov ramp_x,#low ramptable       ; initialise pointer to acceleration table
		mov ramp_x+1,#high ramptable

		mov r0,x_updown             ;lsb
		mov r1,x_updown+1           ;msb
		
		mov r4,#xASAP           ; timer 0 function call
		cjne r4,#NULL,$         ; wait til timer interrupts finished
;                lcall waitdoneabort
		ret


;WaitDoneAbort:
;        mov  dptr,#manual1
;        movx a,@dptr
;        mov  c,acc.7
;;        mov  spindle_on,c
;        jnc  done
;        cjne r4,#NULL,WaitDoneAbort
;        ret
;
;Done:    
;        setb abort
;        mov r4,#PURGE
;        cjne r4,#NULL,$
;        ret



; move ASAP the x z number of steps

movexyz_s:
		
		lcall inctable_a        ; copy data from control table
		movx a,@dptr            ; x and z, lsb then msb
		mov  dptr,#xz_steps
		movx @dptr,a

		lcall inctable_a
		movx a,@dptr
		mov  dptr,#xz_steps+1
		movx @dptr,a


		lcall inctable_a
		movx a,@dptr
		mov  dptr,#xz_steps+2
		movx @dptr,a
		
		lcall inctable_a
		movx a,@dptr
		mov  dptr,#xz_steps+3
		movx @dptr,a
		
		jb    dir_nbitsX,Zfirst
		lcall x_asap         
		lcall z_asap  
		ajmp  decode_loop
zfirst:
		lcall z_asap
		lcall x_asap
		ajmp  decode_loop


; move single axis at given feed rate
move_feed:
		
		lcall inctable_a
		
		movx a,@dptr            ; read lsb of number of steps
		mov  r0,a               ; into r0

		lcall inctable_a
		
		movx a,@dptr            ; read msb of number of steps
		mov  r1,a               ; into r1

		
		mov  tl0_seed,feedrate
		mov  tl0,feedrate
		
		mov  th0_seed,feedrate+1
		mov  th0,feedrate+1

		mov r4,r4_seed

move_f1:
		mov  a,mcntrl
		jnz  move_f2

		mov  dptr,#manual1      ; read manual buttons, to see if intervention
		movx a,@dptr
		setb acc.7              ;
		cpl  a                  ; for normal logic
		jz   move_f3
		mov  a,#pause           ; set pause state

move_f2:
		
		mov r4_seed,r4          ; save current function call
		
		mov r4,#PURGE           ; pause current move
		
		
		lcall setup_obey_ibm    ; IBM wants something - do it
		jb abort,move_f4        ; IBM says abort

		mov r4,r4_seed

		mov  tl0_seed,feedrate
		mov  tl0,feedrate
		
		mov  th0_seed,feedrate+1
		mov  th0,feedrate+1

		
move_f3:
		cjne r4,#NULL,move_f1      ;wait til timer interrupts finished

move_f4:        
		ajmp  decode_loop


; Z axis move
movez_s:
		mov r4_seed,#ZFEED           ; timer 0 function call
		ajmp move_feed


; X axis move

movex_s:
		mov r4_seed,#XFEED           ; timer 0 function call
		ajmp move_feed



readZ_t:                            ; read threading table
	clr   a                         ; Stop spindle
	lcall set_spindle_speed
	

	mov   r0,#low screwtable
	
readZ_1:    
	lcall inctable_a
	movx  a,@dptr            
	
	clr   ie.7
	mov   p2,#high screwtable
	movx  @r0,a                 ; write byte into screwtable
	setb  ie.7
	
	inc   r0
	jnz   readZ_1               ; Table will be zero terminated


	lcall wait_spindlestall ; - Wait til spindle stopped

	lcall set_screw_speed
	
	ajmp  decode_loop



moveZ_t:

	lcall inctable_a
	movx a,@dptr            ; read lsb of number of steps
	mov  r0,a               ; into r0

	lcall inctable_a
	movx a,@dptr            ; read msb of number of steps
	mov  r1,a               ; into r1
	
	
	mov timer2_func,#28     ; Set up timer2
	
	jb tr0,$                ; wait until timer 0 has been disabled
							; signals that timer2 has been setup


							; wait for index
	mov dptr,#microsws      ; read spindle switch
th_1:
	movx a,@dptr            ; wait until off switch
	jb acc.2,th_1

th_2:
	movx a,@dptr            ; wait until on switch
	jnb acc.2,th_2
	
	setb tr2                ; start timer2
	
;    jnb tr0,$               ; wait whilst timer 0 is disabled    


;        mov dptr,#manual1
th_3:    
;        movx a,@dptr
;        mov c,acc.7
;        mov spindle_on,c
	jnb tr0,th_3

	ajmp  decode_loop






; decode of miscellaneous commands

miscel:

		lcall inctable_a        ; read miscellaneous context byte

		movx a,@dptr            

miscel1:
		cjne a,#1,miscel2
				
		

		lcall inctable_a

		movx a,@dptr            ; get speed from control table

		lcall set_spindle_speed ; speed from control table
		
		ajmp decode_loop


miscel2:
		cjne a,#2,miscel3

		

		lcall inctable_a

		movx a,@dptr            ; get speed from control table
		
		lcall set_spindle_speed ; speed from control table
		
		ajmp decode_loop


miscel3:
		cjne a,#3,miscel4

		mov  dptr,#spindle      ; switch-off spindle motor = 0 speed
		clr  a
		movx @dptr,a
		mov spindle_speed,a     ; Copy for IBM
		
		ajmp decode_loop


miscel4:
		cjne a,#4,miscel5

		lcall close_vice1       ;Key assumed to already be in

		mov a,#pause
		lcall setup_obey_ibm
		
		ajmp decode_loop


miscel5:
		cjne a,#5,miscel6

		lcall open_vice1        ;Key assumed to already be in

		mov a,#pause
		lcall setup_obey_ibm
		
		ajmp decode_loop

miscel6:
		cjne a,#6,miscel7

		lcall inctable_a

		movx a,@dptr            ; get speed from control table
		
		lcall set_spindle_speed ; speed from control table
		
		ajmp decode_loop


miscel7:
		cjne a,#7,miscel8
		
		lcall inctable_a
		movx a,@dptr            ; copy table bytes to feedrate
		mov feedrate,a
		
		lcall inctable_a
		movx a,@dptr            ; copy table bytes to feedrate
		mov feedrate+1,a

		lcall inctable_a
		movx a,@dptr            ; copy table bytes to feedrate
		mov feedrate+2,a

		lcall inctable_a
		movx a,@dptr            ; copy table bytes to feedrate
		mov feedrate+3,a
		
		ajmp decode_loop



miscel8:
		cjne a,#8,miscel9
		lcall Insert_Key

		mov a,#pause            ; set paused state for Walli prog
		lcall setup_obey_ibm    ; paused state aborted by opening door

		ajmp decode_loop



miscel9:
		cjne a,#9,miscel10

		mov  r1,#scratch        ; Off-sets given in next six bytes of table
		mov  r0,#6
miscel9a:
		lcall inctable_a

		movx a,@dptr            ; copy bytes to scratch pad
		mov  @r1,a
		inc  r1
		djnz r0,miscel9a

		mov  dptr,#off_sets
		mov  r1,#scratch
		mov  r0,#6
miscel9b:
		mov  a,@r1              ; copy from scratch pad to offsets record
		movx @dptr,a
		inc  dptr
		inc  r1
		djnz r0,miscel9b

		lcall do_offsets
		
		ajmp decode_loop


miscel10:
		cjne a,#10,miscel11
		
		mov a,#pause            ; pause function call 
		lcall setup_obey_ibm    ; service ibm and/or buttons
		
		ajmp decode_loop


miscel11:
		cjne a,#11,miscel12

		lcall inctable_a        ; read tool size 
		movx  a,@dptr
		mov   tool,a              
		lcall changetool
		
		ajmp decode_loop        


miscel12:
		cjne a,#12,miscel13

		lcall inctable_a

		movx a,@dptr            ; read workcell outputs data from table
		mov  r0,a

		mov  a,wco_stat
		orl  a,r0               ; modify with mask to set
		mov  wco_stat,a
		
		mov  dptr,#workcell_o   ; send to workcell outputs
		movx @dptr,a            
		
		ajmp decode_loop


miscel13:
		cjne a,#13,miscel14

		lcall inctable_a

		movx a,@dptr            ; read workcell outputs data from table
		cpl  a
		mov  r0,a

		mov  a,wco_stat
		anl  a,r0               ; modify with mask to clear
		mov  wco_stat,a
		
		mov  dptr,#workcell_o   ; send to workcell outputs
		movx @dptr,a            
		
		ajmp decode_loop

		


miscel14:
		cjne a,#14,miscel15

		lcall inctable_a
		
		movx a,@dptr            ; which workcell input to test from table
		mov  r0,a               ; 
		
		
miscel14a:
		mov a,mcntrl
		jz miscel14b

		lcall setup_obey_ibm    ; IBM wants something - do it
		jb abort,miscel14C      ; Table was aborted by IBM

miscel14b:
		mov  dptr,#workcell_i
		movx a,@dptr            ; read workcell inputs
		anl  a,r0               ; test bit set
		jz   miscel14a          ; wait loop till bit set HIGH
miscel14C:
		ajmp decode_loop




miscel15:
		cjne a,#15,miscel16
		lcall inctable_a
		movx a,@dptr            ; workcell input to test from table
		mov  r0,a
miscel15a:
		mov a,mcntrl
		jz miscel15b
		lcall setup_obey_ibm    ; IBM wants something - do it
		jb abort,miscel15C      ; Table was aborted by IBM
miscel15b:
		mov  dptr,#workcell_i
		movx a,@dptr            ; read workcell inputs
		anl  a,r0               ; test bit low
		jnz  miscel15a          ; wait loop till bit set LOW
miscel15C:
		ajmp decode_loop



miscel16:
		cjne a,#16,miscel17

		lcall inctable_a        ; next two bytes on table are current line
		movx a,@dptr
		mov curr_line,a
		
		lcall inctable_a        
		movx a,@dptr
		mov curr_line+1,a

		ajmp decode_loop


miscel17:
		cjne a,#17,miscel18

;        lcall air_on
		
		ajmp decode_loop


miscel18:
		cjne a,#18,miscel19

;        lcall air_off
		
		ajmp decode_loop
		 
miscel19:
		
		ajmp decode_loop        ; end of miscellaneous command definitions



; Action subroutines


set_screw_speed:    
; Aim to set spindle to 100 RPM
; = 1.67 rev/sec = 3000*1.67 pulses per sec
; = 5010 / sec = 19*256
	
	mov   a,#10
	mov spindle_speed,a
	
sss1:    
	mov  a,spindle_speed

;    mov  dptr,#workcell_o
;    mov  wco_stat,a
;    movx @dptr,a

	lcall set_spindle_speed

	clr tr2     ;Disable timer 2
	
	clr a       ;Clear counter
	mov tl2,a
	mov th2,a

	setb t2con.1    ;Make into counter
	
	mov a,#1
	lcall delay_com ;Wait 1/2 sec to let speed settle

	setb tr2   ;Start Counter 2     

	mov a,#1
	lcall delay_com ;Sample for 1/2 sec

	clr tr2     ;Disable timer 2
	
	mov  a,th2
	cjne a,#19,sss2     ;C = 1 if A < 19
	
	clr t2con.1         ;Back into timer
	setb tr2            ;Start Counter 2     
	ret

sss2:
	jc sss3
	
	dec spindle_speed
	jmp sss1

sss3:
	inc spindle_speed
	jmp sss1

	






set_spindle_speed:
	jz  set_spindle_off
	jnb suspend,set_spindle_off
	clr spindle_on          ;want spindle on but power is off

set_spindle_off:
	mov  dptr,#spindle
	movx @dptr,a            ; set required spindle motor speed
	mov spindle_speed,a     ; copy for IBM
	ret




delay_com:                      ; enter a delayed state

; delay of 1 second = 11 059 200/12 = 921600  cycles        
; timer interrupt set to 65536 cycles
; - 1 sec = 14 interrupts


; delay of 1 second = 16 000 000/12 = 1333333 cycles        
; timer interrupt set to 65536 cycles
; - 0.5 sec = 10 interrupts


		jz delay_2              ; finished delay ?

		mov tl0_seed,#0
		mov th0_seed,#0
		mov r0,#10
		mov r4,#delay           ; Set timer 0 delay function

		cjne r4,#NULL,$         ; wait til timer interrupts finished

		dec a
		jmp delay_com
delay_2:                            
		ret





do_offsets:                     ; move X,Y and Z by given offsets as steps
								; but dont change recorded xyz positions
								; done at the 1kh rate, ie. timer set fbc2h
		
		mov  dptr,#off_sets+4   ; do z offset first
		movx a,@dptr            ; lsb
		
		inc  dptr
		movx a,@dptr            ; msb

test_yoff:

		mov  dptr,#off_sets+2   ; test y offset for value
		movx a,@dptr            ; lsb
		
		inc  dptr
		movx a,@dptr            ; msb
		

test_xoff:

		mov  dptr,#off_sets     ; test x offset for value
		movx a,@dptr            ; lsb
		
		inc  dptr
		movx a,@dptr            ; msb

		ret                     ; all offsets done


		page
		subttl Manual Control


manual_control:                 ; decode buttons for what to do

; Entry with acc loaded with the state of the buttons.

; bit0  move X 
; bit1   ..  X 
; bit2   ..  Z 
; bit3   ..  Z 
; bit4  fast feed rate of 10mm/sec, normal feed rate 2mm/sec  
		
		
		lcall Set_Toolmax
		mov  th0_seed,#high MAXMANFEED
		mov  tl0_seed,#low  MAXMANFEED
		
		
		mov  r4,#MAN       ;set timer0 Manual NULL function

man1:
		mov  dptr,#manual1      ; do the read again
		movx a,@dptr
;                mov c,acc.7
;                mov spindle_on,c
		
		cpl  a                  ; for normal logic
		jnz man2

		
		mov  r4,#PURGE     ;set timer0 NULL function
		cjne r4,#NULL,$
		ret

		
man2:
		jnb   acc.4,man3
		mov   th0_seed,#high LOWMANFEED         
		mov   tl0_seed,#low  LOWMANFEED

;        jmp  man4
;
man3:
;        push acc
;        anl a,#1111b
;        jnz man4a
;        
;       mov  th0_seed,#high MAXMANFEED          ; May only change up to fast speed
;       mov  tl0_seed,#low  MAXMANFEED          ; if no other button is pressed
;man4a:
;        pop acc

		
man4:
		jnb  acc.0,man5      ; X to home
		clr  xdir
		clr  XDIRN
		
		push acc
		mov  a,r4                ; Manual X mode
		setb acc.1
		mov  r4,a
		pop  acc
		
		jmp  man7

man5:
		jnb  acc.1,man6      ; X from home
		setb xdir
		setb XDIRN
		push acc
		mov  a,r4                ; Manual X mode
		setb acc.1
		mov  r4,a
		pop  acc
		jmp  man7

man6:
		push acc
		mov a,r4                ; NOT manual X mode
		clr acc.1
		mov r4,a
		pop acc

man7:
		
		jnb  acc.2,man8     ; Z to home, up
		
		clr  zdir
		clr  ZDIRN
		
		push acc
		mov a,r4                ;Set Z manual mode
		setb acc.2
		mov r4,a
		pop acc
		jmp  man1

man8:
		jnb  acc.3,man9     ; Z from home, down

		setb zdir
		setb ZDIRN

		push acc
		mov a,r4                ;Set Z manual mode
		setb acc.2
		mov r4,a
		pop acc
		jmp  man1

man9:

		push acc
		mov a,r4                ;Not Z manual mode
		clr acc.2
		mov r4,a
		pop acc 

		jmp man1





		page
		SUBTTL UTILITY ROUTINES

inctable_a:                     ; Increment current address in table

		mov  dpl,r7             ; recover current address
		mov  dph,r6
		inc  dptr               ; increment current address, ready for use
		mov  r6,dph
		mov  r7,dpl             ; save it for next request, ie, r6,r7 BOOKED!
		
		cjne r7,#0,inctable_ax  ; passed a page boundary in control table ?
		ajmp $+3 

inctable_ax:          
		ret

		clr  ea                 ; IBM must not butt in

		mov  a,table_marker     ; read current state of table
		mov  pagebits,a         ; copy for bit marking

		cjne r6,#3,inctable_a1  ; finished first page
		setb pagebits0          ; mark it used
		jmp  inctable_ac

inctable_a1:
		
		cjne r6,#4,inctable_a2  ; finished second page
		setb pagebits1          ; mark it used
		jmp  inctable_ac         

inctable_a2:
		
		cjne r6,#5,inctable_a3  ; finished third page
		setb pagebits2          ; mark it used
		jmp  inctable_ac

inctable_a3:
		
		cjne r6,#6,inctable_a4  ; finished fourth page
		setb pagebits3          ; mark it used
		jmp  inctable_ac

inctable_a4:
		
		cjne r6,#7,inctable_a5  ; finished fifth page
		setb pagebits4          ; mark it used
		jmp  inctable_ac

inctable_a5:
		
		cjne r6,#8,inctable_a6  ; finished sixth page
		setb pagebits5          ; mark it used
		jmp  inctable_ac

inctable_a6:
		
		cjne r6,#9,inctable_a7  ; finished seventh page
		setb pagebits6          ; mark it used
		jmp  inctable_ac

inctable_a7:
								; finished eigth page
		setb pagebits7          ; mark it used
		mov  r6,#2              ; reset to beginning of table
		

inctable_ac:

		
		mov  a,pagebits
		mov  table_marker,a     ; restate state of table after changes
		
		setb ea                 ; let IBM back in
								
								
								
								;Now check to see if page were on is valid
inctable_waitvalid:             ; bit will be clear
		
		mov a,table_marker           ; read current state of table

		cjne r6,#2,$+6
		jnb acc.0,inctable_exit
		
		cjne r6,#3,$+6
		jnb acc.1,inctable_exit
		
		cjne r6,#4,$+6
		jnb acc.2,inctable_exit
		
		cjne r6,#5,$+6
		jnb acc.3,inctable_exit
		
		cjne r6,#6,$+6
		jnb acc.4,inctable_exit
		
		cjne r6,#7,$+6
		jnb acc.5,inctable_exit
		
		cjne r6,#8,$+6
		jnb acc.6,inctable_exit
		
		jnb acc.7,inctable_exit  ;r6 will be 9 - no need to check

		mov a,mcntrl
		jz inctable_w

		lcall setup_obey_ibm    ; IBM wants something - do it

inctable_w:
		mov led_cnt_seed,#2     ; flash LED FAST    
		ajmp inctable_waitvalid
		
inctable_exit:        
		
		mov led_cnt_seed,#8     ; flash LED normal
		mov dpl,r7              ; r7 will be 0
		mov dph,r6              ; r6 will be 2,3,4,5,6,7,8 or 9
		ret                     ; Caller expects both R6/R7 and dptr


setdirns:                           ; Set direction bits    
		setb ignore_step            ; Don't count step for position
		jnb dir_nbitsX,sd1
		jb  xdir,sd2
		setb xdir                   ; x+ve for sums
		setb XDIRN                   ; x+ve in hardware
		lcall send_x_backlash
		ajmp sd2
sd1:
		jnb xdir,sd2
		clr  xdir                   ; x-ve for sums
		clr  XDIRN                   ; x-ve in hardware
		lcall send_x_backlash

sd2:
		jnb dir_nbitsZ,sd5
		jb zdir,sd6
		setb zdir                   ; z+ve for sums
		setb ZDIRN                    ; z+ve in hardware
		lcall send_z_backlash
		ajmp sd6

sd5:
		jnb zdir,sd6 
		clr  zdir                   ; z-ve for sums
		clr  ZDIRN                   ; z-ve in hardware
		lcall send_z_backlash
sd6:
		clr ignore_step
		ret

send_x_backlash:
		jb  no_back,xb1
		mov r0,x_backlash
		mov r1,#0
		mov th0_seed,#high maxfeed
		mov tl0_seed,#low  maxfeed
		mov r4,#XFEED
		cjne r4,#NULL,$
xb1:
		ret


send_z_backlash:
		jb  no_back,zb1
		mov r0,z_backlash
		mov r1,#0
		mov th0_seed,#high maxfeed
		mov tl0_seed,#low  maxfeed
		mov r4,#ZFEED
		cjne r4,#NULL,$
zb1:
		ret


		page
		subttl Home


home:                           ; return X,Y and Z to the "zero" position
								; at start-up or on command.

								; additionally :-

		
		setb exe_home
		mov scntrl,#183         ; indicate to IBM we're homing

		clr  a
		lcall set_spindle_speed ; - switch off spindle

		mov   tool,#1
		lcall Set_Toolmax

		clr  a
		MOV curr_line,a
		MOV curr_line+1,a
		
		mov t2_intcnt,#0



								; X axis moved first, for safety, 
		

		setb init               ; allow moves past limits
		setb ignore_step        ; don't bother to count steps 

		clr a
		mov zpos,a              ; set recorded position to zero
		mov zpos+1,a
		
		mov xpos,a             ; set recorded X position to zero
		mov xpos+1,a


		mov  r0,#0               ;reset uswitch debounce counters
		mov  r1,#0
		mov  r4,#XSTEP           ; set timer0 function send z steps
		mov  th0_seed,#high MAXFEED
		mov  tl0_seed,#low  MAXFEED
		mov  th0,th0_seed          ; set timer intervals 
		mov  tl0,tl0_seed

		
		Clr  XDIRN              ; X-ve
		clr  Xdir
		
		setb tcon.4             ; enable timer0    


homex:
		lcall readx_us          ; read micro-switch states

		jb   acc.0,homex    ; ie. switch set
		

		mov r4,#NULL            ; set timer0 NULL function -  timer OFF
		
		mov  th0_seed,#high LOWFEED
		mov  tl0_seed,#low  LOWFEED    ; change rate to 50 mm/min
		MOV th0,th0_seed
		MOV tl0,tl0_seed
		
		setb XDIRN                ; change x direction
		setb xdir                 

		setb tcon.4

homex1:
		lcall readx_us          ; read micro-switch states

		jb   acc.0,xthere       ; ie. not switch set

		lcall set_sendxstep
		
		jb sendxstep,$          ; wait til previous step sent
		
		ajmp  homex1             ; continue till micro-switch not set

		clr tcon.4

xthere:

;        mov r0,x_calib
;        mov r1,x_calib+1
;        mov th0_seed,#high maxfeed
;        mov tl0_seed,#low  maxfeed
;        mov r4,#XFEED
;        setb tcon.4
;        cjne r4,#NULL,$
;        
		
		Clr tcon.4         
		
		
		mov r0,#0               ;reset uswitch debounce counters
		mov r1,#0
		mov r4,#ZSTEP           ; set timer0 function send z steps
		mov  th0_seed,#high MAXFEED
		mov  tl0_seed,#low  MAXFEED
		mov  th0,th0_seed          ; set timer intervals 
		mov  tl0,tl0_seed

		

		Clr  ZDIRN              ; z-ve
		
		setb tcon.4             ; enable timer0    

homez:
		lcall readz_us          ; read micro-switch states
		jb    acc.1,homez 



		mov  r4,#PURGE           ; set timer0 NULL function
		cjne r4,#NULL,$
		
		
		mov  th0_seed,#HIGH LOWFEED
		mov  tl0_seed,#low  LOWFEED     ; change rate to 50 mm/min
		
		Setb ZDIRN                ; change z direction
		

homez1:
		lcall readz_us          ; read micro-switch states

		jb  acc.1,zthere        ; ie. not switch set

		lcall set_sendzstep
		
		jb sendzstep,$          ; wait til previous step sent
		
		ajmp  homez1             ; continue till micro-switch not set



zthere:
		clr tcon.4

;        mov r0,z_calib
;        mov r1,z_calib+1
;        mov th0_seed,#high maxfeed
;        mov tl0_seed,#low  maxfeed
;        mov r4,#ZFEED
		setb tcon.4
;        cjne r4,#NULL,$


		;lcall Home_Open_Vice
		;lcall Open_Vice
		
		
		clr exe_home
		clr init                ; limits are now set - use them
		clr ignore_step
		
		ret




readx_us:
		mov  dptr,#microsws     ; read micro-switch states
rg3:
		movx a,@dptr
		
		jb acc.0,rg2
		
		mov r0,#0
		inc r1
		cjne r1,#0,rg3
		dec r1
		ret
rg2:
		mov r1,#0
		inc r0
		cjne r0,#0,rg3
		dec r0
		ret


readz_us:
		mov  dptr,#microsws     ; read micro-switch states
rg9:
		movx a,@dptr
		jb acc.1,rg8
		
		mov r0,#0
		inc r1
		cjne r1,#0,rg9
		dec r1
		ret
rg8:
		mov r1,#0
		inc r0
		cjne r0,#0,rg9
		dec r0
		ret


changetool:
;tool =
;Send X and Z to zero
		clr  xdir               ; x-ve for sums
		clr  XDIRN              ; x-ve in hardware
		
		clr  zdir               ; z-ve for sums
		clr  ZDIRN              ; z-ve in hardware
		
		mov  dptr,#xz_steps
		mov  a,xpos
		movx @dptr,a

		mov  dptr,#xz_steps+1
		mov  a,xpos+1
		movx @dptr,a

		mov  dptr,#xz_steps+2
		mov  a,zpos
		movx @dptr,a
		
		mov  dptr,#xz_steps+3
		mov  a,zpos+1
		movx @dptr,a
		
		lcall x_asap
		lcall z_asap
;Set turret position
		lcall Set_Toolmax
		ret

Set_Toolmax:

;Set max X and Z
	mov a,tool
	jnz tool1
	mov xmax,  #low  7620       ;Tool 0 gives unlimited travel
	mov xmax+1,#high 7620
	mov zmax,  #low  20320
	mov zmax+1,#high 20320
	Clr  Toolb
	Setb Toola
	ajmp tool5

tool1:
	cjne a,#1,tool2
	mov xmax,tool1_xmax         ;Tool 1 Forward cutting
	mov xmax+1,tool1_xmax+1
	mov zmax,tool1_zmax
	mov zmax+1,tool1_zmax+1
	Setb Toolb
	Clr  Toola
	ajmp tool5

tool2:
	cjne a,#2,tool3
	
	mov xmax,tool2_xmax         ;Tool 2 Backward cutting
	mov xmax+1,tool2_xmax+1
	
	clr c                       ;Not Full travel - 10mm limit
	mov a,tool2_zmax
	Subb a,#low TOOL2MIN
	mov zmax,a
	mov a,tool2_zmax+1
	Subb a,#High TOOL2MIN
	mov zmax+1,a
	Clr  Toolb
	Setb Toola
	ajmp tool5

tool3:
	cjne a,#3,tool4             ;Tool 3 Threading
	mov xmax,tool3_xmax
	mov xmax+1,tool3_xmax+1

	clr c                       ;Not Full travel - 3mm limit
	mov a,tool3_zmax
	Subb a,#low TOOL3MIN
	mov zmax,a
	mov a,tool3_zmax+1
	Subb a,#High TOOL3MIN
	mov zmax+1,a
	
	Setb Toolb
	Clr  Toola
	ajmp tool5

tool4:
	cjne a,#4,tool5             ;Tool 4 Boring
	mov  xmax,#0                ;NO X travel
	mov  xmax+1,#0                 
	mov  zmax,  tool4_zmax
	mov  zmax+1,tool4_zmax+1
	Clr  Toola
	Setb Toolb

tool5:
	jb   suspend,toolOff
	;jnb  spindle_on,toolOff
	Mov  a,darl_Stat
	Mov  dptr,#opto_darl
	movx @dptr,a
	ret

toolOff:
	Setb Toola                  ;Spindle Off
	Setb Toolb                  ;Tool holder Off    
	Mov  a,darl_Stat
	Mov  dptr,#opto_darl
	movx @dptr,a
	ret



Retract_key:
	Clr   Chuck_A    ; Motor Dis-Eng
	Setb  Chuck_B

	Setb Key_Power
	Setb key_A      ; chuck key motor - undo
	Clr  Key_B

	mov dptr,#step_vice
	mov  a,os_stat
	movx @dptr,a            ; Write to control port

	mov r0,#8               ; Reverse the key motor 
	mov r1,#0
	mov r2,#0

rk1:    
	mov  r3,#2              ; to clear snags - was 4 9/7/96
	cpl  key_A
	Cpl  Key_B
	mov  a,os_stat
	movx @dptr,a            ; Write to control port
rk2:    
	djnz r1,$
	djnz r2,rk2
	djnz r3,rk2
	
	djnz r0,rk1

	
	Setb key_A              ; Stop the key motor
	Setb Key_B
							; The Eng motor is left running
							; else key can hit chuck


	mov dptr,#step_vice
	mov  a,os_stat
	movx @dptr,a            ; Write to control port
	ret



;want_close:        
;        setb exe_pause
;        setb door_close
;        jb   suspend,$  ; WAIT FOR DOOR CLOSE OR CONTINUE
;        clr  door_close  ; thankyou
;        clr  exe_pause 
;        ret


insert_key:
	Setb   Chuck_B          ; Motor Dis Eng
	Clr    Chuck_A

	mov dptr,#step_vice
	mov  a,os_stat
	movx @dptr,a            ; Write to control port
	
	
	Clr a
	lcall set_spindle_speed ; - Stop spindle
	lcall wait_spindlestall ; - Wait til spindle stopped
	
	
	mov a,#SLOWSPIN
	lcall set_spindle_speed ; - rotate slowly
	

	mov dptr,#microsws      ; read spindle switch
ink1:    
	JB  suspend,insert_key
	movx a,@dptr            ; wait until off switch
	jb acc.2,ink1

ink2:
	JB  suspend,insert_key
	movx a,@dptr            ; wait until on switch
	jnb acc.2,ink2

	Setb   Chuck_A          ; Motor Eng
	Clr    Chuck_B

	mov dptr,#step_vice
	mov  a,os_stat
	movx @dptr,a            ; Write to control port

	lcall wait_spindlestall
	JB  suspend,insert_key

	
	mov a,#MEDSPIN
	lcall set_spindle_speed ; - Try to rotate slowly


	
	mov dptr,#step_vice
	
	Setb Key_Power          ;Low Power
	Setb key_A       
	Clr  Key_B

	mov r0,#6               ; Reverse the key motor 
	mov r1,#0
	mov r2,#0

ik1:                        ; to clear snags
	mov r3,#2               ;Added 16/9/95 - reduced from 4 9/7/96 - motor low current increased
	cpl  key_A
	Cpl  Key_B
	mov  a,os_stat
	movx @dptr,a            ; Write to control port
ik2:    
	djnz r1,$
	djnz r2,ik2
	djnz r3,ik2     ;Added 16/9/95
	
	djnz r0,ik1

	Setb key_A              ; Stop the key motor
	Setb Key_B              
	mov  a,os_stat
	movx @dptr,a            ; Write to control port


	Clr a
	lcall set_spindle_speed ; - Stop spindle

	
;        mov a,#pause            ; set paused state for Walli prog
;        lcall setup_obey_ibm    ; paused state aborted by opening door
	
;        setb exe_pause
;        setb door_open          ; want door open

;        jnb  suspend,$  ; WAIT FOR DOOR OPEN OR CONTINUE
;        clr  door_open  ; thankyou
;        clr  exe_pause

	ret


Waggle_Open:
	mov dptr,#step_vice
	Clr  Key_Power          ; High Power
	Setb key_A              ; Open
	Clr  Key_B              ; First Pulse will be closing - low power

Waggle:
	mov r0,#4               ; Pulse count
	mov r1,#0
	mov r2,#0

ik3:                        
	mov r3,#3               
	Cpl  Key_Power          ; Low Power when going opposite to desired dirn
	cpl  key_A              ; High power going desired direction
	Cpl  Key_B
	mov  a,os_stat
	movx @dptr,a            
ik4:    
	djnz r1,$               ; R1 R2 R3 = Pulse length
	djnz r2,ik4
	djnz r3,ik4     
	djnz r0,ik3             ; R0 = Repeat count

	Setb Key_Power          ; Continue with Low Power
	mov  a,os_stat
	movx @dptr,a            

	ret


Waggle_Closed:
	mov dptr,#step_vice
	Clr  Key_Power          ; High Power
	Setb key_B              ; Close
	Clr  Key_A              ; First Pulse will be closing - low power
	jmp  Waggle



Close_Vice:
	lcall insert_key
Close_Vice1:
	lcall Waggle_Closed
	lcall wait_keystall

	mov a,Vpos+1
	cjne a,#VPOSMAX,cv2
	lcall retract_key
	jmp Close_Vice

CV2:
	Clr  Key_Power          ; Full power
	mov  a,os_stat
	movx @dptr,a            
	lcall wait_keystall     ; Re-check Fully Open

	lcall Waggle_Closed     ; Hammer shut

	lcall retract_key
;        lcall want_close
;        mov a,#pause
;        lcall setup_obey_ibm
	ret



Home_Open_Vice:
	lcall retract_key
	Setb  Key_Reset

Open_Vice:
	lcall insert_key

Open_Vice1:
;wait until door open

	lcall Waggle_Open
	lcall wait_keystall     ; Vice now Fully Open


	mov a,Vpos+1
	cjne a,#VPOSMAX,ov2
	lcall retract_key
	jmp Open_Vice

ov2:
	Clr  Key_Power          ; Full power
	mov  a,os_stat
	movx @dptr,a            
	lcall wait_keystall     ; Re-check Fully Open

	lcall retract_key
;        lcall want_close
;        mov a,#pause
;        lcall setup_obey_ibm
	ret





;Reset_Key:    
;    push acc
;    push dpl
;    push dph
;    mov dptr,#step_vice
;    
;    Clr  Key_Reset
;    mov  a,os_stat
;    movx @dptr,a            ; Write to control port
;
;    nop
;    nop
;    nop
;    nop
;    nop
;    nop
;    nop
;    nop
;
;    Setb Key_Reset
;    mov  a,os_stat
;    movx @dptr,a            ; Write to control port
;    pop dph
;    pop dpl
;    pop acc
;    ret



wait_spindlestall:
	mov r0,#0
	mov r1,#0
	
	mov acc,p1              ; read states of encoders 0 to 3
	anl a,#11b
	mov lastp1,a
	
wss1:    
	mov r3,#64

wss2:
	mov r2,#0               ; Set scan time
waitmove:
	mov acc,p1              ; read states of encoders 0 to 3
	cjne a,lastp1,axis0     ; process if its changed
	djnz r2,waitmove
	
	clr a
	Xch a,r0                ; how much movement
	
	jnb acc.7,pos_change
	cpl a
	inc a
	
pos_change:
	cjne a,#8,$+3           ; a < 8 c=1
	jnc  wss1

	djnz r3,wss2            ; moved less than 8 lots of times? 

	Clr   a
	lcall set_spindle_speed ; - Stop spindle
	ret


axis0:                      ;if get high-> low transition then mod counter   
	jb  Enc0A, axis0lh      ;jump if encoder still high   
	jnb last0A,axis0q       ;jump if encoder was low last time also
	jb  Enc0B, incaxis0     ;get direction, jump positive

decaxis0:        
	dec r0
	jmp axis0x

incaxis0:
	inc r0
	jmp axis0x

axis0lh:                    ;test for low-> high transition   
	jb  last0A,axis0q       ;high this time, was it low last time    
	jb  Enc0B, decaxis0
	jmp incaxis0

axis0q:                  ;quadrature testing, ie. on other signal channel, p1.1
	
	jb  Enc0B,axis0lhq   ;jump if encoder still high
	jnb last0B,axis0x      ;jump if encoder was low last time also
	jnb Enc0A,incaxis0  ;get direction, jump positive (phase)
	dec r0
	jmp axis0x

axis0lhq:                ;test for low-> high transition
	jb  last0B,axis0x      ;high this time, was it low last time
	jnb Enc0A,decaxis0
	jmp incaxis0

axis0x:                   ;if get high-> low transition then mod counter   
	mov lastp1,a         ;last state now replaced by current
	jmp waitmove







wait_Keystall:
	mov Vpos,#0
	mov Vpos+1,#0
	
	mov r1,#0
	
	mov acc,p1              ; read states of encoders 0 to 3
	anl a,#1100b
	mov lastp1,a
	
wk1:    
	mov r5,#32              ; counts of times with "no" move

wk2:
	mov r2,#0               ; Set scan time
	mov r3,#32
wk3:
	mov acc,p1              ; read states of encoder
	anl a,#1100b
	cjne a,lastp1,axis1     ; process if its changed
	djnz r2,wk3
	djnz r3,wk3
	
	clr a
	Xch a,r1                ; how much movement
	
	jnb acc.7,wk4
	cpl a
	inc a
	
wk4:
	push acc
	add a,Vpos
	mov Vpos,a
	
	jnc wk5
	inc Vpos+1
	mov a,VPos+1
	cjne a,#VPOSMAX,wk5
	pop acc
	ret


wk5:

	pop acc
	cjne a,#4,$+3           ; a < 4 c=1
	jnc  wk1

	djnz r5,wk2             ; moved less than 8 lots of times? 




	
	ret



axis1:                      
	jb  Enc1A, axis1lh      ; jump if encoder high
	jnb last1A,axis1x       ; jump if encoder was low last time
	
	jb  Enc1B, incaxis1     ;get direction, jump positive

decaxis1:        
	dec r1
	jmp axis1x

incaxis1:
	inc r1
	jmp axis1x

axis1lh:                    ;test for low-> high transition   
	jb  last1A,axis1x       ; jump if encoder was high last time
	jb  Enc1B, decaxis1
	jmp incaxis1


axis1x:                   ;if get high-> low transition then mod counter   
	mov lastp1,a         ;last state now replaced by current
	jmp wk3


;    org high $ + 2 * 256    



wait_motorstart:
	mov r1,#0
	mov r2,#0
	mov r3,#4
wm1:
	djnz r1,wm1
	djnz r2,wm1
	djnz r3,wm1

	ret




r0r1_0:
	mov r0,#0
	mov r1,#0
	clr  spindle_on
	ret


r0r1_xmin:
	jb suspend,r0r1_0
	mov r0,xpos             ; set max number of steps to send
	mov r1,xpos+1
	clr  xdir               ; x-ve for sums
	clr  XDIRN               ; x-ve in hardware
	ret

		
r0r1_xmax:
	jb suspend,r0r1_0
	lcall Set_Toolmax
	clr c                   ; set max number of steps to send
	mov  a,xpos
	subb a,xmax             ;#low XMAX
	mov  r0,a
		
	mov  a,xpos+1
	subb a,xmax+1           ;#high XMAX
	mov  r1,a

	setb xdir               ; x+ve for sums
	setb XDIRN                ; x+ve in hardware
	ret

r0r1_zmin:
	jb suspend,r0r1_0
	mov r0,zpos             ; set maximum steps to send
	mov r1,zpos+1
	clr  zdir                   ; z-ve for sums
	clr  ZDIRN                   ; z-ve in hardware
	ret
		
r0r1_zmax:
	jb suspend,r0r1_0
	lcall Set_Toolmax
	clr  c                  ; set maximum steps to send
	mov  a,zpos
	subb a,zmax             ;#low ZMAX
	mov  r0,a
	mov  a,zpos+1
	subb a,zmax+1           ;#high ZMAX
	mov  r1,a
	setb zdir                   ; z+ve for sums
	setb ZDIRN                    ; z+ve in hardware
	ret









include ibmint.inc              ; External interrupt 0 service

include timer0.inc              ; Timer 0
	
include timer2.inc              ; Timer 2



		page
		subttl Ramp Table
		
	org high $ + 1 * 256    

ramptable:                      ; 16 bit entries for timer delay settings
	
		nolist

include ramp.inc
		list
		db "END"
		nolist


end
