        PAGE
        SUBTTL "IBM ISR"

IBMint:                         ; IBM interrupt on ext0, uses register set 1

;r0 = lsb address
;r1 = tmp
;r7 = msb address/page


;IBM sends 4 requests
; S1 S0
;  0  0 - Write data to 8031 - cont0
;  0  1 - Read data from 8031 - cont1 
;  1  0 - Set page - cont2
;  1  1 - Dummy cycle - IBM reading - cont3


        push acc
        push psw
        mov  psw,#00001000b     ; set register set 1
        
        jb p3.5,cont2           ; S1
        jb p3.4,cont1           ; S0


cont0:                          ; write data to 8031
        
        mov  p2,#high lsblatch
        mov  r1,#low lsblatch
        movx a,@r1              ; read lsb address from RJ45 interface
        mov  r0,a

        mov  r1,#low datalatch
        movx a,@r1              ; read data from RJ45 interface
        
        
        cjne r7,#INTERNAL,IBMext_0  ; which page?        
        
        mov  @r0,a                  ; write data to internal RAM
        cjne r0,#update_marker,exit ;update_marker written to ?

                                    ; yes, register the update 
        
                                    ; which pages updated
        cpl  a
        mov  r1,a                   ; ready to clear pages bit marked

        mov  a,table_marker         ; read current state of table pages
        anl  a,r1                   ; clear bit markers of updated pages
        mov  table_marker,a         ; and restate


        mov p2,#high intlatch
        mov  r1,#low intlatch
        
        movx a,@r1              ; clear interrupt latch

        pop  psw                ; will restore register bank
        pop  acc
        
        reti


        

        
IBMext_0:         
        mov  p2,r7              ; set page
        movx @r0,a              ; write data to external RAM
        
        mov p2,#high intlatch
        mov  r1,#low intlatch
        
        movx a,@r1              ; clear interrupt latch

        pop  psw                ; will restore register bank
        pop  acc
        
        reti
        


cont1:                          
                                ; read data from 8031

        mov  p2,#high lsblatch
        mov  r1,#low lsblatch
        movx a,@r1               ; read lsb address from RJ45 interface
        mov  r0,a

        cjne r7,#INTERNAL,IBMint_1      
        
        mov  a,@r0              ; read data from internal RAM

        mov p2,#high datastrobe 
        movx @r0,a                ; strobe data onto interface

        mov p2,#high dataoen 
        movx @r0,a                ; enable data output

        mov p2,#high intlatch
        mov  r1,#low intlatch
        
        movx a,@r1              ; clear interrupt latch

        pop  psw                ; will restore register bank
        pop  acc
        
        reti
        

IBMint_1:      

        cjne r7,#EPROM,IBMint_2      
        push dpl
        push dph
        
        clr  a
        movc a,@a+dptr          ; read data from EPROM
        
        mov  dptr,#datastrobe
        movx @dptr,a            ; strobe data onto interface
        mov  dptr,#dataoen
        movx @dptr,a            ; enable data output
        
        pop dph
        pop dpl
        
        ajmp exit
        
        

IBMint_2:      

        mov  p2,r7              ; set page
        movx a,@r0              ; read data from given address

        mov p2,#high datastrobe 
        movx @r0,a                ; strobe data onto interface

        mov p2,#high dataoen 
        movx @r0,a                ; enable data output
        
        mov p2,#high intlatch
        mov  r1,#low intlatch
        
        movx a,@r1              ; clear interrupt latch

        pop  psw                ; will restore register bank
        pop  acc
        
        reti
        



cont2:
        jb p3.4,CONT3            ; S0

        mov  p2,#high lsblatch
        mov  r1,#low lsblatch
        movx a,@r1               ; read lsb address from RJ45 interface
        mov  r7,a                ; store it as the new msb address

cont3:


exit:
        mov p2,#high intlatch
        mov  r1,#low intlatch
        
        movx a,@r1              ; clear interrupt latch

        pop  psw                ; will restore register bank
        pop  acc
        
        reti



