    PAGE
    SUBTTL Timer 2 ISR

SEND_RS232      equ 2           ;timer2 function call
stretch_pulse   equ 24          ;timer2 function call
multi_pulse     equ 26          ;timer2 function call



;BAUD_9600       equ 0ffffh - 94 ;timer2 timeout counter
                                ;11059200/(12*9600) = 96
                                ;jmp timer2 takes 2 = 94 


;BAUD_9600       equ 0ffffh - 102 ;timer2 timeout counter
                                 ;12000000/(12*9600) = 104
                                 ;jmp timer2 takes 2 = 102 


BAUD_9600       equ 0ffffh - 137 ;timer2 timeout counter
                                 ;16000000/(12*9600) = 139
                                 ;jmp timer2 takes 2 = 137 



; Toggles LED
; Polls wcntrl for workcell commands from IBM
; Executes workcell commands


timer2:
        clr  t2con.7
        push psw
        push acc
        push dpl
        push dph
        
        mov  a,timer2_func
        mov  dptr,#timer2_tab
        jmp  @a+dptr

;        ORG high $ + 1 * 256    ; Not needed but makes addresses tidy

timer2_tab:
        ajmp t2_poll_wcntrl     ;0 Default NULL function, look for work to do
        
        ajmp t2_rs232_init      ;2
        ajmp t2_rs232_start     ;4  send start bit - setup bit 0
        ajmp t2_rs232           ;6  send bit 0     - setup bit 1
        ajmp t2_rs232           ;8  send bit 1     - setup bit 2
        ajmp t2_rs232           ;10 send bit 2     - setup bit 3
        ajmp t2_rs232           ;12 send bit 3     - setup bit 4
        ajmp t2_rs232           ;14 send bit 4     - setup bit 5
        ajmp t2_rs232           ;16 send bit 5     - setup bit 6
        ajmp t2_rs232           ;18 send bit 6     - setup bit 7
        ajmp t2_rs232_stop      ;20 send bit 7     - setup stop
        ajmp t2_rs232_end       ;22 send stop bit  - end sequence

        ajmp t2_stretch_pulse   ;24 stretched single pulse

        ajmp t2_multi_pulse     ;26 send pulse stream


t2_poll_wcntrl:        
        
        djnz led_cnt,t2_0       ; important things first
        cpl  p3.1               ; flash the LED!
        mov  led_cnt,led_cnt_seed
t2_0:


        mov  dptr,#manual1      ; read manual control buttons
        movx a,@dptr
        jb   acc.7,t2_spindleon
        
        acall shutdown
        
        clr  spindle_on
        SetB  Suspend
        Mov   door_cnt,#60      ; about 3 sec
        ajmp t2_testdoor


t2_spindleon:
        Setb spindle_on
        

t2_testdoor:

;If door is open switch power off
        mov dptr,#microsws
        movx a,@dptr
        jnb acc.3,doorclosed

        acall shutdown
        SetB  DoorOpen
        SetB  Suspend
        Mov   door_cnt,#60      ; about 3 sec
        ajmp  DoorOpn


doorclosed:
        Clr doorOpen
        mov  dptr,#opto_darl    
        mov  a,opto_stat
        movx @dptr,a

        mov  dptr,#step_vice
        mov  a,os_stat
        movx @dptr,a 
        
        mov  a,Spindle_Speed
        mov  dptr,#spindle
        movx @dptr,a            
        
        
        jnb  spindle_on,DoorOpn
        djnz door_cnt,DoorOpn
        Clr  Suspend           ; let spindle get up to speed

DoorOpn:
        mov  a,power_xy_cnt
        jz   t2_poweroff

t2_poweron:
        dec  power_xy_cnt          ; count down to switch power off
        ajmp  t2_1

t2_poweroff:
;        jb   stepper_xy_power,t2_1  ; power already off

        setb stepper_xy_power      ; switch power off
;        mov  dptr ,#step_vice
;        mov  a,os_stat
;        movx @dptr,a
t2_1:
        
        mov  a,power_z_cnt
        jz   t2_z_poweroff
t2_z_poweron:
        dec  power_z_cnt          ; count down to switch power off
        ajmp  t2_z_1

t2_z_poweroff:
;        jb   stepper_z_power,t2_z_1  ; power already off

        setb stepper_z_power      ; switch power off
;        mov  dptr ,#step_vice
;        mov  a,os_stat
;        movx @dptr,a
t2_z_1:   



        mov a,t2_intcnt
        jz t2_z_2
        dec t2_intcnt
t2_z_2:

        mov  a,wcntrl           ; wcntrl will be cleared when function done
        
        jb   acc.0,t2_error     ; Control byte should be even and 
        jb   acc.7,t2_error     ; < 128 else its an error

        mov  dptr,#wcntrl_decode
        jmp  @a+dptr            ; decode master control byte via jump table 

t2_error:
        mov  wcntrl,#0
        ajmp t2_exit 

;        ORG high $ + 1 * 256    ; Not needed but makes addresses tidy

wcntrl_decode:
        ajmp t2_exit            ;0  NULL
        ajmp t2_air_off         ;2
        ajmp t2_air_on          ;4

        ajmp t2_spindle_speed   ;6

        ajmp t2_wcell_1         ;8  set workcell output 1
        ajmp t2_wcell_2         ;10
        ajmp t2_wcell_3         ;12
        ajmp t2_wcell_4         ;14
        ajmp t2_wcell_5         ;16
        ajmp t2_wcell_6         ;18
        ajmp t2_wcell_7         ;20
        ajmp t2_wcell_8         ;22
        
        ajmp t2_rcell_1         ;24  reset workcell output 1
        ajmp t2_rcell_2         ;26
        ajmp t2_rcell_3         ;28
        ajmp t2_rcell_4         ;30
        ajmp t2_rcell_5         ;32
        ajmp t2_rcell_6         ;34
        ajmp t2_rcell_7         ;36
        ajmp t2_rcell_8         ;38

        ajmp t2_pcell_1         ;40 pulse workcell output 1.  pulse 10us
        ajmp t2_pcell_2         ;42 high if low and vice versa
        ajmp t2_pcell_3         ;44 pulses @locn.21, rate @locn.23 in shared
        ajmp t2_pcell_4         ;46 
        ajmp t2_pcell_5         ;48
        ajmp t2_pcell_6         ;50
        ajmp t2_pcell_7         ;52
        ajmp t2_pcell_8         ;54
                         
        ajmp t2_send_1          ;56 send serial message to workcell output 1
        ajmp t2_send_2          ;58
        ajmp t2_send_3          ;60 RS232 format @ 9600 baud, st, par, stp
        ajmp t2_send_4          ;62 1-3 bytes, message @locn.17-19, 
        ajmp t2_send_5          ;64 message length @locn.20 in shared memory
        ajmp t2_send_6          ;66
        ajmp t2_send_7          ;68
        ajmp t2_send_8          ;70

        ajmp t2_error           ;72
        ajmp t2_error           ;74
        ajmp t2_error           ;76
        ajmp t2_error           ;78
        ajmp t2_error           ;80
        ajmp t2_error           ;82
        ajmp t2_error           ;84
        ajmp t2_error           ;86
        ajmp t2_error           ;88
        ajmp t2_error           ;90
        ajmp t2_error           ;92
        ajmp t2_error           ;94
        ajmp t2_error           ;96
        ajmp t2_error           ;98
        ajmp t2_error           ;100
        ajmp t2_error           ;102
        ajmp t2_error           ;104
        ajmp t2_error           ;106
        ajmp t2_error           ;108
        ajmp t2_error           ;110
        ajmp t2_error           ;112
        ajmp t2_error           ;114
        ajmp t2_error           ;116
        ajmp t2_error           ;118
        ajmp t2_error           ;120
        ajmp t2_error           ;122
        ajmp t2_error           ;124
        ajmp t2_error           ;126


t2_exit:
        pop dph
        pop dpl
        pop acc
        pop psw
        reti


t2_air_on:
        Setb air

;        mov  dptr,#opto_darl
;        mov  a,opto_stat
;        orl  a,#20h             ; air blast ON
;        mov  opto_stat,a
;        movx @dptr,a
        
        mov  wcntrl,#0
        ajmp t2_exit
        

t2_air_off:
        Clr air
        
;        mov  dptr,#opto_darl
;        mov  a,opto_stat
;        anl  a,#0dfh            ; air blast OFF
;        mov  opto_stat,a
;        movx @dptr,a
        
        mov  wcntrl,#0
        ajmp t2_exit


t2_spindle_speed:
        mov  wcntrl,#0
        mov  a,spindle_speed
        jb   DoorOpen,t2_exit  ;Can't send if door open
        mov  dptr,#spindle
        movx @dptr,a            ; set required spindle motor speed
        ajmp t2_exit



t2_wcell_1:                  ; set workcell output 1
        setb wco_stat0
        ajmp  t2_wcell_do

t2_wcell_2:
        setb wco_stat1
        ajmp  t2_wcell_do

t2_wcell_3:
        setb wco_stat2
        ajmp  t2_wcell_do

t2_wcell_4:
        setb wco_stat3
        ajmp  t2_wcell_do

t2_wcell_5:
        setb wco_stat4
        ajmp  t2_wcell_do

t2_wcell_6: 
        setb wco_stat5
        ajmp  t2_wcell_do

t2_wcell_7:
        setb wco_stat6
        ajmp  t2_wcell_do

t2_wcell_8:
        setb wco_stat7
        ajmp  t2_wcell_do


        


t2_rcell_1:          ; reset workcell output 1
        clr  wco_stat0
        ajmp  t2_wcell_do

t2_rcell_2:
        clr  wco_stat1
        ajmp  t2_wcell_do

t2_rcell_3: 
        clr  wco_stat2
        ajmp  t2_wcell_do

t2_rcell_4:
        clr  wco_stat3
        ajmp  t2_wcell_do

t2_rcell_5:
        clr  wco_stat4
        ajmp  t2_wcell_do

t2_rcell_6:
        clr  wco_stat5
        ajmp  t2_wcell_do

t2_rcell_7:
        clr  wco_stat6
        ajmp  t2_wcell_do

t2_rcell_8: 
        clr  wco_stat7
        ajmp  t2_wcell_do


t2_wcell_do:
        
        mov  dptr,#workcell_o
        mov  a,wco_stat
        movx @dptr,a            ; send modified data byte to workcell outputs
        mov  wcntrl,#0
        ajmp t2_exit



; send serial message to workcell output 1

; RS232 format @ 9600 baud, st, par, stp
; 1-3 bytes, message @locn.17-19, 
; message length @locn.20 in shared memory


t2_send_1:
        clr  wco_stat0              ; set-up for output 1
        mov  wco_low,wco_stat       ; has bit set low
        setb wco_stat0
        mov  wco_high,wco_stat      ; has bit set high
        ajmp  rs232

t2_send_2:  
        clr  wco_stat1              ; set-up for output 2
        mov  wco_low,wco_stat
        setb wco_stat1
        mov  wco_high,wco_stat
        ajmp  rs232

t2_send_3:           
        clr  wco_stat2              ; set-up for output 3
        mov  wco_low,wco_stat
        setb wco_stat2
        mov  wco_high,wco_stat
        ajmp  rs232

t2_send_4:            
        clr  wco_stat3              ; set-up for output 4
        mov  wco_low,wco_stat
        setb wco_stat3
        mov  wco_high,wco_stat
        ajmp  rs232

t2_send_5:          
        clr  wco_stat4              ; set-up for output 5
        mov  wco_low,wco_stat
        setb wco_stat4
        mov  wco_high,wco_stat
        ajmp  rs232

t2_send_6:
        clr  wco_stat5              ; set-up for output 6
        mov  wco_low,wco_stat
        setb wco_stat5
        mov  wco_high,wco_stat
        ajmp  rs232

t2_send_7:  
        clr  wco_stat6              ; set-up for output 7
        mov  wco_low,wco_stat
        setb wco_stat6
        mov  wco_high,wco_stat
        ajmp  rs232

t2_send_8:
        clr  wco_stat7              ; set-up for output 8
        mov  wco_low,wco_stat
        setb wco_stat7
        mov  wco_high,wco_stat
        ajmp  rs232



rs232:
    setb want_all                   ; set request to take over
    jnb  ack_want_all,rs232_1       ; has request been ack'ed? by timer1    

    mov timer2_func,#SEND_RS232     ; Timer 2 function call
    mov rcap2h,#high BAUD_9600   ;Reprogram interval timer for
    mov rcap2l,#low  BAUD_9600   ;9600 Baud

    setb ip.5                    ; promote timer 2 to level 1 interrupt
    clr  pt0                     ; demote timer 0
    clr  pt1                     ; demote timer 1
    
rs232_1:

    ajmp t2_exit



t2_rs232_init:
    inc timer2_func         ;First call is to synch to new clock rate
    inc timer2_func
    ajmp t2_exit



t2_rs232_start:             ;send start bit/ bit low
    mov dptr,#workcell_o
    mov a,wco_low
    movx @dptr,a
    ajmp t2_rs232_0          ; set up bit, in wco_next, for next interrupt




t2_rs232:                   ;send data bits
    mov dptr,#workcell_o
    mov a,wco_next
    movx @dptr,a            ;send bit



t2_rs232_0:  
    mov a,rs232_mess        ;byte being sent
    rrc a                   ;rotate it - bit 0 is now in carry
    mov rs232_mess,a
    jc t2_rs232_1     
    
    mov a,wco_low           ; if bit is low send low bit
    ajmp t2_rs232_2 

t2_rs232_1:    
    mov a,wco_high          ; if bit is high send high bit

t2_rs232_2: 
    
    mov wco_next,a
    inc timer2_func
    inc timer2_func
    ajmp t2_exit

                           

t2_rs232_stop:             ;send bit 7
    mov dptr,#workcell_o    
    mov a,wco_next
    movx @dptr,a
    inc timer2_func
    inc timer2_func
    ajmp t2_exit 


t2_rs232_end:             ;send stop bit/ bit high
    mov dptr,#workcell_o    
    mov a,wco_high
    movx @dptr,a
    
;    mov timer2_func,#SEND_RS232  

    mov timer2_func,#NULL

    clr ip.5                    ; demote timer 2 to level 0 interrupt
    
    setb pt0                     ; promote timer 0
    setb pt1                     ; promote timer 1


    clr want_all                ; back to normal
    clr ack_want_all

    
    mov rcap2h,#high maxdelay   ;Reprogram interval timer for
    mov rcap2l,#low  maxdelay   ;Polling

    mov  wcntrl,#0              ; finished sending RS232 byte 
    
    ajmp t2_exit 


t2_pcell_1:
        jnb  wco_stat0,t2_pcell_1a  ; op currently low send high going pulse
        clr  wco_stat0              ; set-up for output 1
        mov  wco_next,wco_stat       ; has bit set low
        setb wco_stat0
        ajmp  t2_pulse        
t2_pcell_1a:
        setb wco_stat0              ; set-up for output 1
        mov  wco_next,wco_stat       ; has bit set high
        clr  wco_stat0
        ajmp  t2_pulse       


t2_pcell_2:
        jnb  wco_stat1,t2_pcell_2a  ; op currently low send high going pulse
        clr  wco_stat1              ; set-up for output 2
        mov  wco_next,wco_stat       ; has bit set low
        setb wco_stat1
        ajmp  t2_pulse        
t2_pcell_2a:
        setb wco_stat1              ; set-up for output 2
        mov  wco_next,wco_stat       ; has bit set high
        clr  wco_stat1
        ajmp  t2_pulse       



t2_pcell_3:
        jnb  wco_stat2,t2_pcell_3a  ; op currently low send high going pulse
        clr  wco_stat2              ; set-up for output 3
        mov  wco_next,wco_stat       ; has bit set low
        setb wco_stat2
        ajmp  t2_pulse        
t2_pcell_3a:
        setb wco_stat2              ; set-up for output 3
        mov  wco_next,wco_stat       ; has bit set high
        clr  wco_stat2
        ajmp  t2_pulse       



t2_pcell_4:
        jnb  wco_stat3,t2_pcell_4a  ; op currently low send high going pulse
        clr  wco_stat3              ; set-up for output 4
        mov  wco_next,wco_stat       ; has bit set low
        setb wco_stat3
        ajmp  t2_pulse        
t2_pcell_4a:
        setb wco_stat3              ; set-up for output 4
        mov  wco_next,wco_stat       ; has bit set high
        clr  wco_stat3
        ajmp  t2_pulse       



t2_pcell_5:
        jnb  wco_stat4,t2_pcell_5a  ; op currently low send high going pulse
        clr  wco_stat4              ; set-up for output 5
        mov  wco_next,wco_stat       ; has bit set low
        setb wco_stat4
        ajmp  t2_pulse        
t2_pcell_5a:
        setb wco_stat4              ; set-up for output 5
        mov  wco_next,wco_stat       ; has bit set high
        clr  wco_stat4
        ajmp  t2_pulse       
        


t2_pcell_6:
        jnb  wco_stat5,t2_pcell_6a  ; op currently low send high going pulse
        clr  wco_stat5              ; set-up for output 6
        mov  wco_next,wco_stat       ; has bit set low
        setb wco_stat5
        ajmp  t2_pulse        
t2_pcell_6a:
        setb wco_stat5              ; set-up for output 6
        mov  wco_next,wco_stat       ; has bit set high
        clr  wco_stat5
        ajmp  t2_pulse       



t2_pcell_7:
        jnb  wco_stat6,t2_pcell_7a  ; op currently low send high going pulse
        clr  wco_stat6              ; set-up for output 7
        mov  wco_next,wco_stat       ; has bit set low
        setb wco_stat6
        ajmp  t2_pulse        
t2_pcell_7a:
        setb wco_stat6              ; set-up for output 7
        mov  wco_next,wco_stat       ; has bit set high
        clr  wco_stat6
        ajmp  t2_pulse       



t2_pcell_8:
        jnb  wco_stat7,t2_pcell_8a  ; op currently low send high going pulse
        clr  wco_stat7              ; set-up for output 8
        mov  wco_next,wco_stat       ; has bit set low
        setb wco_stat7
        ajmp  t2_pulse        
t2_pcell_8a:
        setb wco_stat7              ; set-up for output 8
        mov  wco_next,wco_stat      ; has bit set high
        clr  wco_stat7
        ajmp  t2_pulse 



t2_pulse:      
        mov a,pulses_no
        cjne a,#1,t2_set_multi_pulse    ;more than 1 pulse to send
        mov a,pulses_no+1
        cjne a,#0,t2_set_multi_pulse

        mov dptr,#workcell_o                ;single pulse to send
        mov a,wco_next
        movx @dptr,a
        
        mov pulse_width,#4
        mov timer2_func,#stretch_pulse  ;will call t2_stretch_pulse next int
        mov wcntrl,#0                   ; ok to process next function
        ajmp t2_exit



t2_stretch_pulse:
        djnz pulse_width,t2_stretch_pulse_1
        ajmp t2_stretch_pulse_1b

t2_stretch_pulse_1:
        mov a,wcntrl                    ; another command waiting?
        jz t2_stretch_pulse_1a          ; no
        mov pulse_width,#0              ; yes - get ready to service it
                                        ; next interrupt

t2_stretch_pulse_1b:
        mov dptr,#workcell_o            ; return o/p to original state 
        mov a,wco_stat         
        movx @dptr,a

        mov timer2_func,#NULL
t2_stretch_pulse_1a:        
        ajmp t2_exit



t2_set_multi_pulse:
        mov timer2_func,#multi_pulse        ;will call t2_multi_pulse next int
    
        mov rcap2l,pulses_rate              ; Reprogram interval timer
        mov rcap2h,pulses_rate+1            ; not rate set by WALLI

        ajmp t2_exit



t2_multi_pulse:        
        mov a,pulses_no
        cjne a,#0,t2_more_1a
        mov a,pulses_no+1
        cjne a,#0,t2_more_1b
        
        mov rcap2l,#low  maxdelay       ; Reprogram interval timer back
        mov rcap2h,#high maxdelay       ; to polling for instructions
        
        mov timer2_func,#NULL           ; sent all the pulses
        mov  wcntrl,#0
        ajmp t2_exit

t2_more_1a:    
        dec pulses_no
        ajmp t2_more_1c    
t2_more_1b:
        dec pulses_no
        dec pulses_no+1
t2_more_1c:
        
        mov dptr,#workcell_o   
        mov a,wco_next         
        movx @dptr,a            ; send step
        
        mov a,wco_stat         
        movx @dptr,a            ; return o/p to original state 
        ajmp t2_exit
        



Shutdown:
        setb stepper_xy_power      ; switch power off
        setb stepper_z_power       ; switch power off
        
;        Clr Vice_1              ; Stop Vice
;        Clr Vice_2
;        Clr Finger_1
;        Clr Finger_2

        mov  a,os_stat
        anl a,#11000011b        ;Vice lines low
        
        mov  dptr ,#step_vice
        movx @dptr,a


        mov  dptr,#opto_darl    
        mov  a,opto_stat
        anl  a,#11011111b       ;Air blaster off
        movx @dptr,a

        
        Clr  a                  ;Stop spindle
        mov  dptr,#spindle
        movx @dptr,a            


        mov a,R4
        jz shut1        ;timer 0 idle 
        cjne a,#14,$+3
        jc abortramp    ;timer 0 ramping

shut1:
        mov a,r5
        jz shut2        ;timer 1 idle
        cjne a,#8,$+3
        jc abortramp    ;timer 1 ramping

shut2:  
        ret

abortramp:
        setb abort
        ret
