include cyber.inc

.code

wait25 macro
	mov cx,2
	.repeat
		timer2wait
	.untilcxz
	endm


wait1000 macro
	mov cx,800
	.repeat
		timer2wait
	.untilcxz
	endm
	
waiteoc macro
comment *
	conversion time max 25usec - according to data sheet
*
	mov si,robot[bx].read
	dlwait
	mov al,es:[si]
	dlwait
	mov al,es:[si]
	and al,1
	
	.if al==0               ;EOC should be high
		call putoffline
	.else
		wait25              ; Wait for ADC to do conversion
		mov al,es:[si]
		dlwait
		mov al,es:[si]
		and al,1
		.if al==1           ;EOC should now be low
			call putoffline
		.else
			call putonline
		.endif
	.endif


;    call putonline
	
	endm


readadc macro channel
	mov di,robot[bx].write
	mov al,080h
	add al,channel
	dlwait
	mov es:[di+14],al
	
	mov al,channel
	dlwait
	mov es:[di+14],al
	
	mov al,080h
	add al,channel
	dlwait
	mov es:[di+14],al
	
	waiteoc
	mov si,robot[bx].read
	dlwait

	mov al,es:[si]
	dlwait
	mov al,es:[si]
			
	dlwait
	mov ah,es:[si+1]
	dlwait
	mov ah,es:[si+1]

	shr ax,4
		
	push ax


	mov al,08fh             ;Set mux to unused channel
	dlwait                  ;reduces interchannel noise?
	mov es:[di+14],al

	pop ax

	
	if channel le 5         ; calibrate feedback axes 
		
		.if ax!=0 
			mov cx,robot[bx].NepCalib.Cal_M[channel*2]
			mul cx
		
			mov cx,1000
			div cx
		
			add ax,robot[bx].NepCalib.Cal_C[channel*2]

			.if ax>32768
				mov ax,0
			.endif

			.if ax>4095
				mov ax,4095
			.endif
		
		.endif
	
	endif


	endm



NoseDec Macro axis
	MOV robot[bx].stop[axis*2],10
	mov ax,robot[bx].feedback[axis*2]
	.if robot[bx].teachmode==Noselead
		sub ax, robot[bx].NepCalib.Cal_IncFast[axis*2]
	.else
		sub ax, robot[bx].NepCalib.Cal_IncSlow[axis*2]
	.endif
	.if sign?
		mov ax,0
	.endif
	mov robot[bx].despos[axis*2],ax
	endm


NoseInc Macro axis
	MOV robot[bx].stop[axis*2],10
	mov ax,robot[bx].feedback[axis*2]
	.if robot[bx].teachmode==Noselead
		add ax,robot[bx].NepCalib.Cal_IncFast[axis*2]
	.else
		add ax,robot[bx].NepCalib.Cal_IncSlow[axis*2]
	.endif
	.if ax>4095
		mov ax,4095
	.endif
	mov robot[bx].despos[axis*2],ax
	endm





service_neptune2_pulses macro
	mov di,robot[bx].read
	add di,2
	add di,robot[bx].pulseop     ;1 to 7
	dlwait
	mov al,es:[di]
	endm


service_neptune2 proc

	mov robot[bx].SimPres,1


	mov ax,es                               ; Check if turbo board exists
	.if ax != 0
		readadc 7                        ; Read Workcell input
		mov robot[bx].WC.ANA_inputs[0],ax   ; checks if neptune2 is online
	.endif
	
	.IF robot[bx].online==DEV_ONLINE
		.if robot[bx].pulsecount == 1
			service_neptune2_pulses
			mov robot[bx].pulsecount,0
		.endif

		.if (robot[bx].teachmode==INIT) || (robot[bx].teachmode==EmerStop)

			MOV robot[bx].stop[0],1
			MOV robot[bx].stop[2],1
			MOV robot[bx].stop[4],1
			MOV robot[bx].stop[6],1
			MOV robot[bx].stop[8],1
			MOV robot[bx].stop[10],1
			MOV robot[bx].stop[12],1

			mov robot[bx].teachmode,KEYBOARD
		.endif

		readadc 0
		.if robot[bx].stop[0] != FALSE
			DEC robot[BX].stop[0]
			mov robot[bx].despos[0],AX 
		.ENDIF
		invoke filter ,0,ax,robot[bx].feedback[0],2
		mov robot[bx].feedback[0],AX    ;Axis 0
					
		readadc 1
		.if robot[bx].stop[2] != FALSE      
			dec robot[BX].stop[2]
			mov robot[bx].despos[2],AX 
		.ENDIF
		invoke filter ,1,ax,robot[bx].feedback[2],2
		mov robot[bx].feedback[2],AX    ;Axis 1
					
		readadc 2
		.if robot[bx].stop[4] != false
			dec robot[BX].stop[4]
			mov robot[bx].despos[4],AX 
		.ENDIF
		invoke filter ,2,ax,robot[bx].feedback[4],2
		mov robot[bx].feedback[4],AX    ;Axis 2
					
		readadc 3
		.if robot[bx].stop[6] != false
			dec robot[BX].stop[6]
			mov robot[bx].despos[6],AX 
		.ENDIF
		invoke filter ,3,ax,robot[bx].feedback[6],2
		mov robot[bx].feedback[6],AX    ;Axis 3
		
		readadc 4
		.if robot[bx].stop[8] != false
			dec robot[BX].stop[8]
			mov robot[bx].despos[8],AX 
		.ENDIF
		invoke filter ,4,ax,robot[bx].feedback[8],2
		mov robot[bx].feedback[8],AX    ;Axis 4
					
		readadc 5
		.if robot[bx].stop[10] != false
			dec robot[BX].stop[10]
			mov robot[bx].despos[10],AX 
		.ENDIF
		invoke filter ,5,ax,robot[bx].feedback[10],2
		mov robot[bx].feedback[10],AX    ;Axis 5
					
		readadc 6
		.if robot[bx].stop[12] != false
			dec robot[BX].stop[12]
			mov robot[bx].despos[12],AX 
		.ENDIF
		invoke filter ,6,ax,robot[bx].feedback[12],2
		mov robot[bx].feedback[12],AX   ;Axis 6,Gripper


		
		.if (robot[bx].teachmode==Noselead) || (robot[bx].teachmode==Noselead2)
			
			readadc 15      ;Switches
			.if ax < 10     ;No switch hit 
			    mov robot[bx].LastRead,0
			.else
				inc  robot[bx].LastRead
				.if robot[bx].LastRead == 5
					.if ax>4000     ;Speed pressed
						.if robot[bx].teachmode==Noselead
							mov robot[bx].teachmode,Noselead2
						.else
							mov robot[bx].teachmode,Noselead
						.endif

					.elseif ax >3000        ;Esc (Both)        
						mov robot[bx].teachmode,KeyBoard

					.elseif ax >1500        ;Store
						or robot[bx].Insert,128

					.endif
				.endif
			.endif
			
			readadc 8
			.if ax<1840
				NoseDec 0                
			.elseif ax>3312
				NoseInc 0
			.endif

			readadc 9
			.if ax<2656
				NoseDec 1       ;,50
			.elseif ax>3840
				NoseInc 1       ;,50
			.endif
			
			readadc 10
			.if ax<2656
				NoseDec 2       ;,75
			.elseif ax>3840
				NoseInc 2       ;,75
			.endif

			readadc 11
			.if ax<2112
				NoseDec 3       ;,150
			.elseif ax>3488
				NoseInc 3       ;,150
			.endif
			
			readadc 12
			.if ax<1360
				NoseDec 4       ;,175
			.elseif ax>2944
				NoseInc 4       ;,175
			.endif

			readadc 13
			.if ax<1168
				NoseDec 5       ;,200
			.elseif ax>2496
				NoseInc 5       ;,200
			.endif

			readadc 14
			invoke filter ,14,ax,robot[bx].learn[12],10
			mov robot[bx].learn[12],ax
			mov robot[bx].despos[12],ax   ;Axis 6


		
		.elseif robot[bx].teachmode==SIMULATOR
			readadc 8
			invoke filter ,8,ax,robot[bx].learn[0],2
			mov robot[bx].learn[0],ax
			mov robot[bx].despos[0],ax  ;Axis 0
							
			readadc 9
			invoke filter ,9,ax,robot[bx].learn[2],2
			mov robot[bx].learn[2],ax
			mov robot[bx].despos[2],ax  ;Axis 1
							
			readadc 10
			invoke filter ,10,ax,robot[bx].learn[4],2
			mov robot[bx].learn[4],ax
			mov robot[bx].despos[4],ax  ;Axis 2

			readadc 11
			invoke filter ,11,ax,robot[bx].learn[6],2
			mov robot[bx].learn[6],ax
			mov robot[bx].despos[6],ax   ;Axis 3
							
			readadc 12
			invoke filter ,12,ax,robot[bx].learn[8],2
			mov robot[bx].learn[8],ax
			mov robot[bx].despos[8],ax   ;Axis 4

			readadc 13
			invoke filter ,13,ax,robot[bx].learn[10],2
			mov robot[bx].learn[10],ax
			mov robot[bx].despos[10],ax   ;Axis 5
			
			readadc 14
			invoke filter ,14,ax,robot[bx].learn[12],2
			mov robot[bx].learn[12],ax
			mov robot[bx].despos[12],ax   ;Axis 6
		.endif


			
			mov robot[bx].inpos,True ;assume there

			mov si,0
			.repeat
				mov ax,robot[bx].despos[si]
				.if ax>robot[bx].Feedback[si]
					sub ax,robot[bx].Feedback[si]
				.else
					mov ax,robot[bx].Feedback[si]
					sub ax,robot[bx].despos[si]
				.endif
				
				
				.if  robot[bx].despos[si]<100
					.if ax>50
						mov robot[bx].inpos,False
						.break
					.endif

				.elseif robot[bx].despos[si] > 4000
					.if ax>50
						mov robot[bx].inpos,False
						.break
					.endif

				.elseif ax > robot[bx].NepCalib.Cal_Win[Si]
					mov robot[bx].inpos,False
					.break
				.endif

				add si,2
			.until si==12

			;.if robot[bx].inpos!=False
			;        mov si,0
			;        .repeat
			;                invoke AxisChanged ,si
			;                .if ax!=0
			;                        mov robot[bx].inpos,False
			;                        .break
			;                .endif
			;                inc si
			;        .until si==6
			;.endif



			

			mov di,robot[bx].write
			
			mov ax,robot[bx].despos[0]
			shl ax,4
			dlwait
			mov es:[di+0],ah
			dlwait
			mov es:[di+1],al

			
			mov ax,robot[bx].despos[2]
			shl ax,4
			dlwait
			mov es:[di+2],ah
			dlwait
			mov es:[di+3],al
			
			mov ax,robot[bx].despos[4]
			shl ax,4
			dlwait
			mov es:[di+4],ah
			dlwait
			mov es:[di+5],al

			mov ax,robot[bx].despos[6]
			shl ax,4
			dlwait
			mov es:[di+6],ah
			dlwait
			mov es:[di+7],al

			mov ax,robot[bx].despos[8]
			shl ax,4
			dlwait
			mov es:[di+8],ah
			dlwait
			mov es:[di+9],al

			
			mov ax,robot[bx].despos[10]
			shl ax,4
			dlwait
			mov es:[di+10],ah
			dlwait
			mov es:[di+11],al

			mov ax,robot[bx].despos[12]
			shl ax,4
			dlwait
			mov es:[di+12],ah
			dlwait
			mov es:[di+13],al


			mov si,robot[bx].read
			dlwait
			mov al,es:[si]
			dlwait
			mov al,es:[si]
			test ax,2
			.if zero?
				mov robot[bx].WC.DIG_inputs[0],0
			.else
				mov robot[bx].WC.DIG_inputs[0],1
			.endif
			test ax,4
			.if zero?
				mov robot[bx].WC.DIG_inputs[2],0
			.else
				mov robot[bx].WC.DIG_inputs[2],1
			.endif
			test ax,8
			.if zero?
				mov robot[bx].WC.DIG_inputs[4],0
			.else
				mov robot[bx].WC.DIG_inputs[4],1
			.endif


		;.endif
	
	.else
		call service_neptune2_offline
	.endif


	
	mov robot.pulsecount[bx],0
	mov robot.rs232flag[bx],false
	mov robot.Serialflag[bx],false


	ret
service_neptune2 endp



service_neptune2_offline proc

	.if robot[bx].teachmode == init
		mov robot[bx].despos[0],2048
		mov robot[bx].despos[2],2048
		mov robot[bx].despos[4],2048
		mov robot[bx].despos[6],2048
		mov robot[bx].despos[8],2048
		mov robot[bx].despos[10],2048
		mov robot[bx].despos[12],2048
		mov si,0
		.repeat
			mov ax,robot[bx].despos[si]
			mov robot[bx].lastdespos[si],ax
			mov robot[bx].feedback[si],ax
			add si,2
		.until si==14
	.endif
	
	mov robot[bx].teachmode,keyboard
	mov robot.pulsecount[bx],0
	mov robot.rs232flag[bx],false
	mov robot.Serialflag[bx],false
	mov robot.inpos[bx],TRUE        

	Simulate 0,32
	Simulate 1,32
	Simulate 2,32
	Simulate 3,32
	Simulate 4,32
	Simulate 5,64
	Simulate 6,128
	ret
service_neptune2_offline endp


end

