include cyber.inc

.code

wait25 macro
	mov cx,2
	.repeat
		timer2wait
	.untilcxz
	endm


wait1000 macro
	mov cx,800
	.repeat
		timer2wait
	.untilcxz
	endm
	
waiteoc macro
comment *
	conversion time max 25usec - according to data sheet
*
	mov si,robot[bx].read
	dlwait
	mov al,es:[si]
	dlwait
	mov al,es:[si]
	and al,1
	
	.if al==0               ;EOC should be high
		call putoffline
	.else
		wait25              ; Wait for ADC to do conversion
		mov al,es:[si]
		dlwait
		mov al,es:[si]
		and al,1
		.if al==1           ;EOC should now be low
			call putoffline
		.else
			call putonline
		.endif
	.endif
	
	endm


readadc macro channel
	mov di,robot[bx].write
	mov al,080h
	or  ax,robot[bx].pneu
	or  al,channel
	dlwait
	mov es:[di+14],al
	
	mov al,channel
	or  ax,robot[bx].pneu
	dlwait
	mov es:[di+14],al
	
	mov al,080h
	or  ax,robot[bx].pneu
	or  al,channel
	dlwait
	mov es:[di+14],al
	
	waiteoc

	mov si,robot[bx].read
	dlwait
	mov al,es:[si]
	dlwait
	mov al,es:[si]
	
	dlwait
	mov ah,es:[si+1]
	dlwait
	mov ah,es:[si+1]

	shr ax,4
	push ax

	mov al,08fh             ;Set mux to unused channel
	or  ax,robot[bx].pneu   ;reduces interchannel noise?
	dlwait
	mov es:[di+14],al

	pop ax


	endm


doke macro addr,dat

	mov di,robot[bx].write
	add di,addr+1

	ifdif <dx>,<dat>
		mov dx,dat
	endif
	sal dx,4                ;Shift up a nibble
	
	poke di,dx              ;Send LSB nibble
	
	dec di
	xchg dh,dl
	poke di,dx              ;Send MSB byte

	endm


datainc equ 32              ;8 gives speed control ?

sendes macro axis
	mov dx,robot[bx].lastdespos[axis*2]
	mov ax,robot[bx].despos[axis*2]            
	.if dx == ax

	.elseif dx < ax                          ;Need to increase data sent
		add dx,datainc
		.if dx > ax
			mov dx,ax
		.endif

	.else                                   ;Need to decrease data sent
		.if dx>datainc
			sub dx,datainc
			.if dx < ax
				mov dx,ax
			.endif
		.else
			mov dx,ax
		.endif

	.endif

	mov robot[bx].lastdespos[axis*2],dx
	doke 2 + (axis*2),dx

	endm




readerror macro axis
	readadc 3+axis                   ;Read error channel    
	invoke filter, 3+axis, ax, robot[bx].rawfeedback[axis*2],2
	mov robot[bx].rawfeedback[axis*2],ax
	endm


calibrate macro axis
	mov ax,robot[bx].feedback[axis*2]
	sub ax,robot[bx].despos[axis*2]
	.if sign?
		mov ax,robot[bx].despos[axis*2]
		sub ax,robot[bx].feedback[axis*2]
	.endif
		
	.if ax<100        ;Almost inposition
		readerror axis
				
		push ax
		push ax
	
		doke 8 + (axis*2) ,0            ;set zero gain
		
		mov dx,robot[bx].despos[axis*2]
		
		.if dx < 2048
			pop robot[bx].SerpCalib.cal0[axis*2]
			
			add dx,2000                 
			doke 2 +(axis*2),dx         ;Set an error of 2000
			wait1000
			readerror axis
			mov robot[bx].SerpCalib.cal2000[axis*2],ax
		
		.else
			pop robot[bx].SerpCalib.cal2000[axis*2]
			
			sub dx,2000
			doke 2 +(axis*2),dx         ;Set an error of 2000
			wait1000
			readerror axis
			mov robot[bx].SerpCalib.cal0[axis*2],ax

		.endif

		mov cx,robot[bx].SerpCalib.cal0[axis*2]
		sub cx,robot[bx].SerpCalib.cal2000[axis*2]
		.if cx!=0
			mov ax,lowword  2000000             ; Error is 2000
			mov dx,highword 2000000             ; Change in error channel is cx
			div cx                          
			
			push ax
			
			mov ax,robot[bx].SerpCalib.cal2.cal_m[axis*2]     ; Shift last values
			mov robot[bx].SerpCalib.cal3.cal_m[axis*2],ax
			
			mov ax,robot[bx].SerpCalib.cal1.cal_m[axis*2]
			mov robot[bx].SerpCalib.cal2.cal_m[axis*2],ax
			
			pop ax
			mov robot[bx].SerpCalib.cal1.cal_m[axis*2],ax
			
			add ax,robot[bx].SerpCalib.cal.cal_m[axis*2]
			add ax,robot[bx].SerpCalib.cal2.cal_m[axis*2]
			add ax,robot[bx].SerpCalib.cal3.cal_m[axis*2]
			shr ax,2
			mov robot[bx].SerpCalib.cal.cal_m[axis*2],ax  ; 25% change

			pop cx
			.if cx!=0
				mul cx
				mov cx,1000
				div cx
				
				push ax
				mov ax,robot[bx].SerpCalib.cal2.cal_c[axis*2]     ; Shift last values
				mov robot[bx].SerpCalib.cal3.cal_c[axis*2],ax
				
				mov ax,robot[bx].SerpCalib.cal1.cal_c[axis*2]
				mov robot[bx].SerpCalib.cal2.cal_c[axis*2],ax
				
				pop ax
				mov robot[bx].SerpCalib.cal1.cal_c[axis*2],ax

				add ax,robot[bx].SerpCalib.cal.cal_c[axis*2]
				add ax,robot[bx].SerpCalib.cal2.cal_c[axis*2]
				add ax,robot[bx].SerpCalib.cal3.cal_c[axis*2]
				shr ax,2
				mov robot[bx].SerpCalib.cal.cal_c[axis*2],ax  ; 25% change
			.endif


		.else
			pop ax
		.endif

		
		mov dx,robot[bx].despos[axis*2]
		doke 2 +(axis*2),dx 
		wait1000
	.endif

	endm

readfeedback macro axis:REQ
	readerror axis
	
	.if ax != 0
		mov cx,ROBOT[BX].SerpCalib.cal.CAL_M[axis*2]     ;mul by 2.019
		mul cx
		mov cx,1000
		div cx
	.endif

	mov dx,ROBOT[BX].SerpCalib.cal.CAL_C[axis*2]        ; 4044  
	sub dx,ax           ;dx = ERROR    
	pushf               ; Save underlow

	mov Ax,robot[bx].lastdespos[axis*2]
	SUB AX,DX
	.if sign?
		mov ax,0        ;Test for feedback underflow
	.endif

	.if ax>4095         ;Test for feedback overflow
		mov ax,4095
	.endif

	popf
	.if sign?
		push ax
		mov ax,0
		sub ax,dx
		mov dx,ax
		pop ax
	.endif

	.if dx>4095         ;Test for error overflow
		mov dx,4095
	.endif

	endm                ;AX=Feedback, DX=ABS(error)

	
seterror macro axis:REQ
	mov ax,robot[bx].feedback[axis*2]
	mov robot[bx].lastfeedback[axis*2],ax

	readfeedback axis                      ;AX=Feedback, DX=ABS(error) 
	mov robot[bx].FEEDBACK[axis*2],ax

	mov si,dx
	add si,si
	mov dx,serpent_ramp[si]
	
	push dx
	;.if robot[bx].teachmode==NOSELEAD
	;    mov dx,0
	;.endif
	doke 8 + (axis*2) ,dx                ;Set gain
	pop dx

	mov ax,robot[bx].despos[axis*2]
	.if ax!=robot[bx].lastdespos[axis*2]
		mov robot[bx].inpos,false 
		mov robot[bx].samefeedbackcnt[axis*2],0
		
		mov robot[bx].SerpCalib.bump[axis*2],0
	.else
		mov ax,robot[bx].lastfeedback[axis*2]
		sub ax,robot[bx].feedback[axis*2]
		.if sign?
			push dx
			mov dx,0
			sub dx,ax
			mov ax,dx
			pop dx
		.endif

		.if ax<5    ;Moved much?
			.if robot[bx].samefeedbackcnt[axis*2] < 100
				inc robot[bx].samefeedbackcnt[axis*2]
			.endif
		.else
			mov robot[bx].samefeedbackcnt[axis*2],0
		.endif
		
		.if robot[bx].SerpCalib.bump[axis*2]==0
		
		.if dx>=2                              ;DX = Abs Error
			mov robot[bx].inpos,false
			.if robot[bx].samefeedbackcnt[axis*2]==100      ;Error > 2 and arm not moving
				mov robot[bx].samefeedbackcnt[axis*2],0
				.if robot[bx].teachmode!=NOSELEAD
					calibrate axis
				.endif
				mov robot[bx].SerpCalib.bump[axis*2],1
			.endif
		.else
			.if robot[bx].samefeedbackcnt[axis*2]>=10

			.else
				mov robot[bx].inpos,false
			.endif
		.endif

		.endif
	.endif 

	mov ax,robot[bx].samefeedbackcnt[axis*2]
	mov robot[bx].feedback[10+(axis*2)],ax

	endm


setdespos macro axis
	doke 8 + (axis*2) ,0                ;Set gain 0

	mov dx,2048
	mov robot[bx].lastdespos[axis*2],dx
	doke 2 + (axis*2) ,dx 
	
	wait1000
	
	readfeedback axis
	mov robot[bx].despos[axis*2],ax
	mov robot[bx].lastdespos[axis*2],ax
	mov robot[bx].feedback[axis*2],ax
			
	endm



service_serpent_pulses macro
	mov di,robot[bx].read
	add di,2
	add di,robot[bx].pulseop     ;1 to 7
	dlwait
	mov al,es:[di]
	endm




service_turbo_serpent proc
	Mov robot[bx].SimPres,1


	mov ax,es                               ; Check if turbo board exists
	.if ax != 0
		readadc 7                           ; Read Workcell input
		mov robot[bx].WC.ANA_inputs[0],ax   ; checks if Serpent is online
											; Note uses old pneu data
	.endif
	
	
	.IF robot[bx].online==DEV_ONLINE
	

		.if robot[bx].pulsecount == 1
			service_serpent_pulses
			mov robot[bx].pulsecount,0
		.endif


		.if robot[bx].teachmode==INIT
			;mov ax,1
			;mov ax,robot[bx].feedback[6]        ;Up
			;mov robot[bx].despos[6],ax           
						
			;mov ax,0
			;mov ax,robot[bx].feedback[8]        ;Gripper off
			;mov robot[bx].despos[8],ax           

			mov ax,robot[bx].pneu
			test ax,64
			.if zero?
				mov robot[bx].despos[6],0
			.else
				mov robot[bx].despos[6],1
			.endif
			test ax,32
			.if zero?
				mov robot[bx].despos[8],0
			.else
				mov robot[bx].despos[8],1
			.endif


			setdespos 0
			setdespos 1
			setdespos 2

			mov robot[bx].teachmode,KEYBOARD
		.else

		mov ax,0                        ; Calculate new pneumatics data
		.if robot[bx].despos[6] == 1    ; is sent as part of ADC read routine    
			or ax,64
		.endif

		.if robot[bx].despos[8] == 1    
			or ax,32
		.endif
		mov robot[bx].pneu,ax



			.if robot[bx].teachmode==SIMULATOR
				readadc 8
				invoke filter, 8, ax, robot[bx].rawlearn[0],2
				mov robot[bx].despos[0],ax
				
				readadc 9
				invoke filter, 9, ax, robot[bx].rawlearn[2],2
				mov robot[bx].despos[2],ax
				
				readadc 10
				invoke filter, 10, ax, robot[bx].rawlearn[4],2
				mov robot[bx].despos[4],ax

				readadc 11
				.if ax>2048
					mov robot[bx].despos[6],1
				.else
					mov robot[bx].despos[6],0
				.endif
				
				readadc 12
				.if ax>2048
					mov robot[bx].despos[8],1
				.else
					mov robot[bx].despos[8],0
				.endif

			.elseif robot[bx].teachmode==NOSELEAD
				;mov si,0
				;.repeat
				;    mov ax,robot[bx].feedback[si]
				;    mov robot[bx].lastdespos[si],ax
				;    mov robot[bx].despos[si],ax
				;    add si,2
				;.until si==6    ;Do for axes 0 1 2

				setdespos 0
				setdespos 1
				setdespos 2


			.endif

			
			mov robot[bx].inpos,True ;assume there
										; following 6 may alter state

			
			.if robot[bx].teachmode!=NOSELEAD
			
			.if robot[bx].stop[0] != false
				setdespos 0
				mov robot[bx].stop[0],false
			.else
				sendes 0                 ;Send theta 1 des position
				seterror 0               ; set error and send gain
			.endif
			
			.if robot[bx].stop[2] != false
				setdespos 1
				mov robot[bx].stop[2],false
			.else
				sendes 1                 ;Send theta 2 des position    
				seterror 1
			.endif

			.if robot[bx].stop[4] != false
				setdespos 2
				mov robot[bx].stop[4],false
			.else
				sendes 2                 ;Send theta 3 des position    
				seterror 2
			.endif
			.endif


			mov ax,robot[bx].despos[6]
			mov robot[bx].feedback[6],ax
			
			mov ax,robot[bx].despos[8]
			mov robot[bx].feedback[8],ax 
			



			mov si,robot[bx].read
			dlwait
			mov al,es:[si]
			dlwait
			mov al,es:[si]
			test ax,2
			.if zero?
				mov robot[bx].WC.DIG_inputs[0],0
			.else
				mov robot[bx].WC.DIG_inputs[0],1
			.endif
			test ax,4
			.if zero?
				mov robot[bx].WC.DIG_inputs[2],0
			.else
				mov robot[bx].WC.DIG_inputs[2],1
			.endif
			test ax,8
			.if zero?
				mov robot[bx].WC.DIG_inputs[4],0
			.else
				mov robot[bx].WC.DIG_inputs[4],1
			.endif


		.endif
	
	.else
		call service_turbo_serpent_offline
	.endif


	
	mov robot.pulsecount[bx],0
	mov robot.rs232flag[bx],false
	mov robot.Serialflag[bx],false


	ret
service_turbo_serpent endp



service_turbo_serpent_offline proc

	mov robot[bx].teachmode,KEYBOARD
	
	
	mov robot.inpos[bx],TRUE        
	mov si,0                        ;not present - fudge all the data
	.repeat
		mov ax,robot.despos[bx+si]
		mov robot.learn[bx+si],ax
		.if robot[bx].snap!=0 
			mov robot.feedback[bx+si],ax
		.else
			.if ax > robot.feedback[bx+si]
				add robot.feedback[bx+si],10
				.if robot.feedback[bx+si]>ax
					mov robot.feedback[bx+si],ax
				.else
					mov robot.inpos[bx],FALSE
				.endif
			.elseif ax < robot.feedback[bx+si]
				.if robot.feedback[bx+si]<10
					mov robot.feedback[bx+si],ax
				.else
					sub robot.feedback[bx+si],10
					.if ax > robot.feedback[bx+si]
						mov robot.feedback[bx+si],ax
					.else
						mov robot.inpos[bx],FALSE
					.endif
			   .endif
			.endif
		.endif
	   mov robot[bx].stop[si],false
		add si,2
	.until si==6

	mov ax,robot[bx].despos[6]
	mov robot[bx].feedback[6],ax
			
	mov ax,robot[bx].despos[8]
	mov robot[bx].feedback[8],ax 
	

	ret
service_turbo_serpent_offline endp


service_turbo_serpent_pulses proc
	service_serpent_pulses
	ret
service_turbo_serpent_pulses endp


end

