;        
	load
	list

	ip_fullstep bit p3.0
	
	op_dir   bit p3.3
	op_inc   bit p3.4
	op_reset bit p3.5
	op_shut  bit p3.6
	op_led   bit p3.7

	clear    bit p1.7
	
	
	
	;base equ 0a000h        ;Use when sending to SRAM
	base equ 0              ;Use when making EPROM        
	
	org base
	pad $ff
	
	org base+00h            ;reset address
	jmp begin               ;start of program
	
	
	org base+0bh            ;timer 0 address
	jmp timer



	org base+23h
	jmp serial

	

;Version 1.0 6/5/94 - First Release
;Version 1.1 20/7/94 - Debounce pulse inputs
;Version 1.2 5/9/94 - Ignore stop input when held low


db "5/9/94 (C) Cybernetic Applications 1994 - (Version 1.2)"
	

counter equ 20h
flags equ counter+1
freerun bit flags.0

halfstep bit flags.1
skipstep bit flags.2
is_lsb bit flags.3
ser_mode bit flags.4
init_serial bit flags.5
ignore_stop bit flags.6         ;Ignore stop when incrementing - skt or serial

rampptr equ flags+1


tmpmsb_posn equ rampptr+1
msb_posn equ tmpmsb_posn+1
lsb_posn equ msb_posn+1

boost equ lsb_posn+1

enable equ boost+1 
step_en bit enable.0
start_en bit enable.1
stop_en bit enable.2


p1copy equ enable+1         
	in0      bit p1copy.0       ;step        
	in1      bit p1copy.1       ;start
	in2      bit p1copy.2       ;stop  
	
	
	in3      bit p1.3       ;rev
	in5      bit p1.5       ;medium speed
	in6      bit p1.6       ;high speed


stack equ p1copy+1

begin:
	mov sp,#stack

	mov r0,#0               
	mov r1,#0
waitpowerup:        
	djnz r0,$
	djnz r1,waitpowerup


	setb op_reset            ;toggle rest line
	clr  op_reset
	setb op_reset

	setb clear              ;Clear 279 latch
	clr  clear
	setb clear
	
	clr  ignore_stop




	setb op_shut
	mov a,#0
	
	setb op_inc
	setb op_dir             ;default to CW



	
	
	mov r0,#0
reallystop:
	acall stop_it
	clr  clear
	setb clear
	djnz r0,reallystop

	
	clr is_lsb

	clr ser_mode             ;default to Non-Serial connection
	clr init_serial

	mov enable,#255        ;"See" all inputs


	


	mov psw,#0                      ;set reg bank 0
	mov pcon,#0                     ;set SMOD low
	mov tmod,#00100010b             ;set timer 1 to mode 2 - timer 0 to mode 1
	mov tcon,#01010000b             ;start timer 1 and 0        
	
	
	mov th1,#253                   ;set countdown for 9600baud
	mov scon,#01010000b             ;set serial mode 1
	
	mov th0,#5
	mov tl0,#0
	mov ie,#10010010b


;Start of main control loop 

toggle:        
	mov p1copy,p1        

;Clear the 279 if
;Had step last time
;Had start last time
	jnb step_en,clear279
	jnb start_en,clear279
	ajmp noclear

clear279:
	clr  clear
	setb clear              ;Clear 279 latch if used LAST time
noclear:
	
	
	jnb in0,notstep         ;Gets set by pulse on !step input
	jnb step_en,usedstep
;*
;*      Step Input Active       
;*
	clr step_en             ;Don't "see" step again
	clr ser_mode
	acall stop_it
	cpl skipstep
	jb ip_fullstep,full_step
	jb skipstep,half_step
full_step:        
	clr op_inc              ;step motor
	clr op_shut
	mov boost,#20               ;keep shut low - high current
half_step:        
	setb op_inc
	jmp toggle


notstep:
	setb step_en 

usedstep:
	jnb in1,notstart
	jnb start_en,usedstart
;*
;*      Start Input Active
;*
	clr start_en            ;Don't "see" start again
	clr ser_mode

	clr  clear              ;Clear 279 latch , this time
	setb clear              ;To clear previous stop
	jb p1.2,usedstart       ;Ignore start if stop is still low
	
	clr ignore_stop         ;Re-enable stop skt
	setb freerun
	jmp toggle


notstart:
	setb start_en
usedstart:
	jb ignore_stop,notstop
	jnb in2,notstop
	jnb stop_en,usedstop  
;*
;*      Stop Input Active
;*
	clr stop_en             ;Don't "see" stop again
	clr ser_mode
	acall stop_it
	jmp toggle

notstop:
	setb stop_en
usedstop:
	jmp toggle




stop_it:
	clr freerun
	mov rampptr,#0
	mov msb_posn,#0
	mov lsb_posn,#0
	ret



serial:
	push acc
	mov a,sbuf
	clr ri
	
	jb is_lsb,mode4
	jb init_serial,readcntrl
	cjne a,#27,exit_serial  ;start with escape?
	
	clr  clear
	setb clear              ;Clear 279 latch


	setb init_serial        ;allow
	jmp exit_serial

readcntrl:
	setb ser_mode           ;Serial connection
	clr init_serial

	cjne a,#255,mode2       ;255==start fwd
	clr ignore_stop         ;Enable stop
	setb freerun            
	setb op_dir
	jmp exit_serial
mode2:        
	cjne a,#254,mode3       ;254==stop
	acall stop_it
	jmp exit_serial
mode3:        
	cjne a,#253,mode5       ;253==start rev
	clr ignore_stop         ;Enable stop
	setb freerun            
	clr op_dir
	jmp exit_serial
mode5:
	jb acc.7,setrev
	setb op_dir             ;increment FWD
	jmp mode6
setrev:
	clr op_dir              ;increment REV
mode6:        
	clr acc.7
	mov tmpmsb_posn,a       ;else first part of increment
	setb is_lsb
	acall stop_it
	jmp exit_serial
mode4:        
	mov lsb_posn,a
	mov a,tmpmsb_posn
	mov msb_posn,a
	clr is_lsb
	setb ignore_stop        ;Increment past Stops
	
exit_serial:        
	pop acc
	reti
	


timer:

	jb ser_mode,serdir
	;reflect state of rquested direction 
	mov c,in3
	mov op_dir,c
serdir:        
	
	djnz counter,exit_timer
	
	mov a,boost             ;should turn off high current?
	jz lowcurrent
	clr op_shut
	dec boost
	jmp tim1
lowcurrent:
	setb op_shut

tim1:
	;Enter next bit if either:-
	;Freerun is set
	;OR msb/lsb posn <> 0
	
	jb freerun,dostep
	mov a,msb_posn
	jnz dostep1
	mov a,lsb_posn
	jnz dostep1
	jmp exit_timer
dostep1:
	dec lsb_posn
	mov a,lsb_posn
	cjne a,#255,dostep      ;Underflown
	dec msb_posn            ;Yep, dec msb  - its not 0
dostep:
	mov a,counter           ;current speed
	
	mov counter,#24         ;default to slow speed
	
	jb in6,notfast
	mov counter,#8          ;high speed requested
	jmp notmed
notfast:        
	jb in5,notmed
	mov counter,#12         ;medium speed
notmed: 
	
	jb ser_mode,fullstep    ;Ignore if serial in control
	jb ip_fullstep,fullstep
	mov a,counter
	rl a                    ;double counter - half speed
	mov counter,a
fullstep:
	clr op_inc   
	setb op_inc             ;steps on rising edge
	mov boost,#20           ;keep high current

exit_timer:        
	reti
	
	end


