SUBTTL Definitions and Constants
TITLE  ** 8032 Kestrel Stepper Monitor - Version 1, 31/1/95 **
PLEN 100
;load
LIST

;Slave 0 encoders X and Z
;Slave 1 encoders Y and R
;Slave 2 Motors X and Z
;Slave 3 Motors Y and R


; Timer funtions

STOP    equ 0
START   equ 2
powerlen equ 02000h


dir0   bit p1.0
step0  bit p1.1
power0 bit p1.2
reset0 bit p1.3

dir1   bit p1.4
step1  bit p1.5
power1 bit p1.6
reset1 bit p1.7



led    bit  p3.5


;               INTERNAL RAM MAP


flags       equ 020h
nextdir0    bit flags.0
nextdir1    bit flags.1
t0serviced  bit flags.2
t1serviced  bit flags.3


ramp0       equ flags+1        ; pointer to axis 0 ramp table
ramp1       equ ramp0+2         ; pointer to axis 1 ramp table

maxramp0    equ ramp1+2        ; max value for ramp0
maxramp1    equ maxramp0+2     ; max value for ramp1


tl0_seed    equ maxramp1+2
th0_seed    equ tl0_seed+1
tl1_seed    equ th0_seed+1
th1_seed    equ tl1_seed+1

retval equ  th1_seed+1

power0_cnt  equ retval+1
power1_cnt  equ power0_cnt+2




stack       equ power1_cnt+2



; r0 timer0 function
; r1 timer1 function


;    base equ 0a000h
	base equ 0

	org base
	pad $ff
	
	org base+00h                ;reset address
		ljmp begin              ;start of program
	
	ORG     base+0bh            ; timer0 interrupt vector
		mov th0,th0_seed        ; reseed timer
		mov tl0,tl0_seed        ; NB. MSB first, else if LSB<2 MSB will not be decremented
		cpl step0  
		reti

	ORG     base+1bh            ; timer1 interrupt vector
		mov th1,th1_seed        ; reseed timer
		mov tl1,tl1_seed        ; NB. MSB first, else if LSB<2 MSB will not be decremented
		cpl step1
		reti
	


db "Kestrel Stepper Slave v1.00"  
db "14/06/95 (C) Cybernetic Applications 1995",0


org base + 0100h

ramptable:             ;ramp table 

nolist
	include ramp.inc
	
	maxstop0  equ ramptable + 200      ; must be >= axis 0 speed 1
	maxstop1  equ ramptable + 200



list

org base + 0900h


page
subttl Initialisation and Baseloop

begin:
	mov sp,#stack
	mov psw,#0
	
	clr a                   ; clear ALL internal RAM 
	mov r0,#255
clearram:   
	mov @r0,a
	djnz r0,clearram
	
	
	movx a,@dptr            ; clear any current external interrupt

	mov tcon,#0h            ; set external interrupt 0 low level trigggered 
	
		
	mov tmod,#00010001b     ; timer0, timer1 both 16 bit timers
	mov tcon,#00000000b     ; ext0 and ext1 level triggered

	
	setb reset0               ; Stepper 0 reset high
	setb reset1               ; Stepper 1 reset high
	
	clr reset0                ; Stepper 0 reset low
	clr reset1                ; Stepper 1 reset low
	
	setb power0               ; Stepper 0 power off
	setb power1               ; Stepper 1 power off
	
	setb reset0               ; Stepper 0 reset high
	setb reset1               ; Stepper 1 reset high
	

	mov c,nextdir0            ; set default direction states  
	mov dir0,c
	
	mov c,nextdir1
	mov dir1,c
	
	
	mov ramp0  ,#low  ramptable
	mov ramp0+1,#high ramptable
	
	mov ramp1  ,#low  ramptable
	mov ramp1+1,#high ramptable
	

	
	mov  ie  ,#10001010b    ; enable timer0,timer1
	

	mov   r5,#040h
	


baseloop:
	lcall letterbox         ; check if master wants to transmit data
	lcall timer0            ; service timer0
	lcall timer1            ; service timer1
	djnz  r4,baseloop
	djnz  r5,baseloop
	mov   r5,#040h
	cpl   led
	jmp   baseloop


page
subttl Letterbox

letterbox:
	jnb p3.2,lb0 
	ret

lb0:
	movx a,@dptr                    ;read masters' control byte

; bits 7-5  define axis and type of data
;0 0 0  Axis 0 error
;0 0 1  Axis 1 error


;1 0 0  Axis 0 scale    Unused in version 1
;1 0 1  Axis 1 scale


; With error info bits 4-1 define speed(size of error) to travel
; bit 0 defines direction to travel

	mov retval,a
	mov c,acc.0                     ; transfer direction into carry flag
	clr acc.0
	
	mov dptr,#ajmp_tbl
	jmp @a+dptr                     ; jump to start+ctrl
ajmp_tbl: 
	ajmp axis0_spd0                 ; 0 0 0  0  0 0 0 x  0   STOPPED
	ajmp setaxis0                   ; 0 0 0  0  0 0 1 x  2   Slowest
	ajmp setaxis0                   ; 0 0 0  0  0 1 0 x  4
	ajmp setaxis0                   ; 0 0 0  0  0 1 1 x  6
	ajmp setaxis0                   ; 0 0 0  0  1 0 0 x  8
	ajmp setaxis0                   ; 0 0 0  0  1 0 1 x  10
	ajmp setaxis0                   ; 0 0 0  0  1 1 0 x  12
	ajmp setaxis0                   ; 0 0 0  0  1 1 1 x  14   
	
	ajmp setaxis0                   ; 0 0 0  1  0 0 0 x  16   
	ajmp setaxis0                   ; 0 0 0  1  0 0 1 x  18   
	ajmp setaxis0                   ; 0 0 0  1  0 1 0 x  20   
	ajmp setaxis0                   ; 0 0 0  1  0 1 1 x  22   
	ajmp setaxis0                   ; 0 0 0  1  1 0 0 x  24   
	ajmp setaxis0                   ; 0 0 0  1  1 0 1 x  26   Fastest
	ajmp axis0_status               ; 0 0 0  1  1 1 0 x  28   Send status
	ajmp axis0_kill                 ; 0 0 0  1  1 1 1 x  30   Emer Stop

	ajmp axis1_spd0                 ; 0 0 1  0  0 0 0 x  32   STOPPED
	ajmp setaxis1                   ; 0 0 1  0  0 0 1 x  34   Slowest
	ajmp setaxis1                   ; 0 0 1  0  0 1 0 x  36
	ajmp setaxis1                   ; 0 0 1  0  0 1 1 x  38
	ajmp setaxis1                   ; 0 0 1  0  1 0 0 x  40
	ajmp setaxis1                   ; 0 0 1  0  1 0 1 x  42
	ajmp setaxis1                   ; 0 0 1  0  1 1 0 x  44
	ajmp setaxis1                   ; 0 0 1  0  1 1 1 x  46   
	
	ajmp setaxis1                   ; 0 0 1  1  0 0 0 x  48   
	ajmp setaxis1                   ; 0 0 1  1  0 0 1 x  50   
	ajmp setaxis1                   ; 0 0 1  1  0 1 0 x  52   
	ajmp setaxis1                   ; 0 0 1  1  0 1 1 x  54   
	ajmp setaxis1                   ; 0 0 1  1  1 0 0 x  56   
	ajmp setaxis1                   ; 0 0 1  1  1 0 1 x  58   Fastest
	ajmp axis1_status               ; 0 0 1  1  1 1 0 x  60   Send status
	ajmp axis1_kill                 ; 0 0 1  1  1 1 1 x  62   Emer Stop



axis0_status:
	mov retval,#0
	jb tr0,status_busy
	ajmp exit_lb
axis1_status:
	mov retval,#0
	jb tr1,status_busy
	ajmp exit_lb

status_busy:
	mov retval,#1
	ajmp exit_lb


setaxis0:
	mov nextdir0,c              ; Set required direction

	push acc
	add  a,#9
	movc a,@a+pc
	mov  maxramp0+1,a           ; 2 bytes

	pop  acc                    ; 2 bytes
	add  a,#3                   ; 2 bytes
	movc a,@a+pc                ; 1 byte
	mov  maxramp0,a             ; 2 bytes
	ajmp setaxis0_2             ; 2 bytes

	dw ramptable             ;axis 0 speed 1       (Speed mult 1.321)
	dw ramptable + 2         ;axis 0 speed 2
	dw ramptable + 4         ;axis 0 speed 3
	dw ramptable + 10        ;axis 0 speed 4
	dw ramptable + 22        ;axis 0 speed 5
	dw ramptable + 38        ;axis 0 speed 6
	dw ramptable + 68        ;axis 0 speed 7
	dw ramptable + 122       ;axis 0 speed 8
	dw ramptable + 216       ;axis 0 speed 9
	dw ramptable + 384       ;axis 0 speed 10
	dw ramptable + 672       ;axis 0 speed 11
	dw ramptable + 1176      ;axis 0 speed 12
	dw ramptable + 2046      ;axis 0 speed 13


setaxis0_2:
	jb  nextdir0,setrev0_0  ; Change of direction?
	jb  dir0,setrev0_1                    
	ajmp setrev0_2
setrev0_0:
	jnb dir0,setrev0_1
	ajmp setrev0_2
setrev0_1:
	jnb  tr0,setrev0_2      ; timer not running reverse is ok
	
	mov  retval,#0          ; Tell master we're stopping
	ajmp axis0_spd0
setrev0_2:
	lcall start_t0

exit_lb:
	mov a,retval
	movx @dptr,a            ;put data onto master data latch
	ret




setaxis1:
	mov nextdir1,c              ; Set required direction
	
	push acc                    ; acc 34 = for speed 1
	add  a,#4
	movc a,@a+pc
	mov  maxramp1+1,a           ; 2 bytes
	
	pop  acc                    ; 2 bytes
	movc a,@a+pc                ; 1 byte
	mov  maxramp1,a             ; 2 bytes
	
	ajmp setaxis1_2             ; 2 bytes
	nop                         ; 1 byte
	
	dw 0,0,0,0,0,0,0,0,0,0      ; 20 bytes
	dw 0,0,0,0                  ; 8  bytes    

	dw ramptable             ;axis 1 speed 1       (Speed mult 1.321)
	dw ramptable + 2         ;axis 1 speed 2
	dw ramptable + 4         ;axis 1 speed 3
	dw ramptable + 10        ;axis 1 speed 4
	dw ramptable + 22        ;axis 1 speed 5
	dw ramptable + 38        ;axis 1 speed 6
	dw ramptable + 68        ;axis 1 speed 7
	dw ramptable + 122       ;axis 1 speed 8
	dw ramptable + 216       ;axis 1 speed 9
	dw ramptable + 384       ;axis 1 speed 10
	dw ramptable + 672       ;axis 1 speed 11
	dw ramptable + 1176      ;axis 1 speed 12
	dw ramptable + 2046      ;axis 1 speed 13

setaxis1_2:
	jb  nextdir1,setrev1_0  ; Change of direction?
	jb  dir1,setrev1_1                    
	ajmp setrev1_2
setrev1_0:
	jnb dir1,setrev1_1
	ajmp setrev1_2
setrev1_1:
	jnb  tr1,setrev1_2      ; timer not running reverse is ok
	mov  retval,#32         ; Tell master we're stopping
	ajmp axis1_spd0
setrev1_2:
	lcall start_t1
	mov a,retval
	movx @dptr,a            ;put data onto master data latch
	ret




axis0_kill:                         ; Stop
	lcall stop_t0
	ajmp  exit_lb

axis1_kill:                         ; Stop
	lcall stop_t1
	ajmp  exit_lb



axis0_spd0:                         ; Go down table and then stop
	mov  r0,#STOP
	mov  maxramp0  ,#low  maxstop0
	mov  maxramp0+1,#high maxstop0
	ajmp exit_lb

axis1_spd0:                         ; Go down table and then stop
	mov r1,#STOP
	mov maxramp1  ,#low  maxstop1
	mov maxramp1+1,#high maxstop1
	ajmp exit_lb




page
subttl Axis 0/1 Control


timer0: 
	jnb step0,t0_a
	clr t0serviced
	jb   tr0,t0_e               ; timer running?
	djnz power0_cnt,t0_e        ; no. think about switching power off
	djnz power0_cnt+1,t0_e      ; no. think about switching power off
	setb power0
t0_e:
	ret


t0_a:
	jb t0serviced,t0_b          ; already serviced?
	setb t0serviced
		
	mov   a,ramp0+1
	cjne  a,maxramp0+1,t0_c     ; set carry if a < maxramp
	mov   a,ramp0
	cjne  a,maxramp0,t0_c
	cjne  r0,#STOP,t0_b
t0_stop:    
	lcall stop_t0
t0_b:                                
	ret                         ; at desired position in ramp
	   

t0_c:                           ; ramp position wrong
	jc    t0_up                 ; go up ramp if c=1
	
	clr   c 
	mov   a,ramp0
	subb  a,#2
	mov   ramp0,a
	mov   a,ramp0+1
	subb  a,#0
	mov   ramp0+1,a
	acall ramp0_to_t0seed       ; read data from table into timer registers
	ret


t0_up:
	cjne  r0,#STOP,t0_upa           
	jmp   t0_stop

t0_upa:
	
	mov   a,ramp0
	add   a,#2
	mov   ramp0,a
	mov   a,ramp0+1
	addc  a,#0
	mov   ramp0+1,a
	acall ramp0_to_t0seed    ; read data from table into timer registers
	ret


timer1: 
	jnb  step1,t1_a
	clr  t1serviced
	jb   tr1,t1_e               ; timer running?
	djnz power1_cnt,t1_e        ; no. think about switching power off
	djnz power1_cnt+1,t1_e        ; no. think about switching power off
	setb power1
t1_e:
	ret


t1_a:
	jb   t1serviced,t1_b          ; already serviced?
	setb t1serviced
		
	mov   a,ramp1+1
	cjne  a,maxramp1+1,t1_c     ; set carry if a < maxramp
	mov   a,ramp1
	cjne  a,maxramp1,t1_c
	cjne  r1,#STOP,t1_b
t1_stop:    
	lcall stop_t1
t1_b:                                
	ret                         ; at desired position in ramp
	   

t1_c:                           ; ramp position wrong
	jc    t1_up                 ; go up ramp if c=1
	
	clr   c 
	mov   a,ramp1
	subb  a,#2
	mov   ramp1,a
	mov   a,ramp1+1
	subb  a,#0
	mov   ramp1+1,a
	acall ramp1_to_t1seed       ; read data from table into timer registers
	ret


t1_up:
	cjne  r1,#STOP,t1_upa           
	jmp   t1_stop

t1_upa:
	mov   a,ramp1
	add   a,#2
	mov   ramp1,a
	mov   a,ramp1+1
	addc  a,#0
	mov   ramp1+1,a
	acall ramp1_to_t1seed    ; read data from table into timer registers
	ret





stop_t0:        
	mov   r0,#STOP
	clr   tr0                   ; stop timer 0
	setb  step0                 ; return step line high
	mov   c,nextdir0            ; set required direction
	mov   dir0,c
	ret
  

start_t0:
	mov   r0,#START
	jb    tr0,started_t0        ; timer already running?

	mov   tl0,#0
	mov   th0,#0
	mov   ramp0,  #low  ramptable
	mov   ramp0+1,#high ramptable
	lcall ramp0_to_t0seed
	
	mov   c,nextdir0            ; set required direction
	mov   dir0,c
	clr   power0                ; turn power on
	clr   t0serviced
	setb  tr0
	mov   power0_cnt,  #low powerlen
	mov   power0_cnt+1,#high powerlen   

started_t0:
	ret

stop_t1:        
	mov   r1,#STOP
	clr   tr1                   ; stop timer 1
	setb  step1                 ; return step line high
	mov   c,nextdir1            ; set required direction
	mov   dir1,c
	ret
  

start_t1:
	mov   r1,#START
	jb    tr1,started_t1
	
	mov   tl1,#0
	mov   th1,#0
	mov   ramp1,  #low  ramptable
	mov   ramp1+1,#high ramptable
	lcall ramp1_to_t1seed
	
	mov   c,nextdir1            ; set required direction
	mov   dir1,c
	clr   power1                ; turn power on
	clr   t1serviced
	setb  tr1
	mov   power1_cnt,#low powerlen
	mov   power1_cnt+1,#high powerlen   

started_t1:
	ret

ramp0_to_t0seed:
	mov dph,ramp0+1
	mov dpl,ramp0
	clr  a
	movc a,@a+dptr          ; read byte from table, msb of timer set value
	mov  r3,a
	inc  dptr
	clr  a
	movc a,@a+dptr          ; read second byte, lsb of timer set value
	
	clr  et0                ; stop timer 0 interrupt
	mov  th0_seed,r3
	mov  tl0_seed,a
	setb et0
	ret

ramp1_to_t1seed:
	mov dph,ramp1+1
	mov dpl,ramp1
	clr  a
	movc a,@a+dptr          ; read byte from table, msb of timer set value
	mov  r3,a
	inc  dptr
	clr  a
	movc a,@a+dptr          ; read second byte, lsb of timer set value
	
	clr  et1                ; stop timer 1 interrupt
	mov  th1_seed,r3
	mov  tl1_seed,a
	setb et1 
	ret



end
